@@ -13,18 +13,24 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <aitools.h>
-#include <map.h>
-#include <game.h>
-#include <unit.h>
-#include <citytools.h>
-#include <advmilitary.h>
-#include <unitfunc.h>
-#include <cityhand.h>
-#include <aicity.h>
-#include <aiunit.h>
-#include <settlers.h>
-#include <unittools.h>
+#include "game.h"
+#include "government.h"
+#include "log.h"
+#include "map.h"
+#include "unit.h"
+
+#include "cityhand.h"
+#include "citytools.h"
+#include "settlers.h"
+#include "unitfunc.h"
+#include "unittools.h"
+
+#include "advmilitary.h"
+#include "aicity.h"
+#include "aitools.h"
+#include "aiunit.h"
+
+#include "advdomestic.h"
 
 /********************************************************************** 
 ... this function should assign a value to choice and want and type, where 
@@ -33,7 +39,7 @@
     type = 1 means unit, type = 0 means building
 ***********************************************************************/
 
-int ai_best_tile_value(struct city *pcity)
+static int ai_best_tile_value(struct city *pcity)
 {
   int x, y, bx, by, food, best, cur;
 
@@ -58,7 +64,7 @@ int ai_best_tile_value(struct city *pcity)
   return 0;
 }
 
-int building_value(int max, struct city *pcity, int val)
+static int building_value(int max, struct city *pcity, int val)
 {
   int i, j = 0;
   int elvis = pcity->ppl_elvis;
@@ -70,23 +76,25 @@ int building_value(int max, struct city *pcity, int val)
 
   i = pcity->ppl_unhappy[3] - pcity->ppl_unhappy[2];
   sad += i; /* if units are making us unhappy, count that too. */
-/*  printf("In %s, unh[0] = %d, unh[4] = %d, sad = %d\n",
-       pcity->name, pcity->ppl_unhappy[0], pcity->ppl_unhappy[4], sad); */
+  freelog(LOG_DEBUG, "In %s, unh[0] = %d, unh[4] = %d, sad = %d",
+	  pcity->name, pcity->ppl_unhappy[0], pcity->ppl_unhappy[4], sad);
 
   i = max;
   while (i && elvis) { i--; elvis--; j += val; }
   while (i && sad) { i--; sad--; j += 16; }
-  if (city_unhappy(pcity)) j += 16 * (sad + bored); /* Desperately seeking Colosseum */
+  if (city_unhappy(pcity))
+    j += 16 * (sad + bored); /* Desperately seeking Colosseum */
   while (i) { i--; j += 16; } /* 16 is debatable value */
-/* using (i && bored) led to a lack of foresight, especially re: Chapel -- Syela */
-/*  printf("%s: %d elvis %d sad %d bored %d size %d max %d val\n",
-    pcity->name, pcity->ppl_elvis, pcity->ppl_unhappy[4],
-    pcity->ppl_content[4], pcity->size, max, j);  */
+  /* using (i && bored) led to a lack of foresight,
+     especially re: Chapel -- Syela */
+  freelog(LOG_DEBUG, "%s: %d elvis %d sad %d bored %d size %d max %d val",
+		pcity->name, pcity->ppl_elvis, pcity->ppl_unhappy[4],
+		pcity->ppl_content[4], pcity->size, max, j);
 
   return(j);
 }
 
-int ocean_workers(struct city *pcity)
+static int ocean_workers(struct city *pcity)
 {
   int x, y, i = 0;
   city_map_iterate(x, y) {
@@ -96,32 +104,33 @@ often everyone was stuck farming grassland. */
       if (is_worker_here(pcity, x, y)) i++;
     }
   }
-  return(i>>1);
+  return(i/2);
 }
 
-int railroad_trade(struct city *pcity)
+static int road_trade(struct city *pcity)
 {
   int x, y, i = 0; 
   city_map_iterate(x, y) {
     if (is_worker_here(pcity, x, y)) {
-      if (map_get_special(pcity->x+x-2, pcity->y+y-2) & S_RAILROAD) i++;
+      if (map_get_special(pcity->x+x-2, pcity->y+y-2) & S_ROAD) i++;
     }
   }  
   return(i); 
 }
 
-int farmland_food(struct city *pcity)
+static int farmland_food(struct city *pcity)
 {
   int x, y, i = 0; 
   city_map_iterate(x, y) {
     if (is_worker_here(pcity, x, y)) {
-      if (map_get_special(pcity->x+x-2, pcity->y+y-2) & S_IRRIGATION) i++;
+      if (map_get_special(pcity->x+x-2, pcity->y+y-2) & S_FARMLAND) i++;
     }
   }  
   return(i); 
 }
 
-int pollution_cost(struct player *pplayer, struct city *pcity, enum improvement_type_id id)
+static int pollution_cost(struct player *pplayer, struct city *pcity,
+			  enum improvement_type_id id)
 {
   int p, mod = 0, poppul = 0, a, b, c, x, y, tmp = 0;
   p = 0;
@@ -147,38 +156,37 @@ int pollution_cost(struct player *pplayer, struct city *pcity, enum improvement_
            id == B_HYDRO || id == B_HOOVER || id == B_NUCLEAR) p /= 2;
 
   if (!city_got_building(pcity, B_MASS) && id != B_MASS) {
-    if (get_invention(pplayer, A_INDUSTRIALIZATION)==TECH_KNOWN)  mod=1;
-    if (get_invention(pplayer, A_AUTOMOBILE)==TECH_KNOWN) mod=2;
-    if (get_invention(pplayer, A_MASS)==TECH_KNOWN) mod=3;
-    if (get_invention(pplayer, A_PLASTICS)==TECH_KNOWN) mod=4;
+    mod = player_knows_techs_with_flag(pplayer, TF_POPULATION_POLLUTION_INC);
     poppul=(pcity->size*mod)/4;
     p += poppul;
   }
   p -= 20;
   if (p < 0) p = 0;
-  b = (POLLUTION_WEIGHTING + pplayer->ai.warmth)<<6;
+  b = (POLLUTION_WEIGHTING + pplayer->ai.warmth)*64;
   if (pcity->pollution > 0) {
     a = amortize(b, 100 / pcity->pollution);
-    c = ((a * b) / (MAX(1, b - a)))>>6;
-/*printf("City: %s, Pollu: %d, cost: %d ", pcity->name, pcity->pollution, c);
-printf("Id: %d, P: %d\n", id, p);*/
+    c = ((a * b) / (MAX(1, b - a)))/64;
+    freelog(LOG_DEBUG, "City: %s, Pollu: %d, cost: %d, Id: %d, P: %d",
+	    pcity->name, pcity->pollution, c, id, p);
   } else c = 0;
   if (p) {
     a = amortize(b, 100 / p);
-    c -= ((a * b) / (MAX(1, b - a)))>>6;
+    c -= ((a * b) / (MAX(1, b - a)))/64;
   } 
   return(c); /* benefit or cost of this building */
 }
  
 void ai_eval_buildings(struct city *pcity)
 {
-  int i, gov, tech, val, a, t, food, j, k, hunger, bar, grana;
+  struct government *g = get_gov_pcity(pcity);
+  int i, val, a, t, food, j, k, hunger, bar, grana;
   int tax, prod, sci, values[B_LAST];
   int est_food = pcity->food_surplus + 2 * pcity->ppl_scientist + 2 * pcity->ppl_taxman; 
   struct player *pplayer = city_owner(pcity);
   int needpower;
+  int wwtv = worst_worker_tile_value(pcity);
   
-  a = get_race(city_owner(pcity))->attack;
+  a = get_nation_by_plr(city_owner(pcity))->attack;
   t = pcity->ai.trade_want; /* trade_weighting */
   sci = (pcity->trade_prod * pplayer->economic.science + 50) / 100;
   tax = pcity->trade_prod - sci;
@@ -187,8 +195,8 @@ void ai_eval_buildings(struct city *pcity)
   tax *= t;
 #else
 /* better might be a longterm weighted average, this is a quick-n-dirty fix -- Syela */
-  sci = ((sci + pcity->trade_prod) * t)>>1;
-  tax = ((tax + pcity->trade_prod) * t)>>1;
+  sci = ((sci + pcity->trade_prod) * t)/2;
+  tax = ((tax + pcity->trade_prod) * t)/2;
 #endif
   if (pplayer->research.researching == A_NONE) sci = 0; /* don't need libraries!! */
   prod = pcity->shield_prod * 100 / city_shield_bonus(pcity) * SHIELD_WEIGHTING;
@@ -205,9 +213,6 @@ void ai_eval_buildings(struct city *pcity)
   if (i > 0 && !pcity->ppl_scientist && !pcity->ppl_taxman) hunger = i + 1;
   else hunger = 1;
 
-  gov = get_government(pcity->owner);
-  tech = (pplayer->research.researching != A_NONE);
-
   for (i=0;i<B_LAST;i++) {
     values[i]=0;
   } /* rewrite by Syela - old values seemed very random */
@@ -216,15 +221,17 @@ void ai_eval_buildings(struct city *pcity)
     int asz = game.aqueduct_size;
     if (city_happy(pcity) && pcity->size > asz-1 && est_food > 0)
       values[B_AQUEDUCT] = ((((city_got_effect(pcity, B_GRANARY) ? 3 : 2) *
-         pcity->size * game.foodbox)>>1) - pcity->food_stock) * food;
-    else values[B_AQUEDUCT] = food * est_food * asz * game.foodbox /
-           MAX(1, ((asz+1 - MIN(asz, pcity->size)) * MAX(asz, pcity->size) *
-           game.foodbox - pcity->food_stock));
+         (pcity->size+1) * game.foodbox)/2) - pcity->food_stock) * food;
+    else {
+      int tmp = ((asz+1 - MIN(asz, pcity->size)) * (MAX(asz, pcity->size)+1) *
+	       game.foodbox - pcity->food_stock);
+      values[B_AQUEDUCT] = food * est_food * (asz+1) * game.foodbox / MAX(1, tmp);
+    }
   }
 
 
   if (could_build_improvement(pcity, B_BANK))
-    values[B_BANK] = tax>>1;
+    values[B_BANK] = (tax + 3*pcity->ppl_taxman + pcity->ppl_elvis*wwtv)/2;
   
   j = 0; k = 0;
   city_list_iterate(pplayer->cities, acity)
@@ -240,7 +247,9 @@ void ai_eval_buildings(struct city *pcity)
              acity->currently_building == B_SUNTZU) j += prod;
     k++;
   city_list_iterate_end;
-  if (!k) printf("Gonna crash, 0 k, looking at %s (ai_eval_buildings)\n", pcity->name);
+  if (!k) freelog(LOG_FATAL,
+		  "Gonna crash, 0 k, looking at %s (ai_eval_buildings)",
+		  pcity->name);
   /* rationale: barracks effectively double prod while building military units */
   /* if half our production is military, effective gain is 1/2 city prod */
   bar = j / k;
@@ -261,7 +270,8 @@ TRADE_WEIGHTING * 100 / MORT.  This is comparable, thus the same weight -- Syela
 
   if (could_build_improvement(pcity, B_CATHEDRAL) && !built_elsewhere(pcity, B_MICHELANGELO))
     values[B_CATHEDRAL] = building_value(get_cathedral_power(pcity), pcity, val);
-  else if (get_invention(pplayer, A_THEOLOGY) != TECH_KNOWN)
+  else if (tech_exists(game.rtech.cathedral_plus) &&
+	   get_invention(pplayer, game.rtech.cathedral_plus) != TECH_KNOWN)
     values[B_CATHEDRAL] = building_value(4, pcity, val) - building_value(3, pcity, val);
 
 /* old wall code depended on danger, was a CPU hog and didn't really work anyway */
@@ -280,51 +290,58 @@ TRADE_WEIGHTING * 100 / MORT.  This is comparable, thus the same weight -- Syela
 
   if (could_build_improvement(pcity, B_COLOSSEUM))
     values[B_COLOSSEUM] = building_value(get_colosseum_power(pcity), pcity, val);
-  else if (get_invention(pplayer, A_ELECTRICITY) != TECH_KNOWN)
+  else if (tech_exists(game.rtech.colosseum_plus) &&
+	   get_invention(pplayer, game.rtech.colosseum_plus) != TECH_KNOWN)
     values[B_COLOSSEUM] = building_value(4, pcity, val) - building_value(3, pcity, val);
   
   if (could_build_improvement(pcity, B_COURTHOUSE)) {
-    values[B_COURTHOUSE] = (pcity->corruption * t)>>1;
-    if (gov == G_DEMOCRACY) values[B_COLOSSEUM] += building_value(1, pcity, val);
+    if (g->corruption_level == 0) {
+      values[B_COURTHOUSE] += building_value (1, pcity, val);
+    } else {
+      values[B_COURTHOUSE] = (pcity->corruption * t)/2;
+    }
   }
   
   if (could_build_improvement(pcity, B_FACTORY))
-    values[B_FACTORY] = (prod>>1) + pollution_cost(pplayer, pcity, B_FACTORY);
+    values[B_FACTORY] = (prod/2) + pollution_cost(pplayer, pcity, B_FACTORY);
   
-  if (could_build_improvement(pcity, B_GRANARY) && !built_elsewhere(pcity, B_PYRAMIDS))
+  if (could_build_improvement(pcity, B_GRANARY) &&
+      !(improvement_variant(B_PYRAMIDS)==0 &&
+	built_elsewhere(pcity, B_PYRAMIDS)))
     values[B_GRANARY] = grana * pcity->food_surplus;
   
   if (could_build_improvement(pcity, B_HARBOUR))
     values[B_HARBOUR] = food * ocean_workers(pcity) * hunger;
 
   if (could_build_improvement(pcity, B_HYDRO) && !built_elsewhere(pcity, B_HOOVER))
-    values[B_HYDRO] = ((needpower * prod)>>2) + pollution_cost(pplayer, pcity, B_HYDRO);
+    values[B_HYDRO] = ((needpower * prod)/4) + pollution_cost(pplayer, pcity, B_HYDRO);
 
   if (could_build_improvement(pcity, B_LIBRARY))
-    values[B_LIBRARY] = sci>>1;
+    values[B_LIBRARY] = sci/2;
 
   if (could_build_improvement(pcity, B_MARKETPLACE))
-    values[B_MARKETPLACE] = tax>>1;
+    values[B_MARKETPLACE] = (tax + 3*pcity->ppl_taxman +
+     pcity->ppl_elvis*wwtv)/2;
 
   if (could_build_improvement(pcity, B_MFG))
     values[B_MFG] = ((city_got_building(pcity, B_HYDRO) ||
                      city_got_building(pcity, B_NUCLEAR) ||
                      city_got_building(pcity, B_POWER) ||
                      city_affected_by_wonder(pcity, B_HOOVER)) ?
-                     (prod * 3)>>2 : prod>>1) +
+                     (prod * 3)/4 : prod/2) +
                      pollution_cost(pplayer, pcity, B_MFG);
 
   if (could_build_improvement(pcity, B_NUCLEAR))
-    values[B_NUCLEAR] = ((needpower * prod)>>2) + pollution_cost(pplayer, pcity, B_NUCLEAR);
+    values[B_NUCLEAR] = ((needpower * prod)/4) + pollution_cost(pplayer, pcity, B_NUCLEAR);
 
   if (could_build_improvement(pcity, B_OFFSHORE)) /* ignoring pollu.  FIX? */
     values[B_OFFSHORE] = ocean_workers(pcity) * SHIELD_WEIGHTING;
 
   if (could_build_improvement(pcity, B_POWER))
-    values[B_POWER] = ((needpower * prod)>>2) + pollution_cost(pplayer, pcity, B_POWER);
+    values[B_POWER] = ((needpower * prod)/4) + pollution_cost(pplayer, pcity, B_POWER);
 
   if (could_build_improvement(pcity, B_RESEARCH) && !built_elsewhere(pcity, B_SETI))
-    values[B_RESEARCH] = sci>>1;
+    values[B_RESEARCH] = sci/2;
 
   if (could_build_improvement(pcity, B_SDI))
     values[B_SDI] = 50; /* WAG */
@@ -333,17 +350,37 @@ TRADE_WEIGHTING * 100 / MORT.  This is comparable, thus the same weight -- Syela
     int ssz = game.sewer_size;
     if (city_happy(pcity) && pcity->size > ssz-1 && est_food > 0)
       values[B_SEWER] = ((((city_got_effect(pcity, B_GRANARY) ? 3 : 2) *
-         pcity->size * game.foodbox)>>1) - pcity->food_stock) * food;
-    else values[B_SEWER] = food * est_food * ssz * game.foodbox /
-          MAX(1, ((ssz+1 - MIN(ssz, pcity->size)) * MAX(ssz, pcity->size) *
-          game.foodbox - pcity->food_stock));
+         (pcity->size+1) * game.foodbox)/2) - pcity->food_stock) * food;
+    else {
+      int tmp = ((ssz+1 - MIN(ssz, pcity->size)) * (MAX(ssz, pcity->size)+1) *
+	       game.foodbox - pcity->food_stock);
+      values[B_SEWER] = food * est_food * (ssz+1) * game.foodbox / MAX(1, tmp);
+    }
+  }
+
+  if (game.spacerace) {
+    if (could_build_improvement(pcity, B_SCOMP)) {
+      if (pplayer->spaceship.components < NUM_SS_COMPONENTS) {
+        values[B_SCOMP] = values[B_CAPITAL]+1;
+      }
+    }
+    if (could_build_improvement(pcity, B_SMODULE)) {
+      if (pplayer->spaceship.modules < NUM_SS_MODULES) {
+        values[B_SMODULE] = values[B_CAPITAL]+1;
+      }
+    }
+    if (could_build_improvement(pcity, B_SSTRUCTURAL)) {
+      if (pplayer->spaceship.structurals < NUM_SS_STRUCTURALS) {
+        values[B_SSTRUCTURAL] = values[B_CAPITAL]+1;
+      }
+    }
   }
 
   if (could_build_improvement(pcity, B_STOCK))
-    values[B_STOCK] = tax>>1;
+    values[B_STOCK] = (tax + 3*pcity->ppl_taxman + pcity->ppl_elvis*wwtv)/2;
 
   if (could_build_improvement(pcity, B_SUPERHIGHWAYS))
-    values[B_SUPERHIGHWAYS] = railroad_trade(pcity) * t;
+    values[B_SUPERHIGHWAYS] = road_trade(pcity) * t;
 
   if (could_build_improvement(pcity, B_SUPERMARKET))
     values[B_SUPERMARKET] = farmland_food(pcity) * food * hunger;
@@ -352,7 +389,7 @@ TRADE_WEIGHTING * 100 / MORT.  This is comparable, thus the same weight -- Syela
     values[B_TEMPLE] = building_value(get_temple_power(pcity), pcity, val);
 
   if (could_build_improvement(pcity, B_UNIVERSITY))
-    values[B_UNIVERSITY] = sci>>1;
+    values[B_UNIVERSITY] = sci/2;
 
   if (could_build_improvement(pcity, B_MASS))
     values[B_MASS] = pollution_cost(pplayer, pcity, B_MASS);
@@ -372,7 +409,7 @@ TRADE_WEIGHTING * 100 / MORT.  This is comparable, thus the same weight -- Syela
       if (i == B_COLLOSSUS)
         values[i] = (pcity->size + 1) * t; /* probably underestimates the value */
       if (i == B_COPERNICUS)
-        values[i] = sci>>1;
+        values[i] = sci/2;
       if (i == B_CURE)
         values[i] = building_value(1, pcity, val);
       if (i == B_DARWIN) /* this is a one-time boost, not constant */
@@ -394,14 +431,17 @@ someone learning Metallurgy, and the AI collapsing.  I hate the WALL. -- Syela *
       if (i == B_HOOVER && !city_got_building(pcity, B_HYDRO) &&
                            !city_got_building(pcity, B_NUCLEAR))
         values[i] = (city_got_building(pcity, B_POWER) ? 0 :
-               (needpower * prod)>>2) + pollution_cost(pplayer, pcity, B_HOOVER);
+               (needpower * prod)/4) + pollution_cost(pplayer, pcity, B_HOOVER);
       if (i == B_ISAAC)
         values[i] = sci;
       if (i == B_LEONARDO) {
         unit_list_iterate(pcity->units_supported, punit)
           j = can_upgrade_unittype(pplayer, punit->type);
-          if (j >= 0) values[i] = MAX(values[i], 8 * unit_upgrade_price(pplayer,
-               punit->type, j)); /* this is probably wrong -- Syela */
+  	  if (j >= 0) {
+	     /* this is probably wrong -- Syela */
+	    int tmp = 8 * unit_upgrade_price(pplayer, punit->type, j);
+	    values[i] = MAX(values[i], tmp);
+	  }
         unit_list_iterate_end;
       }
       if (i == B_BACH)
@@ -410,9 +450,13 @@ someone learning Metallurgy, and the AI collapsing.  I hate the WALL. -- Syela *
         values[i] = (pcity->size + 1) * SHIELD_WEIGHTING;
       if (i == B_MICHELANGELO && !city_got_building(pcity, B_CATHEDRAL))
         values[i] = building_value(get_cathedral_power(pcity), pcity, val);
+      
+      /* The following is probably wrong if B_ORACLE req is
+	 not the same as game.rtech.temple_plus (was A_MYSTICISM)
+	 --dwp */
       if (i == B_ORACLE) {
         if (city_got_building(pcity, B_TEMPLE)) {
-          if (get_invention(pplayer, A_MYSTICISM) == TECH_KNOWN)
+          if (get_invention(pplayer, game.rtech.temple_plus) == TECH_KNOWN)
             values[i] = building_value(2, pcity, val);
           else {
             values[i] = building_value(4, pcity, val) - building_value(1, pcity, val);
@@ -420,17 +464,18 @@ someone learning Metallurgy, and the AI collapsing.  I hate the WALL. -- Syela *
 /* The += has nothing to do with oracle, just the tech_Want of mysticism! */
           }
         } else {
-          if (get_invention(pplayer, A_MYSTICISM) != TECH_KNOWN) {
+          if (get_invention(pplayer, game.rtech.temple_plus) != TECH_KNOWN) {
             values[i] = building_value(2, pcity, val) - building_value(1, pcity, val);
             values[i] += building_value(2, pcity, val) - building_value(1, pcity, val);
           }
         }
       }
           
-      if (i == B_PYRAMIDS && !city_got_building(pcity, B_GRANARY))
+      if (i == B_PYRAMIDS && improvement_variant(B_PYRAMIDS)==0
+	  && !city_got_building(pcity, B_GRANARY))
         values[i] = food * pcity->food_surplus; /* different tech req's */
       if (i == B_SETI && !city_got_building(pcity, B_RESEARCH))
-        values[i] = sci>>1;
+        values[i] = sci/2;
       if (i == B_SHAKESPEARE)
         values[i] = building_value(pcity->size, pcity, val);
       if (i == B_SUNTZU)
@@ -440,13 +485,20 @@ someone learning Metallurgy, and the AI collapsing.  I hate the WALL. -- Syela *
           if (punit->unhappiness) values[i] += t * 2;
         unit_list_iterate_end;
       }
-/* ignoring APOLLO, LIGHTHOUSE, MAGELLAN, MANHATTEN, STATUE, UNITED */
+
+      if ((i == B_APOLLO) && game.spacerace) {
+        values[i] = values[B_CAPITAL]+1;
+      }
+
+      /* ignoring APOLLO, LIGHTHOUSE, MAGELLAN, MANHATTEN, STATUE, UNITED */
     }
   }
 
   for (i=0;i<B_LAST;i++) {
-/*    if (values[i]) printf("%s wants %s with desire %d.\n", pcity->name,
-        get_improvement_name(i), values[i]); */
+    if (values[i]) {
+      freelog(LOG_DEBUG, "%s wants %s with desire %d.",
+	      pcity->name, get_improvement_name(i), values[i]);
+    }
     if (!is_wonder(i)) values[i] -= improvement_upkeep(pcity, i) * t;
     values[i] *= 100;
     if (!is_wonder(i)) { /* trying to buy fewer improvements */
@@ -464,7 +516,8 @@ someone learning Metallurgy, and the AI collapsing.  I hate the WALL. -- Syela *
 void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 				   struct ai_choice *choice)
 {
-  int set, con, i, want;
+  struct government *g = get_gov_pplayer(pplayer);
+  int set, con, i, want, iunit;
   struct ai_choice cur;
   int est_food = pcity->food_surplus + 2 * pcity->ppl_scientist + 2 * pcity->ppl_taxman; 
   int vans = 0;
@@ -474,27 +527,26 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
   choice->want   = 0;
   choice->type   = 0;
   set = city_get_settlers(pcity);
-  con = map_get_continent(pcity->x, pcity->y); 
+  con = map_get_continent(pcity->x, pcity->y);
 
-  if (est_food > (get_government(pcity->owner) >= G_REPUBLIC ? 2 : 1)) {
+  i = best_role_unit(pcity, F_SETTLERS);
+
+  if (est_food > utype_food_cost(get_unit_type(i), g)) {
 /* allowing multiple settlers per city now.  I think this is correct. -- Syela */
 /* settlers are an option */
 /* settler_want calculated in settlers.c, called from ai_manage_city */
     want = pcity->ai.settler_want;
 
     if (want > 0) {
-/*      printf("%s (%d, %d) desires settlers with passion %d\n", pcity->x, pcity->y, pcity->name, want);*/
-      if (can_build_unit(pcity, U_ENGINEERS)) choice->choice = U_ENGINEERS;
-      else {
-        choice->choice = U_SETTLERS;
-        pplayer->ai.tech_want[A_EXPLOSIVES] += want;
-      }
+      freelog(LOG_DEBUG, "%s (%d, %d) desires settlers with passion %d",
+	      pcity->name, pcity->x, pcity->y, want);
       choice->want = want;
       choice->type = 1;
+      ai_choose_role_unit(pplayer, pcity, choice, F_SETTLERS, want);
     } else if (want < 0) { /* need boats to colonize! */
       choice->want = 0 - want;
       choice->type = 1;
-      choice->choice = U_SETTLERS; /* default */
+      choice->choice = best_role_unit(pcity, F_SETTLERS); /* default */
       ai_choose_ferryboat(pplayer, pcity, choice);
     }
   }
@@ -518,20 +570,21 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 /* if we just started building a wonder during a_c_c_b, the started_building */
 /* notify comes equipped with an update.  It calls generate_warmap, but this */
 /* is a lot less warmap generation than there would be otherwise. -- Syela */
-      i = pcity->ai.distance_to_wonder_city * 8 / (can_build_unit(pcity, U_FREIGHT) ? 6 : 3);
+      iunit = best_role_unit(pcity, F_CARAVAN);
+      i = pcity->ai.distance_to_wonder_city * 8 /
+	((iunit==U_LAST) ? 3 : get_unit_type(iunit)->move_rate);
       want -= i;
-/* value of 8 is a total guess and could be wrong, but it's better than 0 -- Syela */
-      if (can_build_unit_direct(pcity, U_CARAVAN)) {
+      /* value of 8 is a total guess and could be wrong,
+       * but it's better than 0 -- Syela */
+      iunit = get_role_unit(F_CARAVAN, 0);
+      if (can_build_unit_direct(pcity, iunit)) {
         if (want > choice->want) {
-          if (can_build_unit(pcity, U_FREIGHT)) choice->choice = U_FREIGHT;
-          else {
-            choice->choice = U_CARAVAN;
-            pplayer->ai.tech_want[A_CORPORATION] += i>>1;
-          }
           choice->want = want;
           choice->type = 1;
+	  ai_choose_role_unit(pplayer, pcity, choice, F_CARAVAN, i/2);
         }
-      } else pplayer->ai.tech_want[A_TRADE] += want;
+      } else
+	pplayer->ai.tech_want[get_unit_type(iunit)->tech_requirement] += want;
     }
   city_list_iterate_end;
 
@@ -546,20 +599,15 @@ void domestic_advisor_choose_build(struct player *pplayer, struct city *pcity,
 /* allowing buy of peaceful units after much testing -- Syela */
 
   if (!choice->want) { /* oh dear, better think of something! */
-    if (can_build_unit(pcity, U_CARAVAN)) {
+    iunit = best_role_unit(pcity, F_CARAVAN);
+    if (iunit == U_LAST) {
+      iunit = best_role_unit(pcity, F_DIPLOMAT);
+      /* someday, real diplomat code will be here! */
+    }
+    if (iunit != U_LAST) {
       choice->want = 1;
       choice->type = 1;
-      choice->choice = U_CARAVAN;
-    }
-    else if (can_build_unit(pcity, U_DIPLOMAT)) {
-      choice->want = 1; /* someday, real diplomat code will be here! */
-      choice->type = 1;
-      choice->choice = U_DIPLOMAT;
-    }
-    else if (can_build_unit(pcity, U_SPY)) {
-      choice->want = 1; /* someday, real diplomat code will be here! */
-      choice->type = 1;
-      choice->choice = U_SPY;
+      choice->choice = iunit;
     }
   }
   if (choice->want >= 200) choice->want = 199; /* otherwise we buy caravans in
