@@ -12,18 +12,24 @@
 ***********************************************************************/
 #include <stdio.h>
 #include <string.h>
-#include <aitools.h>
-#include <game.h>
-#include <map.h>
-#include <unitfunc.h>
-#include <citytools.h>
-#include <aicity.h>
-#include <aiunit.h>
-#include <unittools.h> /* for get_defender, amazingly */
-#include <gotohand.h> /* warmap has been redeployed */
-#include <settlers.h>
-#include <cityturn.h>
-#include <log.h>
+
+#include "game.h"
+#include "government.h"
+#include "log.h"
+#include "map.h"
+
+#include "citytools.h"
+#include "cityturn.h"
+#include "gotohand.h"		/* warmap has been redeployed */
+#include "settlers.h"
+#include "unitfunc.h"
+#include "unittools.h"		/* for get_defender, amazingly */
+
+#include "aicity.h"
+#include "aitools.h"
+#include "aiunit.h"
+
+#include "advmilitary.h"
 
 extern struct move_cost_map warmap;
 
@@ -60,7 +66,7 @@ int assess_defense_quadratic(struct city *pcity)
     if (is_military_unit(punit)) def += v;
   unit_list_iterate_end;
   if (def > 1<<12) {
-    flog(LOG_DEBUG, "Very large def in assess_defense_quadratic: %d in %s",
+    freelog(LOG_VERBOSE, "Very large def in assess_defense_quadratic: %d in %s",
 	 def, pcity->name);
   }
   if (def > 1<<15) def = 1<<15; /* more defense than we know what to do with! */
@@ -82,7 +88,7 @@ int assess_defense_unit(struct city *pcity, struct unit *punit, int igwall)
   return(0);
 }
 
-int assess_defense_backend(struct city *pcity, int igwall)
+static int assess_defense_backend(struct city *pcity, int igwall)
 { /* Most of the time we don't need/want positive feedback. -- Syela */
   int def;
   def = 0;
@@ -100,12 +106,12 @@ int assess_defense(struct city *pcity)
   return(assess_defense_backend(pcity, 0));
 }
 
-int assess_defense_igwall(struct city *pcity)
+static int assess_defense_igwall(struct city *pcity)
 {
   return(assess_defense_backend(pcity, 1));
 }
 
-int dangerfunct(int v, int m, int dist)
+static int dangerfunct(int v, int m, int dist)
 {
 #ifdef OLDCODE
   if (dist * dist < m * 3) { v *= m; v /= 3; } /* knights can't attack more than twice */
@@ -129,27 +135,32 @@ int dangerfunct(int v, int m, int dist)
     denom += (denom + m * 2) / (m * 4);
     dist--;
   }
-  v = (v*num + (denom>>1)) / denom;
+  v = (v*num + (denom/2)) / denom;
   return(v);
 #endif
 }
 
-int assess_danger_unit(struct city *pcity, struct unit *punit)
+static int assess_danger_unit(struct city *pcity, struct unit *punit)
 {
-  int v = get_unit_type(punit->type)->attack_strength * 10;
-  if (punit->veteran) v *= 1.5;
-  if (unit_flag(punit->type, F_SUBMARINE)) v = 0;
-  if (is_sailing_unit(punit) && !is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN)) v = 0;
+  int v, sailing;
+  struct unit_type *ptype;
+
+  if (unit_flag(punit->type, F_SUBMARINE)) return(0);
+  sailing = is_sailing_unit(punit);
+  if (sailing && !is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN)) return(0);
+
+  ptype=get_unit_type(punit->type);
 /* get_attack_power will be wrong if moves_left == 1 || == 2 */
-  v *= punit->hp * get_unit_type(punit->type)->firepower;
-  if (city_got_building(pcity, B_COASTAL) && is_sailing_unit(punit)) v >>= 1;
-  if (city_got_building(pcity, B_SAM) && is_air_unit(punit)) v >>= 1;
+  v = ptype->attack_strength * 10 * punit->hp * ptype->firepower;
+  if (punit->veteran) v += v/2;
+  if (sailing && city_got_building(pcity, B_COASTAL)) v /= 2;
+  if (is_air_unit(punit) && city_got_building(pcity, B_SAM)) v /= 2;
   v /= 30; /* rescaling factor to stop the overflow nonsense */
   return(v);
 }
 
-int assess_distance(struct city *pcity, struct unit *punit, int m,
-                    int boatid, int boatdist, int boatspeed)
+static int assess_distance(struct city *pcity, struct unit *punit, int m,
+			   int boatid, int boatdist, int boatspeed)
 {
   int x, y, dist;
   if (is_tiles_adjacent(punit->x, punit->y, pcity->x, pcity->y)) dist = 3;
@@ -185,6 +196,8 @@ int assess_danger(struct city *pcity)
   int danger5 = 0; /* linear for SDI */
   struct player *aplayer, *pplayer;
   int pikemen = 0;
+  int diplomat = 0; /* > 0 mean that this town can defend
+		     * against diplomats or spies */
   int urgency = 0;
   int igwall;
   int badmojo = 0;
@@ -201,15 +214,18 @@ int assess_danger(struct city *pcity)
   generate_warmap(pcity, 0); /* generates both land and sea maps */
 
   pcity->ai.grave_danger = 0;
+  pcity->ai.diplomat_threat = 0;
 
   unit_list_iterate(map_get_tile(pcity->x, pcity->y)->units, punit)
     if (unit_flag(punit->type, F_PIKEMEN)) pikemen++;
+    if (unit_flag(punit->type, F_DIPLOMAT)) diplomat++;
   unit_list_iterate_end;
 
   for(i=0; i<game.nplayers; i++) {
     if (i != pcity->owner) {
       aplayer = &game.players[i];
-      boatspeed = (get_invention(aplayer, A_NAVIGATION) == TECH_KNOWN ? 12 : 6);
+      boatspeed = (get_invention(aplayer, game.rtech.nav)
+		   == TECH_KNOWN ? 12 : 6);
       boatid = find_boat(aplayer, &x, &y, 0);
       if (boatid) boatdist = warmap.seacost[x][y];
       else boatdist = -1;
@@ -231,14 +247,9 @@ want to write a funct that takes nine ints by reference. -- Syela */
           m = get_unit_type(funit->type)->move_rate;
           v = assess_danger_unit(pcity, funit);
           dist = assess_distance(pcity, funit, m, boatid, boatdist, boatspeed);
-          igwall = 0;
-          if (unit_ignores_citywalls(funit) || 
-              (!is_heli_unit(funit) && !is_ground_unit(funit))) igwall++;
+          igwall = unit_really_ignores_citywalls(funit);
           if ((is_ground_unit(funit) && v) ||
-             (get_transporter_capacity(funit) &&
-              !unit_flag(funit->type, F_SUBMARINE) &&
-              !unit_flag(funit->type, F_CARRIER))) {
-
+              (is_ground_units_transport(funit))) {
             if (dist <= m * 3) urgency++;
             if (dist <= m) pcity->ai.grave_danger++;
 /* NOTE: This should actually implement grave_danger, which is supposed
@@ -251,11 +262,14 @@ content to let it remain that way for now. -- Syela 980805 */
           }
 
           if (unit_flag(funit->type, F_HORSE)) {
-            if (pikemen) v >>= 1;
-            else if (get_invention(pplayer, A_FEUDALISM) != TECH_KNOWN)
-              pplayer->ai.tech_want[A_FEUDALISM] += v * m / (dist<<1);
+            if (pikemen) v /= 2;
+            else ai_wants_role_unit(pplayer, pcity, F_PIKEMEN,
+				    (v * m / (dist*2)));
           }
 
+        if (unit_flag(funit->type, F_DIPLOMAT) && (dist <= 2 * m)) 
+	     pcity->ai.diplomat_threat = !diplomat;
+
           v *= v;
 
           if (!igwall) danger2 += v * m / dist;
@@ -274,14 +288,10 @@ content to let it remain that way for now. -- Syela 980805 */
         m = get_unit_type(punit->type)->move_rate;
         v = assess_danger_unit(pcity, punit);
         dist = assess_distance(pcity, punit, m, boatid, boatdist, boatspeed);
-        igwall = 0;
-        if (unit_ignores_citywalls(punit) || 
-            (!is_heli_unit(punit) && !is_ground_unit(punit))) igwall++;
+	igwall = unit_really_ignores_citywalls(punit);
           
         if ((is_ground_unit(punit) && v) ||
-           (get_transporter_capacity(punit) &&
-            !unit_flag(punit->type, F_SUBMARINE) &&
-            !unit_flag(punit->type, F_CARRIER))) {
+            (is_ground_units_transport(punit))) {
           if (dist <= m * 3) urgency++;
           if (dist <= m) pcity->ai.grave_danger++;
 /* NOTE: This should actually implement grave_danger, which is supposed
@@ -294,11 +304,14 @@ content to let it remain that way for now. -- Syela 980805 */
         }
 
         if (unit_flag(punit->type, F_HORSE)) {
-          if (pikemen) v >>= 1;
-          else if (get_invention(pplayer, A_FEUDALISM) != TECH_KNOWN)
-            pplayer->ai.tech_want[A_FEUDALISM] += v * m / (dist<<1);
+          if (pikemen) v /= 2;
+	  else ai_wants_role_unit(pplayer, pcity, F_PIKEMEN,
+				  (v * m / (dist*2)));
         }
 
+        if (unit_flag(punit->type, F_DIPLOMAT) && (dist <= 2 * m))
+	     pcity->ai.diplomat_threat = !diplomat;
+
         v *= v;
 
         if (!igwall) danger2 += v * m / dist;
@@ -319,16 +332,22 @@ content to let it remain that way for now. -- Syela 980805 */
 /* afraid *100 would create integer overflow, but *10 surely won't */
 
   if (danger < 0 || danger > 1<<24) /* I hope never to see this! */
-    printf("Dangerous danger (%d) in %s.  Beware of overflow.\n", danger, pcity->name);
+    freelog(LOG_NORMAL, "Dangerous danger (%d) in %s.  Beware of overflow.",
+	    danger, pcity->name);
   if (danger2 < 0 || danger2 > 1<<24) /* I hope never to see this! */
-    printf("Dangerous danger2 (%d) in %s.  Beware of overflow.\n", danger2, pcity->name);
+    freelog(LOG_NORMAL, "Dangerous danger2 (%d) in %s.  Beware of overflow.",
+	    danger2, pcity->name);
   if (danger3 < 0 || danger3 > 1<<24) /* I hope never to see this! */
-    printf("Dangerous danger3 (%d) in %s.  Beware of overflow.\n", danger3, pcity->name);
+    freelog(LOG_NORMAL, "Dangerous danger3 (%d) in %s.  Beware of overflow.",
+	    danger3, pcity->name);
   if (danger4 < 0 || danger4 > 1<<24) /* I hope never to see this! */
-    printf("Dangerous danger4 (%d) in %s.  Beware of overflow.\n", danger4, pcity->name);
+    freelog(LOG_NORMAL, "Dangerous danger4 (%d) in %s.  Beware of overflow.",
+	    danger4, pcity->name);
   if (danger5 < 0 || danger5 > 1<<24) /* I hope never to see this! */
-    printf("Dangerous danger5 (%d) in %s.  Beware of overflow.\n", danger5, pcity->name);
-  if (pcity->ai.grave_danger) urgency += 10; /* really, REALLY urgent to defend */
+    freelog(LOG_NORMAL, "Dangerous danger5 (%d) in %s.  Beware of overflow.",
+	    danger5, pcity->name);
+  if (pcity->ai.grave_danger)
+    urgency += 10; /* really, REALLY urgent to defend */
   pcity->ai.danger = danger;
   if (pcity->ai.building_want[B_CITY] > 0 && danger2) {
     i = assess_defense(pcity);
@@ -387,7 +406,7 @@ int unit_desirability(int i, int def)
   d = get_unit_type(i)->defense_strength;
   if (def) cur *= d;
   else if (d > a) return(0);
-/*  else if (d < 2) cur = (cur * (a + d))>>1; Don't believe in this anymore */
+/*  else if (d < 2) cur = (cur * (a + d))/2; Don't believe in this anymore */
   else cur *= a; /* wanted to rank Legion > Catapult > Archer */
 /* which we will do by munging f in the attacker want equations */
   if (unit_flag(i, F_IGTER) && !def) cur *= 3;
@@ -396,8 +415,8 @@ int unit_desirability(int i, int def)
   return(cur);  
 }
 
-void process_defender_want(struct player *pplayer, struct city *pcity, int danger,
-                           struct ai_choice *choice)
+static void process_defender_want(struct player *pplayer, struct city *pcity,
+				  int danger, struct ai_choice *choice)
 {
   int i, j, k, l, m, n;
   int best= 0;
@@ -408,7 +427,8 @@ void process_defender_want(struct player *pplayer, struct city *pcity, int dange
   int isdef = has_a_normal_defender(pcity);
 
   memset(desire, 0, sizeof(desire));
-  for (i = U_WARRIORS; i <= U_BATTLESHIP; i++) {
+  for (i = 0; i < game.num_unit_types; i++) {
+    if (!is_ai_simple_military(i)) continue;
     m = unit_types[i].move_type;
     if ((m == LAND_MOVING || m == SEA_MOVING)) {
       k = pplayer->ai.tech_turns[unit_types[i].tech_requirement];
@@ -441,14 +461,14 @@ void process_defender_want(struct player *pplayer, struct city *pcity, int dange
   } /* was getting four-figure desire for battleships otherwise. -- Syela */
 /* Phalanx would be 16 * danger / 20.  Pikemen would be 36 * danger / (20 + l) */
 /* multiply by unit_types[bestid].build_cost / best */
-  for (i = U_WARRIORS; i <= U_BATTLESHIP; i++) {
-    if (desire[i]) {
+  for (i = 0; i < game.num_unit_types; i++) {
+    if (desire[i] && is_ai_simple_military(i)) {
       j = unit_types[i].tech_requirement;
       n = desire[i] * unit_types[bestid].build_cost / best;
       pplayer->ai.tech_want[j] += n; /* not the totally idiotic
       pplayer->ai.tech_want[j] += n * pplayer->ai.tech_turns[j];  I amaze myself. -- Syela */
-/*      printf("%s wants %s for defense with desire %d <%d>\n",
-pcity->name, advances[j].name, n, desire[i]); */
+      freelog(LOG_DEBUG, "%s wants %s for defense with desire %d <%d>",
+		    pcity->name, advances[j].name, n, desire[i]);
     }
   }
   choice->choice = bestid;
@@ -457,7 +477,8 @@ pcity->name, advances[j].name, n, desire[i]); */
   return;
 }
 
-void process_attacker_want(struct player *pplayer, struct city *pcity, int b, int n,
+static void process_attacker_want(struct player *pplayer,
+			    struct city *pcity, int b, int n,
                             int vet, int x, int y, int unhap, int *e0, int *v,
                             int bx, int by, int boatspeed, int needferry)
 { 
@@ -470,7 +491,8 @@ it some more variables for it to meddle with -- Syela */
   struct city *acity = map_get_city(x, y);
   int movetype = unit_types[*v].move_type;
 
-  for (i = U_WARRIORS; i <= U_BATTLESHIP; i++) {
+  for (i = 0; i < game.num_unit_types; i++) {
+    if (!is_ai_simple_military(i)) continue;
     m = unit_types[i].move_type;
     j = unit_types[i].tech_requirement;
     if (j != A_LAST) k = pplayer->ai.tech_turns[j];
@@ -480,13 +502,13 @@ it some more variables for it to meddle with -- Syela */
          unit_types[i].attack_strength && /* otherwise we get SIGFPE's */
          m == movetype) { /* I don't think I want the duplication otherwise -- Syela */
       l = k * (k + pplayer->research.researchpoints) * game.techlevel;
-      if (game.year > 0) l >>= 1;
-      else l >>= 2; /* cost (shield equiv) of gaining these techs */
+      if (game.year > 0) l /= 2;
+      else l /= 4; /* cost (shield equiv) of gaining these techs */
       l /= city_list_size(&pplayer->cities);
 /* Katvrr advises that with danger high, l should be weighted more heavily */
 
       a = unit_types[i].attack_strength *  ((m == LAND_MOVING ||
-          get_invention(pplayer, A_AMPHIBIOUS) == TECH_KNOWN) ? 15 : 10) *
+          player_knows_improvement_tech(pplayer, B_PORT)) ? 15 : 10) *
              unit_types[i].firepower * unit_types[i].hp;
       a /= 30; /* scaling factor to prevent integer overflows */
       if (acity) a += acity->ai.a;
@@ -495,7 +517,7 @@ it some more variables for it to meddle with -- Syela */
 
       m = unit_types[i].move_rate;
       q = (acity ? 1 : unit_types[n].move_rate * (unit_flag(n, F_IGTER) ? 3 : 1));
-      if (get_unit_type(i)->flags & F_IGTER) m *= 3; /* not quite right */
+      if (unit_flag(i, F_IGTER)) m *= 3; /* not quite right */
       if (unit_types[i].move_type == LAND_MOVING) {
         if (boatspeed) { /* has to be a city, so don't bother with q */
           c = (warmap.cost[bx][by] + m - 1) / m + 1 +
@@ -517,7 +539,8 @@ it some more variables for it to meddle with -- Syela */
       d = m;
 
       if (unit_types[i].move_type == LAND_MOVING && acity &&
-          c > (get_invention(get_player(acity->owner), A_MASONRY) == TECH_KNOWN ? 2 : 4) &&
+          c > (player_knows_improvement_tech(city_owner(acity),
+					     B_CITY) ? 2 : 4) &&
           !unit_flag(i, F_IGWALL) && !city_got_citywalls(acity)) d *= 9; 
 
       f = unit_types[i].build_cost;
@@ -549,11 +572,21 @@ it some more variables for it to meddle with -- Syela */
       if (e > 0) {
         if (k) {
           pplayer->ai.tech_want[j] += e;
-/*if (movetype == SEA_MOVING) printf("%s wants %s to punish %s@(%d, %d) with desire %d.\n", 
-pcity->name, unit_name(i), (acity ? acity->name : "punit"), x, y, e);*/
-        } else if (e > *e0) {
-/*printf("%s overriding %s(%d) with %s(%d) [a=%d,b=%d,c=%d,d=%d,f=%d]\n",
-pcity->name, unit_name(*v), *e0, unit_name(i), e, a, b, c, d, f);*/
+	  if (movetype == SEA_MOVING) {	   /* huh? */
+	    freelog(LOG_DEBUG,
+		    "%s wants %s, %s to punish %s@(%d, %d) with desire %d.", 
+		    pcity->name, advances[j].name, unit_name(i),
+		    (acity ? acity->name : "punit"), x, y, e);
+	  } else {
+	    freelog(LOG_DEBUG,
+		    "%s wants %s, %s for something with desire %d.", 
+		    pcity->name, advances[j].name, unit_name(i), e);
+	  }
+	} else if (e > *e0) {
+	  freelog(LOG_DEBUG, "%s overriding %s(%d) with %s(%d)"
+		  " [a=%d,b=%d,c=%d,d=%d,f=%d]",
+		  pcity->name, unit_name(*v), *e0, unit_name(i), e,
+		  a, b, c, d, f);
           *v = i;
           *e0 = e;
         }
@@ -562,8 +595,8 @@ pcity->name, unit_name(*v), *e0, unit_name(i), e, a, b, c, d, f);*/
   }
 }
 
-void kill_something_with(struct player *pplayer, struct city *pcity, 
-                         struct unit *myunit, struct ai_choice *choice)
+static void kill_something_with(struct player *pplayer, struct city *pcity, 
+				struct unit *myunit, struct ai_choice *choice)
 {
   int a, b, c, d, e, f, g; /* variables in the attacker-want equation */
   int m, n, vet, dist, v, a0, b0, fprime;
@@ -582,7 +615,7 @@ void kill_something_with(struct player *pplayer, struct city *pcity,
 
   ferryboat = unit_list_find(&pplayer->units, boatid);
   if (ferryboat) boatspeed = (unit_flag(ferryboat->type, F_TRIREME) ? 6 : 12);
-  else boatspeed = (get_invention(pplayer, A_NAVIGATION) != TECH_KNOWN ? 6 : 12);
+  else boatspeed = (get_invention(pplayer, game.rtech.nav) != TECH_KNOWN ? 6 : 12);
 
   fstk = find_something_to_kill(pplayer, myunit, &x, &y);
 
@@ -610,7 +643,7 @@ before the 1.7.0 release so I'm letting this stay ugly. -- Syela */
       pdef = get_defender(pplayer, myunit, x, y);
 
       m = unit_types[v].move_rate;
-      if (get_unit_type(v)->flags & F_IGTER) m *= 3; /* not quite right */
+      if (unit_flag(v, F_IGTER)) m *= 3; /* not quite right */
 
       sanity = (goto_is_sane(pplayer, myunit, acity->x, acity->y, 1) &&
               warmap.cost[x][y] <= (MIN(6, m) * THRESHOLD));
@@ -621,8 +654,8 @@ before the 1.7.0 release so I'm letting this stay ugly. -- Syela */
                 warmap.seacost[acity->x][acity->y] / boatspeed; /* kluge */
           else c = warmap.seacost[acity->x][acity->y] / boatspeed + 1;
           if (unit_flag(myunit->type, F_MARINES)) c -= 1;
-/*printf("%s attempting to attack via ferryboat (boatid = %d, c = %d)\n",
-unit_types[v].name, boatid, c);*/
+	  freelog(LOG_DEBUG, "%s attempting to attack via ferryboat"
+			" (boatid = %d, c = %d)", unit_types[v].name, boatid, c);
         } else c = (warmap.cost[acity->x][acity->y] + m - 1) / m;
       } else if (is_sailing_unit(myunit))
         c = (warmap.seacost[acity->x][acity->y] + m - 1) / m;
@@ -662,9 +695,10 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
       }
       if (!is_ground_unit(myunit) && !is_heli_unit(myunit) &&
          (!(acity->ai.invasion&1))) b -= 40; /* boats can't conquer cities */
-      if (!myunit->id && (is_ground_unit(myunit) || is_heli_unit(myunit)) &&
-          c > (get_invention(get_player(acity->owner), A_MASONRY) == TECH_KNOWN ? 2 : 4) &&
-          !unit_flag(myunit->type, F_IGWALL) && !city_got_citywalls(acity)) d *= 9;
+      if (!myunit->id && (!unit_really_ignores_citywalls(myunit)) &&
+          c > (player_knows_improvement_tech(city_owner(acity),
+					     B_CITY) ? 2 : 4) &&
+          !city_got_citywalls(acity)) d *= 9;
     } /* end dealing with cities */
 
     else {
@@ -685,7 +719,7 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
       if (!dist) dist = 1;
 
       m = unit_types[v].move_rate;
-      if (get_unit_type(v)->flags & F_IGTER) m *= 3; /* not quite right */
+      if (unit_flag(v, F_IGTER)) m *= 3; /* not quite right */
       c = ((dist + m - 1) / m);
 
       n = pdef->type;
@@ -703,17 +737,8 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
       vet = pdef->veteran;
     } /* end dealing with units */
 
-    if (get_government(pplayer->player_no) == G_REPUBLIC &&
-        (pcity->id == myunit->homecity || !myunit->id)) {
-      unit_list_iterate(pcity->units_supported, punit)
-        if (unit_being_aggressive(punit)) {
-          unhap++;
-        } else if (is_field_unit(punit)) {
-          unhap++;
-        }
-      unit_list_iterate_end;
-      if (city_got_effect(pcity, B_POLICE)) unhap--;
-    } /* handle other governments later */
+    if (pcity->id == myunit->homecity || !myunit->id)
+      unhap = ai_assess_military_unhappiness(pcity, get_gov_pplayer(pplayer));
 
     if (is_ground_unit(myunit) && !sanity && !boatid)
       needferry = 40; /* cost of ferry */
@@ -743,15 +768,21 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
       e = ((a0 * b0) / (MAX(1, b0 - a0))) * 100 / ((fprime + needferry) * MORT);
     } else e = 0;  
 
-/*if (e != fstk && acity) {
-printf("%s (%d, %d), %s#%d -> %s[%d] (%d, %d) [A=%d, B=%d, C=%d, D=%d, E=%d/%d, F=%d]\n",
-pcity->name, pcity->x, pcity->y, unit_types[v].name, myunit->id,
-acity->name, acity->ai.invasion, acity->x, acity->y, a, b, c, d, e, fstk, f);
-} else if (e != fstk && aunit) {
-printf("%s (%d, %d), %s#%d -> %s (%d, %d) [A=%d, B=%d, C=%d, D=%d, E=%d/%d, F=%d]\n", 
-pcity->name, pcity->x, pcity->y, unit_types[v].name, myunit->id,
-unit_types[pdef->type].name, pdef->x, pdef->y, a, b, c, d, e, fstk, f);
-}*/
+#ifdef DEBUG
+    if (e != fstk && acity) {
+	freelog(LOG_DEBUG, "%s (%d, %d), %s#%d -> %s[%d] (%d, %d)"
+		" [A=%d, B=%d, C=%d, D=%d, E=%d/%d, F=%d]",
+		pcity->name, pcity->x, pcity->y, unit_types[v].name,
+		myunit->id, acity->name, acity->ai.invasion,
+		acity->x, acity->y, a, b, c, d, e, fstk, f);
+    } else if (e != fstk && aunit) {
+	freelog(LOG_DEBUG, "%s (%d, %d), %s#%d -> %s (%d, %d)"
+		" [A=%d, B=%d, C=%d, D=%d, E=%d/%d, F=%d]", 
+		pcity->name, pcity->x, pcity->y, unit_types[v].name,
+		myunit->id, unit_types[pdef->type].name,
+		pdef->x, pdef->y, a, b, c, d, e, fstk, f);
+    }
+#endif
 
     if (!myunit->id) {  /* I don't want to know what happens otherwise -- Syela */
       if (sanity)
@@ -766,9 +797,9 @@ unit_types[pdef->type].name, pdef->x, pdef->y, a, b, c, d, e, fstk, f);
     if (e > choice->want && /* Without this &&, the AI will try to make attackers */
         choice->want <= 100) { /* instead of defenders when being attacked -- Syela */
       if (!city_got_barracks(pcity) && is_ground_unit(myunit)) {
-        if (get_invention(pplayer, A_COMBUSTION) == TECH_KNOWN)
+        if (player_knows_improvement_tech(pplayer, B_BARRACKS3))
           choice->choice = B_BARRACKS3;
-        else if (get_invention(pplayer, A_GUNPOWDER) == TECH_KNOWN)
+        else if (player_knows_improvement_tech(pplayer, B_BARRACKS2))
           choice->choice = B_BARRACKS2;
         else choice->choice = B_BARRACKS;
         choice->want = e;
@@ -779,26 +810,30 @@ unit_types[pdef->type].name, pdef->x, pdef->y, a, b, c, d, e, fstk, f);
           choice->type = 2; /* type == 2 identifies attackers */
           choice->want = e;
           if (needferry) ai_choose_ferryboat(pplayer, pcity, choice);
-/*printf("%s has chosen attacker, %s\n", pcity->name, unit_types[choice->choice].name);*/
+	  freelog(LOG_DEBUG, "%s has chosen attacker, %s",
+			pcity->name, unit_types[choice->choice].name);
         } else {
           choice->choice = ai_choose_defender(pcity);
-/*printf("%s has chosen defender, %s\n", pcity->name, unit_types[choice->choice].name);*/
+	  freelog(LOG_DEBUG, "%s has chosen defender, %s",
+			pcity->name, unit_types[choice->choice].name);
           choice->type = 1;
           choice->want = e;
         }
-        if (is_sailing_unit(myunit) && !city_got_building(pcity, B_PORT)) {
-          if (get_invention(pplayer, A_AMPHIBIOUS) == TECH_KNOWN) {
+        if (is_sailing_unit(myunit) && improvement_exists(B_PORT)
+	    && !city_got_building(pcity, B_PORT)) {
+	  int tech = get_improvement_type(B_PORT)->tech_requirement;
+          if (get_invention(pplayer, tech) == TECH_KNOWN) {
             choice->choice = B_PORT;
             choice->want = e;
             choice->type = 0;
-          } else pplayer->ai.tech_want[A_AMPHIBIOUS] += e;
+          } else pplayer->ai.tech_want[tech] += e;
         }
       }
     }
   } 
 }
 
-int port_is_within(struct player *pplayer, int d)
+static int port_is_within(struct player *pplayer, int d)
 {
   city_list_iterate(pplayer->cities, pcity)
     if (city_got_building(pcity, B_PORT) &&
@@ -806,7 +841,7 @@ int port_is_within(struct player *pplayer, int d)
     if (!pcity->is_building_unit && pcity->currently_building == B_PORT &&
         pcity->shield_stock >= improvement_value(B_PORT) &&
         warmap.seacost[pcity->x][pcity->y] <= d) return 1;
-    if (get_invention(pplayer, A_AMPHIBIOUS) != TECH_KNOWN &&
+    if (!player_knows_improvement_tech(pplayer, B_PORT) &&
         pcity->is_building_unit &&
         is_water_unit(pcity->currently_building) &&
         unit_types[pcity->currently_building].attack_strength >
@@ -841,7 +876,34 @@ void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
   def = assess_defense_quadratic(pcity); /* has to be AFTER assess_danger thanks to wallvalue */
 /* changing to quadratic to stop AI from building piles of small units -- Syela */
   danger = pcity->ai.danger; /* we now have our warmap and will use it! */
-/*printf("Assessed danger for %s = %d, Def = %d\n", pcity->name, danger, def);*/
+  freelog(LOG_DEBUG, "Assessed danger for %s = %d, Def = %d",
+	  pcity->name, danger, def);
+
+  if (pcity->ai.diplomat_threat) {
+  /* if I put a && (danger < def), if def = 0 and there is a little enemy unit
+     near, ai will ignore the diplomat threat.
+     On the other hand, if there is a joined attach of diplomats and
+     classic military units , ai will build a diplomat as last defender.
+     With the actual functions asses_danger and assess_danger_unit,
+     I can't know in which case I am.
+     The confusion is between the actual threat and the virtual threat. 
+     How many units may I loose at this turn and
+     how many units may I loose during next turns? (17/12/98) (--NB)
+  */
+
+    int u = best_role_unit(pcity, F_DIPLOMAT);
+    if (u<U_LAST) {
+       freelog(LOG_DEBUG, "A diplomat will be built in city %s.", pcity->name);
+       choice->want = 16000; /* diplomat more important than soldiers */ 
+       pcity->ai.urgency = 1;
+       choice->type = 3; /* defender */
+       choice->choice = u;
+       return;
+    } else if (num_role_units(F_DIPLOMAT)>0) {
+      u = get_role_unit(F_DIPLOMAT, 0);
+      pplayer->ai.tech_want[get_unit_type(u)->tech_requirement] += 16000;
+    }
+  } 
 
   if (danger) { /* otherwise might be able to wait a little longer to defend */
 /* old version had danger -= def in it, which was necessary before disband/upgrade
@@ -890,9 +952,10 @@ creating any other problems that are worse. -- Syela */
         if (city_got_barracks(pcity)) choice->want = MIN(49, danger); /* unlikely */
         else choice->want = MIN(25, danger);
       } else choice->want = danger;
-/*    printf("%s wants %s to defend with desire %d.\n", pcity->name,
-       get_unit_type(choice->choice)->name, choice->want); */
-/*    return; - this is just stupid */
+      freelog(LOG_DEBUG, "%s wants %s to defend with desire %d.",
+		    pcity->name, get_unit_type(choice->choice)->name,
+		    choice->want);
+      /* return; - this is just stupid */
     }
   } /* ok, don't need to defend */
 
@@ -931,14 +994,14 @@ the intrepid David Pfitzner discovered was in error. -- Syela */
 /* and we will use its attack values, otherwise we will use virtualunit */
   if (myunit) kill_something_with(pplayer, pcity, myunit, choice);
   else {
-/*printf("Killing with virtual unit in %s\n", pcity->name);*/
+    freelog(LOG_DEBUG, "Killing with virtual unit in %s", pcity->name);
     v = ai_choose_attacker_sailing(pcity);
     if (v > 0 && /* have to put sailing first before we mung the seamap */
       (city_got_building(pcity, B_PORT) || /* only need a few ports */
       !port_is_within(pplayer, 18))) { /* using move_rate is quirky -- Syela */
       virtualunit.type = v;
 /*     virtualunit.veteran = do_make_unit_veteran(pcity, v); */
-      virtualunit.veteran = (get_invention(pplayer, A_AMPHIBIOUS) == TECH_KNOWN);
+      virtualunit.veteran = (player_knows_improvement_tech(pplayer, B_PORT));
       virtualunit.hp = unit_types[v].hp;
       kill_something_with(pplayer, pcity, &virtualunit, choice);
     } /* ok.  can now mung seamap for ferryboat code.  Proceed! */
@@ -963,9 +1026,8 @@ void establish_city_distances(struct player *pplayer, struct city *pcity)
   if (!pcity->is_building_unit && is_wonder(pcity->currently_building))
     wondercity = map_get_continent(pcity->x, pcity->y);
   else wondercity = 0;
-  if (get_invention(pplayer, A_CORPORATION) == TECH_KNOWN)
-    freight = 6;
-  else freight = 3;
+  freight = best_role_unit(pcity, F_CARAVAN);
+  freight = (freight==U_LAST) ? 3 : get_unit_type(freight)->move_rate;
 
   pcity->ai.downtown = 0;
   city_list_iterate(pplayer->cities, othercity)
