@@ -10,46 +10,50 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <player.h>
-#include <city.h>
-#include <game.h>
-#include <unit.h>
-#include <shared.h>
-#include <packets.h>
-#include <events.h>
-#include <map.h>
-#include <mapgen.h>
-#include <plrhand.h>
-#include <cityhand.h>
-#include <citytools.h>
-#include <cityturn.h>
-#include <unithand.h>
-#include <unitfunc.h>
-#include <aitools.h>
-#include <aihand.h>
-#include <aiunit.h>
-#include <aicity.h>
-#include <advisland.h>
-#include <advleader.h>
-#include <advmilitary.h>
-#include <advdomestic.h>
-#include <gotohand.h>
-#include <settlers.h>
-#include <unittools.h>
-#include <log.h>
 
-int ai_fix_unhappy(struct city *pcity);
-void ai_manage_city(struct player *pplayer, struct city *);
-void ai_scientists_taxmen(struct city *pcity);
+#include "city.h"
+#include "events.h"
+#include "fcintl.h"
+#include "game.h"
+#include "government.h"
+#include "log.h"
+#include "map.h"
+#include "packets.h"
+#include "player.h"
+#include "shared.h"
+#include "unit.h"
 
+#include "cityhand.h"
+#include "citytools.h"
+#include "cityturn.h"
+#include "gotohand.h"
+#include "plrhand.h"
+#include "settlers.h"
+#include "unitfunc.h"
+#include "unithand.h"
+#include "unittools.h"
+
+#include "advdomestic.h"
+#include "advisland.h"
+#include "advleader.h"
+#include "advmilitary.h"
+#include "aihand.h"
+#include "aitools.h"
+#include "aiunit.h"
+
+#include "aicity.h"
+
+#ifdef UNUSED
 /************************************************************************** 
 ...
 **************************************************************************/
-
 void ai_do_build_unit(struct city *pcity, int unit_type)
 {
   pcity->is_building_unit = 1;
@@ -64,34 +68,23 @@ void ai_city_build_defense(struct city *pcity)
 /************************************************************************** 
 ...
 **************************************************************************/
-
 void ai_city_build_settler(struct city *pcity)
 {
-  int i;
-  for (i = 0 ; i < U_LAST ; i ++) {
-    if (unit_flag(i, F_SETTLERS) && can_build_unit(pcity, i)) {
-      ai_do_build_unit(pcity, i);
-      break;
-    }
-  }
+  ai_do_build_unit(pcity, best_role_unit(pcity, F_SETTLERS));
 }
 
 /************************************************************************** 
 ...
 **************************************************************************/
-
 int ai_city_build_peaceful_unit(struct city *pcity)
 {
   int i;
   if (is_building_other_wonder(pcity)) {
-    for (i = 0 ; i < U_LAST ; i ++) {
-      if (unit_flag(i, F_CARAVAN) && can_build_unit(pcity, i)) {
-	ai_do_build_unit(pcity, i);
-	break;
-      }
-    }
-    if (i < U_LAST) 
+    i = best_role_unit(pcity, F_CARAVAN);
+    if (i < U_LAST) {
+      ai_do_build_unit(pcity, i);
       return 1;
+    }
   } else {
     if (can_build_improvement(pcity, B_CAPITAL)) {
       pcity->currently_building=B_CAPITAL;
@@ -101,14 +94,16 @@ int ai_city_build_peaceful_unit(struct city *pcity)
   }
   return 0;
 }
+#endif /* UNUSED */
 
-void ai_manage_buildings(struct player *pplayer)
+static void ai_manage_buildings(struct player *pplayer)
 { /* we have just managed all our cities but not chosen build for them yet */
+  struct government *g = get_gov_pplayer(pplayer);
   int i, j, values[B_LAST], leon = 0, palace = 0, corr = 0;
   memset(values, 0, sizeof(values));
   memset(pplayer->ai.tech_want, 0, sizeof(pplayer->ai.tech_want));
 
-  if (find_palace(pplayer) || pplayer->government == G_DEMOCRACY) palace = 1;
+  if (find_palace(pplayer) || g->corruption_level == 0) palace = 1;
   city_list_iterate(pplayer->cities, pcity)
     ai_eval_buildings(pcity);
     if (!palace) corr += pcity->corruption * 8;
@@ -127,8 +122,12 @@ void ai_manage_buildings(struct player *pplayer)
   values[B_MAGELLAN] *= 100 * SHIELD_WEIGHTING;
   values[B_MAGELLAN] /= (MORT * improvement_value(B_MAGELLAN));
 
-/* this is a weird place to put tech advice */
-  if (pplayer->government > G_DESPOTISM) {
+  /* This is a weird place to put tech advice */
+  /* This was: > G_DESPOTISM; should maybe remove test, depending
+   * on new government evaluation etc, but used for now for
+   * regression testing --dwp */
+  if (g->index != game.default_government
+      && g->index != game.government_when_anarchy) {
     for (i = 0; i < B_LAST; i++) {
       j = improvement_types[i].tech_requirement;
       if (get_invention(pplayer, j) != TECH_KNOWN)
@@ -136,8 +135,10 @@ void ai_manage_buildings(struct player *pplayer)
     }
   } /* tired of researching pottery when we need to learn Republic!! -- Syela */
 
-  if (!game.global_advances[A_PHILOSOPHY])
-    pplayer->ai.tech_want[A_PHILOSOPHY] *= 2; /* this probably isn't right -- Syela */
+  /* was A_PHILOSOPHY: */
+  if (!game.global_advances[game.rtech.get_bonus_tech])
+    pplayer->ai.tech_want[game.rtech.get_bonus_tech] *= 2;
+      /* this probably isn't right -- Syela */
 
   city_list_iterate(pplayer->cities, pcity)
     pcity->ai.building_want[B_MAGELLAN] = values[B_MAGELLAN];
@@ -171,7 +172,7 @@ void ai_manage_buildings(struct player *pplayer)
 ... change the build order.
 **************************************************************************/
 
-void ai_city_choose_build(struct player *pplayer, struct city *pcity)
+static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
 {
   struct ai_choice bestchoice, curchoice;
 
@@ -197,14 +198,15 @@ void ai_city_choose_build(struct player *pplayer, struct city *pcity)
 /* type 0 = building, 1 = non-mil unit, 2 = attacker, 3 = defender */
 
   if (bestchoice.want) { /* Note - on fallbacks, will NOT get stopped building msg */
-/* printf("%s wants %s with desire %d.\n", pcity->name, (bestchoice.type ?
-unit_name(bestchoice.choice) : get_improvement_name(bestchoice.choice)),
-bestchoice.want); */
+    freelog(LOG_DEBUG, "%s wants %s with desire %d.", pcity->name,
+		  (bestchoice.type ? unit_name(bestchoice.choice)
+		   : get_improvement_name(bestchoice.choice)),
+		  bestchoice.want);
     if(!pcity->is_building_unit && is_wonder(pcity->currently_building) &&
       (bestchoice.type || bestchoice.choice != pcity->currently_building))
       notify_player_ex(0, pcity->x, pcity->y, E_WONDER_STOPPED,
-                   "Game: The %s have stopped building The %s in %s.",
-                   get_race_name_plural(pplayer->race),
+                   _("Game: The %s have stopped building The %s in %s."),
+                   get_nation_name_plural(pplayer->nation),
                    get_imp_name_ex(pcity, pcity->currently_building),
                    pcity->name);
 
@@ -212,8 +214,8 @@ bestchoice.want); */
                  pcity->currently_building != bestchoice.choice) &&
                  is_wonder(bestchoice.choice)) {
       notify_player_ex(0, pcity->x, pcity->y, E_WONDER_STARTED,
-                    "Game: The %s have started building The %s in %s.",
-                    get_race_name_plural(city_owner(pcity)->race),
+                    _("Game: The %s have started building The %s in %s."),
+                    get_nation_name_plural(city_owner(pcity)->nation),
                     get_imp_name_ex(pcity, bestchoice.choice), pcity->name);
       pcity->currently_building = bestchoice.choice;
       pcity->is_building_unit    = (bestchoice.type > 0);
@@ -229,20 +231,21 @@ bestchoice.want); */
 
   /* fallbacks should never happen anymore, but probably
      could somehow. -- Syela */
-  flog(LOG_DEBUG, "Falling back - %s didn't want soldiers, settlers,"
-       " or buildings", pcity->name);
+  freelog(LOG_VERBOSE, "Falling back - %s didn't want soldiers, settlers,"
+                       " or buildings", pcity->name);
   if (can_build_improvement(pcity, B_BARRACKS)) {
     pcity->currently_building = B_BARRACKS;
     pcity->is_building_unit = 0;
   } else {
-    pcity->currently_building = U_SETTLERS; /* yes, this could be truly bad */
+    pcity->currently_building = best_role_unit(pcity, F_SETTLERS); /* yes, this could be truly bad */
     pcity->is_building_unit = 1;
   }
 /* I think fallbacks only happen with techlevel 0, and even then are rare.
 I haven't seen them, but I want to somewhat prepare for them anyway. -- Syela */  
 }
 
-int ai_city_defender_value(struct city *pcity, int a_type, int d_type)            
+#ifdef GRAVEDANGERWORKS
+static int ai_city_defender_value(struct city *pcity, int a_type, int d_type)
 {
   int m;
   m = get_virtual_defense_power(a_type, d_type, pcity->x, pcity->y);
@@ -251,8 +254,9 @@ int ai_city_defender_value(struct city *pcity, int a_type, int d_type)
   m /= 30;
   return (m * m);
 }
+#endif
 
-void try_to_sell_stuff(struct player *pplayer, struct city *pcity)
+static void try_to_sell_stuff(struct player *pplayer, struct city *pcity)
 {
   int id;
   for (id = 0; id < B_LAST; id++) {
@@ -264,9 +268,9 @@ void try_to_sell_stuff(struct player *pplayer, struct city *pcity)
   }
 }
 
-void ai_new_spend_gold(struct player *pplayer)
+static void ai_new_spend_gold(struct player *pplayer)
 {
-  int buycost, id, cost, frugal = 0;
+  int buycost, id, cost, frugal = 0, trireme=0;
   int total, build, did_upgrade;
   struct ai_choice bestchoice;
   struct city *pcity = NULL;
@@ -291,19 +295,23 @@ void ai_new_spend_gold(struct player *pplayer)
       unit_list_iterate(map_get_tile(pcity->x, pcity->y)->units, punit)
         id = can_upgrade_unittype(pplayer, punit->type);
         did_upgrade = 0;
-/* printf("%s wants %s, %s -> %s\n", pcity->name, unit_types[bestchoice.choice].name, 
-unit_types[punit->type].name, unit_types[id].name); */
+	freelog(LOG_DEBUG, "%s wants %s, %s -> %s", pcity->name,
+		unit_types[bestchoice.choice].name, 
+		unit_types[punit->type].name,
+		(id>=0 ? unit_types[id].name : "-1"));
         if (id == bestchoice.choice && ai_fuzzy(pplayer,1)) {
 	  /* and I can simply upgrade a unit I already have */
-/* printf("Trying to upgrade in %s.\n", pcity->name); */
+	  freelog(LOG_DEBUG, "Trying to upgrade in %s.", pcity->name);
           cost = unit_upgrade_price(pplayer, punit->type, id);
           if (cost < buycost) { /* and the upgrade would be cheaper */
             if (cost < pplayer->economic.gold) { /* let's just upgrade */
               pplayer->economic.gold -= cost;
               if (punit->hp==get_unit_type(punit->type)->hp)
                 punit->hp=get_unit_type(id)->hp;
-              notify_player(pplayer, "Game: %s upgraded to %s in %s for %d credits",
-                unit_types[punit->type].name, unit_types[id].name, pcity->name, cost);
+              notify_player(pplayer,
+			    _("Game: %s upgraded to %s in %s for %d credits."),
+			    unit_types[punit->type].name, unit_types[id].name,
+			    pcity->name, cost);
               punit->type = id;
               send_unit_info(0, punit, 0);
               send_player_info(pplayer, pplayer);
@@ -313,17 +321,21 @@ unit_types[punit->type].name, unit_types[id].name); */
             }
           }
         }
-        if ((id == bestchoice.choice || (punit->type == U_WARRIORS &&
-            bestchoice.choice == U_PHALANX) || (punit->type == U_PHALANX &&
-            bestchoice.choice == U_MUSKETEERS)) && ai_fuzzy(pplayer,1))  {
+	if ((id == bestchoice.choice
+	     || (unit_has_role(bestchoice.choice, L_DEFEND_GOOD)
+		 && is_on_unit_upgrade_path(bestchoice.choice, punit->type)))
+	    && ai_fuzzy(pplayer, 1)) {
           if (!did_upgrade) { /* might want to disband */
-            build = pcity->shield_stock + (get_unit_type(punit->type)->build_cost>>1);
+            build = pcity->shield_stock + (get_unit_type(punit->type)->build_cost/2);
             total = get_unit_type(bestchoice.choice)->build_cost;
             cost=(total-build)*2+(total-build)*(total-build)/20;
             if ((bestchoice.want <= 100 && build >= total) ||
                   (pplayer->economic.gold >= cost)) {
-              notify_player(pplayer, "Game: %s disbanded in %s.",
-                unit_types[punit->type].name, pcity->name);
+	      freelog(LOG_VERBOSE, "%s disbanding %s in %s to help get %s",
+		   pplayer->name, unit_types[punit->type].name, pcity->name,
+		   unit_types[bestchoice.choice].name);
+              notify_player(pplayer, _("Game: %s disbanded in %s."),
+			    unit_types[punit->type].name, pcity->name);
               pcity->shield_stock+=(get_unit_type(punit->type)->build_cost/2);
               send_city_info(pplayer, pcity, 0);
               wipe_unit(pplayer, punit);
@@ -341,7 +353,7 @@ unit_types[punit->type].name, unit_types[id].name); */
                buycost >= 200) ; /* wait for more vans */
       else if (bestchoice.type && unit_flag(bestchoice.choice, F_SETTLERS) &&
           !city_got_effect(pcity, B_GRANARY) && (pcity->size < 2 ||
-          pcity->food_stock < (pcity->size - 1) * game.foodbox)) ;
+         pcity->food_stock < pcity->size * game.foodbox)) ;
       else if (bestchoice.type && bestchoice.type < 3 && /* not a defender */
         buycost > unit_types[bestchoice.choice].build_cost * 2) { /* too expensive */
         if (unit_flag(bestchoice.choice, F_CARAVAN) &&
@@ -352,7 +364,7 @@ unit_types[punit->type].name, unit_types[id].name); */
         pcity->ai.grave_danger > (assess_defense_quadratic(pcity) +
   ai_city_defender_value(pcity, U_LEGION, bestchoice.choice)) * 2 &&
                pcity->food_stock + pcity->food_surplus < 10) { /* We're DOOMED */
-printf("%s is DOOMED!\n", pcity->name);
+	 freelog(LOG_VERBOSE, "%s is DOOMED!", pcity->name);
          try_to_sell_stuff(pplayer, pcity); /* and don't buy stuff */
          pcity->currently_building = ai_choose_defender_limited(pcity,
                   pcity->shield_stock + pcity->shield_surplus, 0);
@@ -364,9 +376,11 @@ printf("%s is DOOMED!\n", pcity->name);
 	  really_handle_city_buy(pplayer, pcity); /* adequately tested now */
 	}
       } else if (bestchoice.type || !is_wonder(bestchoice.choice)) {
-/*        printf("%s wants %s but can't afford to buy it (%d < %d).\n",
-pcity->name, (bestchoice.type ? unit_name(bestchoice.choice) :
-get_improvement_name(bestchoice.choice)),  pplayer->economic.gold, buycost); */
+	freelog(LOG_DEBUG, "%s wants %s but can't afford to buy it"
+		      " (%d < %d).", pcity->name,
+		      (bestchoice.type ? unit_name(bestchoice.choice)
+		       : get_improvement_name(bestchoice.choice)),
+		      pplayer->economic.gold, buycost);
         if (bestchoice.type && !unit_flag(bestchoice.choice, F_NONMIL)) {
           if (pcity->ai.grave_danger && !assess_defense(pcity)) { /* oh dear */
             try_to_sell_stuff(pplayer, pcity);
@@ -405,10 +419,15 @@ get_improvement_name(bestchoice.choice)),  pplayer->economic.gold, buycost); */
     pcity->ai.choice.want = 0; /* not dealing with this city a second time */
   } while (bestchoice.want > 0);
 /* always upgrade triremes, since they vanish and are FIELDUNITS */
-  id = can_upgrade_unittype(pplayer, U_TRIREME);
+  if (num_role_units(F_TRIREME)==0)
+    id = -1;
+  else {
+    trireme = get_role_unit(F_TRIREME, 0);
+    id = can_upgrade_unittype(pplayer, trireme);
+  }
   if (id > 0 && !frugal) {
     unit_list_iterate(pplayer->units, punit)
-      if (punit->type != U_TRIREME) continue;
+      if (punit->type != trireme) continue;
       pcity = map_get_city(punit->x, punit->y);
       if (!pcity) continue;
       cost = unit_upgrade_price(pplayer, punit->type, id);
@@ -417,8 +436,10 @@ get_improvement_name(bestchoice.choice)),  pplayer->economic.gold, buycost); */
         pplayer->economic.gold -= cost;
         if (punit->hp==get_unit_type(punit->type)->hp)
           punit->hp=get_unit_type(id)->hp;
-        notify_player(pplayer, "Game: %s upgraded to %s in %s for %d credits",
-          unit_types[punit->type].name, unit_types[id].name, pcity->name, cost);
+        notify_player(pplayer,
+		      _("Game: %s upgraded to %s in %s for %d credits."),
+		      unit_types[punit->type].name, unit_types[id].name,
+		      pcity->name, cost);
         punit->type = id;
         send_unit_info(0, punit, 0);
         send_player_info(pplayer, pplayer);
@@ -432,7 +453,8 @@ get_improvement_name(bestchoice.choice)),  pplayer->economic.gold, buycost); */
 **************************************************************************/
 
 void ai_manage_cities(struct player *pplayer)
-{ 
+{
+  int i;
   pplayer->ai.maxbuycost = 0;
 
   city_list_iterate(pplayer->cities, pcity)
@@ -459,15 +481,22 @@ we don't rely on the seamap being current since we will recalculate. -- Syela */
 
   ai_new_spend_gold(pplayer);
 
-  if (get_invention(pplayer, A_CODE) != TECH_KNOWN) {
-    pplayer->ai.tech_want[A_CODE] += 150 * city_list_size(&pplayer->cities) *
-              tech_goal_turns(pplayer, A_CODE);
-  } else if (get_invention(pplayer, A_REPUBLIC) != TECH_KNOWN) {
-    pplayer->ai.tech_want[A_REPUBLIC] += city_list_size(&pplayer->cities) *
-              (tech_goal_turns(pplayer, A_REPUBLIC) * 90 + 90);
-/* these may need to be fudged further sometime -- Syela */
-    if (get_invention(pplayer, A_MONARCHY) != TECH_KNOWN)
-      pplayer->ai.tech_want[A_MONARCHY] += 150 * city_list_size(&pplayer->cities);
+  /* use ai_gov_tech_hints: */
+  for(i=0; i<MAX_NUM_TECH_LIST; i++) {
+    struct ai_gov_tech_hint *hint = &ai_gov_tech_hints[i];
+
+    if (hint->tech == A_LAST)
+      break;
+    if (get_invention(pplayer, hint->tech) != TECH_KNOWN) {
+      pplayer->ai.tech_want[hint->tech] += city_list_size(&pplayer->cities)
+	* (hint->turns_factor * tech_goal_turns(pplayer, hint->tech)
+	   + hint->const_factor);
+      if (hint->get_first)
+	break;
+    } else {
+      if (hint->done)
+	break;
+    }
   }
 }
 
@@ -491,7 +520,8 @@ int city_get_buildings(struct city *pcity)
 ... find a good (bad) tile to remove
 **************************************************************************/
 
-int worst_elvis_tile(struct city *pcity, int x, int y, int bx, int by, int foodneed, int prodneed)
+static int worst_elvis_tile(struct city *pcity, int x, int y, int bx, int by,
+			    int foodneed, int prodneed)
 {
   int a, b;
   a = city_tile_value(pcity, x, y,
@@ -507,9 +537,11 @@ int worst_elvis_tile(struct city *pcity, int x, int y, int bx, int by, int foodn
 ...
 **************************************************************************/
 
-int is_defender_unit(int unit_type) 
+static int is_defender_unit(int unit_type) 
 {
-  return ((U_WARRIORS <= unit_type) && (unit_type <= U_MECH));
+  return unit_has_role(unit_type, L_DEFEND_GOOD)
+      || unit_has_role(unit_type, L_DEFEND_OK);
+  /* return ((U_WARRIORS <= unit_type) && (unit_type <= U_MECH)); */
 }
 
 /************************************************************************** 
@@ -553,7 +585,7 @@ int city_get_settlers(struct city *pcity)
 int ai_in_initial_expand(struct player *pplayer)
 {
   int expand_cities [3] = {3, 5, 7};
-  return (pplayer->score.cities < expand_cities[get_race(pplayer)->expand]);  
+  return (pplayer->score.cities < expand_cities[get_nation_by_plr(pplayer)->expand]);  
 }
 
 /************************************************************************** 
@@ -566,31 +598,19 @@ int unit_attack_desirability(int i)
 
 void ai_choose_ferryboat(struct player *pplayer, struct city *pcity, struct ai_choice *choice)
 {
-  if (can_build_unit(pcity, U_TRANSPORT)) choice->choice = U_TRANSPORT;
-  else {
-    pplayer->ai.tech_want[A_INDUSTRIALIZATION] += choice->want;
-    if (can_build_unit(pcity, U_GALLEON)) choice->choice = U_GALLEON;
-    else {
-      pplayer->ai.tech_want[A_MAGNETISM] += choice->want;
-      if (can_build_unit(pcity, U_CARAVEL)) choice->choice = U_CARAVEL;
-      else {
-        pplayer->ai.tech_want[A_NAVIGATION] += choice->want;
-        if (can_build_unit(pcity, U_TRIREME)) choice->choice = U_TRIREME;
-        else {
-          pplayer->ai.tech_want[A_MAPMAKING] += choice->want;
-        }
-      } /* yeah, this is ugly code.  Thbbbbbbpt. -- Syela */
-    }
-  }
+  ai_choose_role_unit(pplayer, pcity, choice, L_FERRYBOAT,  choice->want);
 }
 
-int ai_choose_attacker(struct city *pcity, enum unit_move_type which)
-{ /* don't ask me why this is in aicity, I can't even remember -- Syela */
+/* don't ask me why this is in aicity, I can't even remember -- Syela */
+static int ai_choose_attacker(struct city *pcity, enum unit_move_type which)
+{ 
   int i;
   int best = 0;
   int bestid = 0;
   int cur;
-  for (i = U_WARRIORS; i <= U_BATTLESHIP ; i++) { /* not dealing with planes yet */
+  for (i = 0; i < game.num_unit_types; i++) {
+    /* not dealing with planes yet */
+    if (!is_ai_simple_military(i)) continue;
     cur = unit_attack_desirability(i);
     if (which == unit_types[i].move_type) {
       if (can_build_unit(pcity, i) && (cur > best || (cur == best &&
@@ -619,7 +639,8 @@ int ai_choose_defender_versus(struct city *pcity, int v)
   int best= 0;
   int bestid = 0;
 
-  for (i = U_WARRIORS; i <= U_BATTLESHIP; i++) {
+  for (i = 0; i < game.num_unit_types; i++) {
+    if (!is_ai_simple_military(i)) continue;
     m = unit_types[i].move_type;
     if (can_build_unit(pcity, i) && (m == LAND_MOVING || m == SEA_MOVING)) {
       j = get_virtual_defense_power(v, i, pcity->x, pcity->y);
@@ -633,13 +654,34 @@ int ai_choose_defender_versus(struct city *pcity, int v)
   return bestid;
 }
 
+/* I'm not sure how to generalize this for rulesets, because I
+   don't understand what its trying to return.  A military ground
+   unit with hp>10 could be a Cannon, which doesn't seem like
+   a "normal defender" to me...   But I think I have to do something,
+   because for the Civ1 ruleset, all units have hp==10, so this
+   would return 0.  Hence this strange a+2*d formula, which gives
+   the same results as (hp>10) for the default set, and does
+   _something_ for other sets.  This should be revisited.  --dwp
+*/
 int has_a_normal_defender(struct city *pcity)
 {
+#if 0 /* pre-rulesets */
   unit_list_iterate(map_get_tile(pcity->x, pcity->y)->units, punit)
     if (is_military_unit(punit) && is_ground_unit(punit) &&
         unit_types[punit->type].hp > 10 && /* for when we don't get Feudalism */
         can_build_unit(pcity, punit->type)) return 1;
   unit_list_iterate_end;
+#else
+  unit_list_iterate(map_get_tile(pcity->x, pcity->y)->units, punit)
+    if (is_military_unit(punit)
+	&& is_ground_unit(punit)
+	&& (unit_types[punit->type].attack_strength
+	    + 2*unit_types[punit->type].defense_strength) >= 9
+	&& can_build_unit(pcity, punit->type)) {
+      return 1;
+    }
+  unit_list_iterate_end;
+#endif
   return 0;
 }
 
@@ -651,7 +693,8 @@ int ai_choose_defender_limited(struct city *pcity, int n, enum unit_move_type wh
   int walls = 1; /* just assume city_got_citywalls(pcity); in the long run -- Syela */
   int isdef = has_a_normal_defender(pcity);
 
-  for (i = U_WARRIORS; i <= U_BATTLESHIP; i++) {
+  for (i = 0; i < game.num_unit_types; i++) {
+    if (!is_ai_simple_military(i)) continue;
     m = unit_types[i].move_type;
     if (can_build_unit(pcity, i) && get_unit_type(i)->build_cost <= n &&
         (m == LAND_MOVING || m == SEA_MOVING) &&
@@ -680,22 +723,24 @@ int ai_choose_defender(struct city *pcity)
   return (ai_choose_defender_limited(pcity, pcity->shield_stock + 40, 0));
 }
 
+#ifdef UNUSED
 /************************************************************************** 
 ...
 **************************************************************************/
 void adjust_build_choice(struct player *pplayer, struct ai_choice *cur, 
-                               struct ai_choice *best, int advisor)
+			 struct ai_choice *best, int advisor)
 {
   island_adjust_build_choice(pplayer, cur, advisor);
   leader_adjust_build_choice(pplayer, cur, advisor);
   copy_if_better_choice(cur, best);
 }
+#endif /* UNUSED */
 
 /**************************************************************************
 ... 
 **************************************************************************/
 
-int building_unwanted(struct player *plr, int i)
+static int building_unwanted(struct player *plr, int i)
 {
   if (plr->research.researching != A_NONE)
     return 0;
@@ -706,7 +751,7 @@ int building_unwanted(struct player *plr, int i)
 ...
 **************************************************************************/
 
-void ai_sell_obsolete_buildings(struct city *pcity)
+static void ai_sell_obsolete_buildings(struct city *pcity)
 {
   int i;
   struct player *pplayer = city_owner(pcity);
@@ -717,7 +762,7 @@ void ai_sell_obsolete_buildings(struct city *pcity)
        && (wonder_replacement(pcity, i) || building_unwanted(city_owner(pcity), i))) {
       do_sell_building(pplayer, pcity, i);
       notify_player_ex(pplayer, pcity->x, pcity->y, E_IMP_SOLD,
-		       "Game: %s is selling %s (not needed) for %d", 
+		       _("Game: %s is selling %s (not needed) for %d."), 
 		       pcity->name, get_improvement_name(i), 
 		       improvement_value(i)/2);
       return; /* max 1 building each turn */
@@ -728,7 +773,6 @@ void ai_sell_obsolete_buildings(struct city *pcity)
 /**************************************************************************
  cities, build order and worker allocation stuff here..
 **************************************************************************/
-
 void ai_manage_city(struct player *pplayer, struct city *pcity)
 {
   city_check_workers(pplayer, pcity); /* no reason not to, many reasons to do so! */
@@ -743,9 +787,10 @@ void ai_manage_city(struct player *pplayer, struct city *pcity)
 ...
 **************************************************************************/
 
-int ai_find_elvis_pos(struct city *pcity, int *xp, int *yp)
+static int ai_find_elvis_pos(struct city *pcity, int *xp, int *yp)
 {
-  int x,y, foodneed, prodneed, gov;
+  struct government *g = get_gov_pcity(pcity);
+  int x,y, foodneed, prodneed;
   int luxneed, pwr, e;
 
   foodneed=(pcity->size *2) + settler_eats(pcity);
@@ -761,11 +806,10 @@ int ai_find_elvis_pos(struct city *pcity, int *xp, int *yp)
   } /* not as good as elvising all at once, but should be adequate */
 
   unit_list_iterate(pcity->units_supported, punit)
-    if (is_military_unit(punit)) prodneed++;
+    prodneed += utype_shield_cost(get_unit_type(punit->type), g);
   unit_list_iterate_end;
-  gov = get_government(pcity->owner);
-  if (gov == G_DESPOTISM) prodneed -= pcity->size;
-  if (gov == G_MONARCHY || gov == G_COMMUNISM) prodneed -= 3;
+  
+  prodneed -= citygov_free_shield(pcity, g);
 
   *xp = 0;
   *yp = 0;
@@ -792,11 +836,13 @@ int ai_find_elvis_pos(struct city *pcity, int *xp, int *yp)
     prodneed -= (e - 1); /* led to remarkable idiocy -- Syela */
   }
   if (foodneed > pcity->food_stock) {
-/*    printf("No elvis_pos in %s - would create famine.\n", pcity->name); */
+    freelog(LOG_DEBUG,
+	    "No elvis_pos in %s - would create famine.", pcity->name);
     return 0; /* Bad time to Elvis */
   }
   if (prodneed > 0) {
-/*    printf("No elvis_pos in %s - would fail-to-upkeep.\n", pcity->name); */
+    freelog(LOG_DEBUG,
+	    "No elvis_pos in %s - would fail-to-upkeep.", pcity->name);
     return 0; /* Bad time to Elvis */
   }
   return(city_tile_value(pcity, *xp, *yp, foodneed, prodneed));
@@ -820,16 +866,7 @@ int ai_make_elvis(struct city *pcity)
 /**************************************************************************
 ...
 **************************************************************************/
-
-int free_tiles(struct city *pcity)
-{
-  return 1;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void make_elvises(struct city *pcity)
+static void make_elvises(struct city *pcity)
 {
   int xp, yp, elviscost;
   pcity->ppl_elvis += (pcity->ppl_taxman + pcity->ppl_scientist);
@@ -857,7 +894,7 @@ void make_elvises(struct city *pcity)
 /**************************************************************************
 ...
 **************************************************************************/
-void make_taxmen(struct city *pcity)
+static void make_taxmen(struct city *pcity)
 {
   while (!city_unhappy(pcity) && pcity->ppl_elvis) {
     pcity->ppl_taxman++;
@@ -875,7 +912,7 @@ void make_taxmen(struct city *pcity)
 /**************************************************************************
 ...
 **************************************************************************/
-void make_scientists(struct city *pcity)
+static void make_scientists(struct city *pcity)
 {
   make_taxmen(pcity); /* reuse the code */
   pcity->ppl_scientist = pcity->ppl_taxman;
@@ -922,7 +959,7 @@ void emergency_reallocate_workers(struct player *pplayer, struct city *pcity)
   struct packet_unit_request pack;
   int i, j;
    
-  flog(LOG_DEBUG, "Emergency in %s! (%d unhap, %d hap, %d food, %d prod)",
+  freelog(LOG_VERBOSE, "Emergency in %s! (%d unhap, %d hap, %d food, %d prod)",
        pcity->name, pcity->ppl_unhappy[4], pcity->ppl_happy[4],
        pcity->food_surplus, pcity->shield_surplus);
   city_list_init(&minilist);
@@ -932,7 +969,7 @@ void emergency_reallocate_workers(struct player *pplayer, struct city *pcity)
     if(acity!=NULL && acity!=pcity && acity->owner==pcity->owner)  {
       if(acity->x==x && acity->y==y) /* can't stop working city center */
 	continue;  
-      /*printf("Availing square in %s\n", acity->name);*/
+      freelog(LOG_DEBUG, "Availing square in %s", acity->name);
       set_worker_city(acity, map_to_city_x(acity, x), 
                       map_to_city_y(acity, y), C_TILE_EMPTY);
       if (!city_list_find_id(&minilist, acity->id))
@@ -945,16 +982,17 @@ void emergency_reallocate_workers(struct player *pplayer, struct city *pcity)
     ai_scientists_taxmen(pcity);
   if (pcity->shield_surplus < 0 || city_unhappy(pcity) ||
       pcity->food_stock + pcity->food_surplus < 0) {
-    flog(LOG_DEBUG, "Emergency unresolved");
+    freelog(LOG_VERBOSE, "Emergency unresolved");
   } else {
-    flog(LOG_DEBUG, "Emergency resolved");
+    freelog(LOG_VERBOSE, "Emergency resolved");
   }
-  /*printf("Rearranging workers in %s\n", pcity->name);*/
+  freelog(LOG_DEBUG, "Rearranging workers in %s", pcity->name);
 
   unit_list_iterate(pcity->units_supported, punit)
     if (city_unhappy(pcity) && punit->unhappiness && !punit->ai.passenger) {
-       flog(LOG_DEBUG, "%s's %s is unhappy and causing unrest, disbanding it",
-	    pcity->name, unit_types[punit->type].name);
+       freelog(LOG_VERBOSE,
+	       "%s's %s is unhappy and causing unrest, disbanding it",
+	       pcity->name, unit_types[punit->type].name);
        pack.unit_id = punit->id;
        /* in rare cases the _safe might be needed? --dwp */
        handle_unit_disband_safe(pplayer, &pack, &myiter);
@@ -970,7 +1008,7 @@ void emergency_reallocate_workers(struct player *pplayer, struct city *pcity)
     city_refresh(acity);
     if (ai_fix_unhappy(acity) && ai_fuzzy(pplayer,1))
       ai_scientists_taxmen(acity);
-/*printf("Readjusting workers in %s\n", acity->name);*/
+    freelog(LOG_DEBUG, "Readjusting workers in %s", acity->name);
     send_city_info(city_owner(acity), acity, 1);
   city_list_iterate_end;
 }
