@@ -14,25 +14,30 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <player.h>
-#include <city.h>
-#include <cityturn.h>
-#include <citytools.h>
-#include <game.h>
-#include <unit.h>
-#include <unithand.h>
-#include <shared.h>
-#include <cityhand.h>
-#include <packets.h>
-#include <map.h>
-#include <mapgen.h>
-#include <aitools.h>
-#include <aihand.h>
-#include <aiunit.h>
-#include <aicity.h>
-#include <aitech.h>
-#include <sys/time.h>
-#include <log.h>
+
+#include "city.h"
+#include "game.h"
+#include "government.h"
+#include "log.h"
+#include "map.h"
+#include "packets.h"
+#include "player.h"
+#include "shared.h"
+#include "unit.h"
+
+#include "cityhand.h"
+#include "citytools.h"
+#include "cityturn.h"
+#include "spacerace.h"
+#include "unithand.h"
+
+#include "aicity.h"
+#include "aitech.h"
+#include "aitools.h"
+#include "aiunit.h"
+#include "advspace.h"
+
+#include "aihand.h"
 
 /****************************************************************************
   A man builds a city
@@ -76,7 +81,7 @@
 - continent attack
 4 Superadvanced:
 - Transporters (attack on other continents)
-- diplomati (ambassader/bytte tech/bytte kort med anden AI)
+- diplomacy (embassies / tech trading / map trading with other AIs)
 
 
 Step 1 is the basics, can be used for the blank seat people usually have when
@@ -91,12 +96,12 @@ So first in step 4 will we introduce attacking AI. (if it ever gets that far)
 -- I don't know who wrote this, but I've gone a different direction -- Syela
  */
 
-void ai_before_work(struct player *pplayer);
-void ai_manage_taxes(struct player *pplayer); 
-void ai_manage_government(struct player *pplayer);
-void ai_manage_diplomacy(struct player *pplayer);
-void ai_after_work(struct player *pplayer);
-
+static void ai_before_work(struct player *pplayer);
+static void ai_manage_taxes(struct player *pplayer); 
+static void ai_manage_government(struct player *pplayer);
+static void ai_manage_diplomacy(struct player *pplayer);
+static void ai_manage_spaceship(struct player *pplayer);
+static void ai_after_work(struct player *pplayer);
 
 /**************************************************************************
  Main AI routine.
@@ -115,23 +120,26 @@ void ai_do_last_activities(struct player *pplayer)
 instead of right before it.  Managing units before end-turn reset now. -- Syela */
   calculate_tech_turns(pplayer); /* has to be here thanks to the above */
   ai_manage_cities(pplayer);
-/* manage cities will establish our tech_wants. */
-/* if I were upgrading units, which I'm not, I would do it here -- Syela */ 
-/* printf("Managing %s's taxes.\n", pplayer->name); */
+  /* manage cities will establish our tech_wants. */
+  /* if I were upgrading units, which I'm not, I would do it here -- Syela */ 
+  freelog(LOG_DEBUG, "Managing %s's taxes.", pplayer->name);
   ai_manage_taxes(pplayer); 
-/* printf("Managing %s's government.\n", pplayer->name); */
+  freelog(LOG_DEBUG, "Managing %s's government.", pplayer->name);
   ai_manage_government(pplayer); 
   ai_manage_diplomacy(pplayer);
   ai_manage_tech(pplayer); 
+  freelog(LOG_DEBUG, "Managing %s's spaceship.", pplayer->name);
+  ai_manage_spaceship(pplayer);
+  freelog(LOG_DEBUG, "Managing %s's taxes.", pplayer->name);
   ai_after_work(pplayer);
-/* printf("Done with %s.\n", pplayer->name); */
+  freelog(LOG_DEBUG, "Done with %s.", pplayer->name);
 }
 
 /**************************************************************************
  update advisors/structures
 **************************************************************************/
   
-void ai_before_work(struct player *pplayer)
+static void ai_before_work(struct player *pplayer)
 {
   ; /* all instances of fnord and references thereto have been deleted. -- Syela */
 }
@@ -141,16 +149,30 @@ void ai_before_work(struct player *pplayer)
  Trade tech and stuff, this one will probably be blank for a long time.
 **************************************************************************/
 
-void ai_manage_diplomacy(struct player *pplayer)
+static void ai_manage_diplomacy(struct player *pplayer)
 {
 
 }
 
+static void ai_manage_spaceship(struct player *pplayer)
+{
+  if (game.spacerace) {
+    if (pplayer->spaceship.state == SSHIP_STARTED) {
+      ai_spaceship_autoplace(pplayer, &pplayer->spaceship);
+      /* if we have built the best possible spaceship  -- AJS 19990610 */
+      if ((pplayer->spaceship.structurals == NUM_SS_STRUCTURALS) &&
+        (pplayer->spaceship.components == NUM_SS_COMPONENTS) &&
+        (pplayer->spaceship.modules == NUM_SS_MODULES))
+        handle_spaceship_launch(pplayer);
+    }
+  }
+}
+
 /**************************************************************************
  well am not sure what will happend here yet, maybe some more analysis
 **************************************************************************/
 
-void ai_after_work(struct player *pplayer)
+static void ai_after_work(struct player *pplayer)
 {
 
 }
@@ -159,7 +181,7 @@ void ai_after_work(struct player *pplayer)
 /**************************************************************************
 ...
 **************************************************************************/
-
+#ifdef UNUSED
 int ai_calc_city_buy(struct city *pcity)
 {
   int val;
@@ -174,6 +196,7 @@ int ai_calc_city_buy(struct city *pcity)
     return val;
   }
 }
+#endif /* UNUSED */
 
 /**************************************************************************
 .. Spend money
@@ -211,15 +234,16 @@ void ai_spend_gold(struct player *pplayer, int gold)
     pplayer->economic.gold -= gold;
   }
 }
-#endif 
+#endif /* UNUSED */
 
 
 /**************************************************************************
 .. Set tax/science/luxury rates. Tax Rates > 40 indicates a crisis.
+ total rewrite by Syela 
 **************************************************************************/
-
-void ai_manage_taxes(struct player *pplayer) 
-{ /* total rewrite by Syela */
+static void ai_manage_taxes(struct player *pplayer) 
+{
+  struct government *g = get_gov_pplayer(pplayer);
   int gnow = pplayer->economic.gold;
   int sad = 0, trade = 0, m, n, i, expense = 0, tot;
   int waste[40]; /* waste with N elvises */
@@ -244,7 +268,7 @@ void ai_manage_taxes(struct player *pplayer)
     city_refresh(pcity);
     sad += pcity->ppl_unhappy[4];
     trade += pcity->trade_prod * city_tax_bonus(pcity) / 100;
-/*    printf("%s has %d trade.\n", pcity->name, pcity->trade_prod); */
+    freelog(LOG_DEBUG, "%s has %d trade.", pcity->name, pcity->trade_prod);
     for (i = 0; i < B_LAST; i++)
       if (city_got_building(pcity, i)) expense += improvement_upkeep(pcity,i);
   city_list_iterate_end;
@@ -259,26 +283,29 @@ void ai_manage_taxes(struct player *pplayer)
 
   pplayer->economic.luxury = 0;
 
-  city_list_iterate(pplayer->cities, pcity) 
+  city_list_iterate(pplayer->cities, pcity) {
 
-/* this code must be ABOVE the elvises[] if SIMPLISTIC is off */
-/* printf("Does %s want to be bigger? %d\n", pcity->name, wants_to_be_bigger(pcity)); */
-    if (pcity->size > 4 && pplayer->government >= G_REPUBLIC &&
+    /* this code must be ABOVE the elvises[] if SIMPLISTIC is off */
+    freelog(LOG_DEBUG, "Does %s want to be bigger? %d",
+		  pcity->name, wants_to_be_bigger(pcity));
+    if (g->rapture_size && pcity->size >= g->rapture_size  &&
         pcity->food_surplus > 0 &&
         !pcity->ppl_unhappy[4] && wants_to_be_bigger(pcity) &&
 	ai_fuzzy(pplayer,1)) {
-/*      printf("%d happy people in %s\n", pcity->ppl_happy[4], pcity->name);*/
-      n = (((pcity->size + 1)>>1) - pcity->ppl_happy[4]) * 20;
+      freelog(LOG_DEBUG, "%d happy people in %s",
+		    pcity->ppl_happy[4], pcity->name);
+      n = ((pcity->size/2) - pcity->ppl_happy[4]) * 20;
       if (n > pcity->ppl_content[1] * 20) n += (n - pcity->ppl_content[1] * 20);
       m = ((((city_got_effect(pcity, B_GRANARY) ? 3 : 2) *
-	     pcity->size * game.foodbox)>>1) -
+	     (pcity->size+1) * game.foodbox)/2) -
            pcity->food_stock) * food_weighting(pcity->size);
-/*printf("Checking HHJJ for %s, m = %d\n", pcity->name, m);*/
+      freelog(LOG_DEBUG, "Checking HHJJ for %s, m = %d", pcity->name, m);
       tot = 0;
       for (i = 0; i <= 10; i++) {
         if (pcity->trade_prod * i * city_tax_bonus(pcity) >= n * 100) {
-/*if (!tot) printf("%s celebrates at %d.\n", pcity->name, i * 10);*/
-          hhjj[i] += (pcity->was_happy ? m : m>>1);
+	  if (!tot) freelog(LOG_DEBUG, "%s celebrates at %d.",
+			    pcity->name, i * 10);
+          hhjj[i] += (pcity->was_happy ? m : m/2);
           tot++;
         }
       }
@@ -310,16 +337,17 @@ void ai_manage_taxes(struct player *pplayer)
       m = ((n * 100 - pcity->trade_prod * city_tax_bonus(pcity) * i + 1999) / 2000);
       if (m >= 0) elvises[i] += waste[m];
     }
+  }
   city_list_iterate_end;
     
   for (i = 0; i <= 10; i++) elvises[i] += (trade * i) / 10 * TRADE_WEIGHTING;
   /* elvises[i] is the production + income lost to elvises with lux = i * 10 */
   n = 0; /* default to 0 lux */
   for (i = 1; i <= maxrate; i++) if (elvises[i] < elvises[n]) n = i;
-/* two thousand zero zero party over it's out of time */
+  /* two thousand zero zero party over it's out of time */
   for (i = 0; i <= 10; i++) {
     hhjj[i] -= (trade * i) / 10 * TRADE_WEIGHTING; /* hhjj is now our bonus */
-/*    printf("hhjj[%d] = %d for %s.\n", i, hhjj[i], pplayer->name);  */
+    freelog(LOG_DEBUG, "hhjj[%d] = %d for %s.", i, hhjj[i], pplayer->name);
   }
 
   m = n; /* storing the lux we really need */
@@ -331,8 +359,8 @@ void ai_manage_taxes(struct player *pplayer)
   while (n > maxrate) n--; /* Better to cheat on lux than on tax -- Syela */
   if (m > 10 - n) m = 10 - n;
 
-/*printf("%s has %d trade and %d expense.  Min lux = %d, tax = %d\n",
-pplayer->name, trade, expense, m, n);*/
+  freelog(LOG_DEBUG, "%s has %d trade and %d expense."
+	  "  Min lux = %d, tax = %d", pplayer->name, trade, expense, m, n);
 
 /* Peter Schaefer points out (among other things) that in pathological cases
 (like expense == 468) the AI will try to celebrate with m = 10 and then abort */
@@ -352,7 +380,7 @@ pplayer->name, trade, expense, m, n);*/
   } else { /* have to balance things logically */
 /* if we need 50 gold and we have trade = 100, need 50 % tax (n = 5) */
 /*  n = ((ai_gold_reserve(pplayer) - gnow - expense) ... I hate typos. -- Syela */
-    n = ((ai_gold_reserve(pplayer) - gnow + expense + cities) * 20 + (trade<<1) - 1) / (trade<<1);
+    n = ((ai_gold_reserve(pplayer) - gnow + expense + cities) * 20 + (trade*2) - 1) / (trade*2);
 /* same bug here as above, caused us not to afford city walls we needed. -- Syela */
     if (n < 0) n = 0; /* shouldn't allow 0 tax? */
     while (n > 10 - (pplayer->economic.luxury / 10) || n > maxrate) n--;
@@ -381,18 +409,22 @@ pplayer->name, trade, expense, m, n);*/
       unit_list_iterate(pcity->units_supported, punit)
         incity = map_get_city(punit->x, punit->y);
         if (incity && pcity->shield_surplus < 0) {
+	  /* Note that disbanding here is automatically safe (we don't
+	   * need to use handle_unit_disband_safe()), because the unit is
+	   * in a city, so there are no passengers to get disbanded. --dwp
+	   */
           if (incity == pcity) {
             if (defender) {
               if (unit_vulnerability_virtual(punit) <
                   unit_vulnerability_virtual(defender)) {
-		flog(LOG_DEBUG, "Disbanding %s in %s",
-		     unit_types[punit->type].name, pcity->name);
+		freelog(LOG_VERBOSE, "Disbanding %s in %s",
+			unit_types[punit->type].name, pcity->name);
                 pack.unit_id = punit->id;
                 handle_unit_disband(pplayer, &pack);
                 city_refresh(pcity);
               } else {
-		flog(LOG_DEBUG, "Disbanding %s in %s",
-		     unit_types[defender->type].name, pcity->name);
+		freelog(LOG_VERBOSE, "Disbanding %s in %s",
+			unit_types[defender->type].name, pcity->name);
                 pack.unit_id = defender->id;
                 handle_unit_disband(pplayer, &pack);
                 city_refresh(pcity);
@@ -404,8 +436,8 @@ pplayer->name, trade, expense, m, n);*/
             pack.city_id = incity->id;
             handle_unit_change_homecity(pplayer, &pack);
             city_refresh(pcity);
-	    flog(LOG_DEBUG, "Reassigning %s from %s to %s",
-		 unit_types[punit->type].name, pcity->name, incity->name);
+	    freelog(LOG_VERBOSE, "Reassigning %s from %s to %s",
+		    unit_types[punit->type].name, pcity->name, incity->name);
           }
         } /* end if */
       unit_list_iterate_end;
@@ -413,10 +445,10 @@ pplayer->name, trade, expense, m, n);*/
         unit_list_iterate(pcity->units_supported, punit)
           if (punit != defender && pcity->shield_surplus < 0) {
 	    /* the defender MUST NOT be disbanded! -- Syela */
-	    flog(LOG_DEBUG, "Disbanding %s's %s",
-		 pcity->name, unit_types[punit->type].name);
+	    freelog(LOG_VERBOSE, "Disbanding %s's %s",
+		    pcity->name, unit_types[punit->type].name);
             pack.unit_id = punit->id;
-            handle_unit_disband(pplayer, &pack);
+            handle_unit_disband_safe(pplayer, &pack, &myiter);
             city_refresh(pcity);
           }
         unit_list_iterate_end;
@@ -431,44 +463,113 @@ pplayer->name, trade, expense, m, n);*/
 /**************************************************************************
  change the government form, if it can and there is a good reason
 **************************************************************************/
-
-void ai_manage_government(struct player *pplayer)
+#ifndef NEW_GOV_EVAL
+static void ai_manage_government(struct player *pplayer)
 {
-  int government = get_race(pplayer)->goals.government;
-  government = G_REPUBLIC; /* need to be REPUBLIC+ to love */
-/* advantages of DEMOCRACY: partisans, no bribes, no corrup, +1 content if courthouse */
-/* disadvantages of DEMOCRACY: doubled unhappiness from attacking units, anarchy */
-/* realistically we should allow DEMOC in some circumstances but not yet -- Syela */
-  if (pplayer->government == government)
-    return;
-  if (can_change_to_government(pplayer, government)) {
-    ai_government_change(pplayer, government);
+  int goal = get_nation_by_plr(pplayer)->goals.government;
+  int subgoal;
+  int failsafe;
+  
+  goal = game.ai_goal_government;
+  /* Was G_REPUBLIC; need to be REPUBLIC+ to love */
+  
+  /* advantages of DEMOCRACY:
+        partisans, no bribes, no corrup, +1 content if courthouse;
+     disadvantages of DEMOCRACY:
+        doubled unhappiness from attacking units, anarchy 
+     realistically we should allow DEMOC in some circumstances but
+     not yet -- Syela
+  */
+  if (pplayer->government == goal) {
+    freelog(LOG_DEBUG, "ai_man_gov (%s): there %d", pplayer->name, goal);
     return;
   }
-  switch (government) {
-  case G_COMMUNISM:
-    if (can_change_to_government(pplayer, G_MONARCHY)) 
-      ai_government_change(pplayer, G_MONARCHY);
-    break;
-  case G_DEMOCRACY:
-    if (can_change_to_government(pplayer, G_REPUBLIC)) 
-      ai_government_change(pplayer, G_REPUBLIC);
-    else if (can_change_to_government(pplayer, G_MONARCHY)) 
-      ai_government_change(pplayer, G_MONARCHY);
-    break;
-  case G_REPUBLIC:
-    if (can_change_to_government(pplayer, G_MONARCHY)) 
-      ai_government_change(pplayer, G_MONARCHY); /* better than despotism! -- Syela */
-    break;
+  if (can_change_to_government(pplayer, goal)) {
+    freelog(LOG_DEBUG, "ai_man_gov (%s): change %d", pplayer->name, goal);
+    ai_government_change(pplayer, goal);
+    return;
+  }
+  failsafe = 0;
+  while((subgoal = get_government(goal)->subgoal) >= 0) {
+    if (can_change_to_government(pplayer, subgoal)) {
+      freelog(LOG_DEBUG, "ai_man_gov (%s): change sub %d (%d)",
+	      pplayer->name, subgoal, goal);
+      ai_government_change(pplayer, subgoal);
+      break;
+    }
+    goal = subgoal;
+    if (++failsafe > game.government_count) {
+      freelog(LOG_NORMAL, "Loop in ai_manage_government? (%s)",
+	      pplayer->name);
+      return;
+    }
   }
 
-  if( pplayer->government == G_ANARCHY ){
+  if (pplayer->government == game.government_when_anarchy) {
     /* if the ai ever intends to stay anarchy, */
     /* change condition to if( (pplayer->revolution==0) && */
-    if( ((pplayer->revolution<=0) || (pplayer->revolution>5)) &&
-       can_change_to_government(pplayer, G_DESPOTISM) 
-       )
-      { ai_government_change(pplayer, G_DESPOTISM); }
-  }/* if ANARCHY */
+    if( ((pplayer->revolution<=0) || (pplayer->revolution>5))
+	&& can_change_to_government(pplayer, game.default_government)) {
+      freelog(LOG_DEBUG, "ai_man_gov (%s): change from anarchy",
+	      pplayer->name);
+      ai_government_change(pplayer, game.default_government);
+    }
+  }
 }
 
+#else  /* following may need updating before enabled --dwp */
+
+#define DEBUG_AMG_RATING        (0)
+#define DEBUG_AMG_BOOST		(0)
+#define DEBUG_AMG_CHANGED	(0)
+#define DEBUG_AMG_CHOICE	(1)
+static void ai_manage_government(struct player *pplayer)
+{
+  static int prev_rating[10][10];  /* these are for debugging only */
+  static int prev_choice[10];      /* -- SKi*/
+  int best_government = pplayer->government,
+      best_rating = ai_evaluate_government(pplayer, get_gov_pplayer(pplayer));
+  int i;
+
+  for (i=0; i<game.government_count; ++i) {
+    struct government *g = &governments[ i ];
+    int rating;
+
+    rating = ai_evaluate_government (pplayer, g);
+    /* give bonus to current government */
+/*    if (i == pplayer->government)
+      rating += 2500; */
+
+    if (DEBUG_AMG_RATING && prev_rating[pplayer->player_no][i] != rating && (rating > 0 || rating > prev_rating[pplayer->player_no][i]))
+      freelog (LOG_DEBUG, "%d: ai_manage_government: %s: %s: %d (%+d)", game.year, pplayer->name, g->name, rating, rating - prev_rating[pplayer->player_no][i]);
+    prev_rating[pplayer->player_no][i] = rating;
+    /* compare this government with the best so far */
+    if (rating > best_rating || best_government == -1) {
+      if (can_change_to_government(pplayer, i)) {
+        /* Use this thing! -- SKi */
+        best_government = i;
+        best_rating = rating;
+      } else {
+        if (get_invention(pplayer, g->required_tech) != TECH_KNOWN && rating > 0) {
+          /* Research this thing! -- SKi */
+	  if (DEBUG_AMG_BOOST)
+            freelog (LOG_DEBUG, "%d: ai_manage_government: boosting %s for %s (%d -> %d)",
+              game.year,
+	      advances[ g->required_tech ].name, pplayer->name,
+              pplayer->ai.tech_want[ g->required_tech ],
+              pplayer->ai.tech_want[ g->required_tech ] + rating * 2);
+          pplayer->ai.tech_want[ g->required_tech ] += 250 + rating / 2; /* need constant modifier? */
+        }
+      }
+    }
+  }
+  if ((DEBUG_AMG_CHANGED && prev_choice[pplayer->player_no] != best_government) || DEBUG_AMG_CHOICE)
+    freelog (LOG_DEBUG, "%d: %s chooses new government: %s", game.year, pplayer->name, governments[best_government].name);
+  prev_choice[pplayer->player_no] = best_government;
+
+  if (pplayer->government != best_government) {
+    ai_government_change(pplayer, best_government);
+  }
+  return;
+}
+#endif /* NEW_GOV_EVAL */
