@@ -14,18 +14,23 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <player.h>
-#include <aitools.h>
-#include <tech.h>
-#include <game.h>
-#include <plrhand.h>
+#include "game.h"
+#include "government.h"
+#include "log.h"
+#include "player.h"
+#include "tech.h"
 
-#include <advleader.h>
-#include <advmilitary.h>
-#include <advtrade.h>
-#include <advscience.h>
-#include <advforeign.h>
-#include <advattitude.h>
+#include "plrhand.h"
+
+#include "advattitude.h"
+#include "advforeign.h"
+#include "advleader.h"
+#include "advmilitary.h"
+#include "advscience.h"
+#include "advtrade.h"
+#include "aitools.h"
+
+#include "aitech.h"
 
 /**************************************************************************
 .. AI got some tech goals, and should try to fulfill them. 
@@ -34,71 +39,103 @@
 /**************************************************************************
 .. calculate next government wish.
 **************************************************************************/
-int get_government_tech(struct player *plr)
+#ifndef NEW_GOV_EVAL
+/*
+   regression testing
+*/
+static Tech_Type_id get_government_tech(struct player *plr)
 {
-  int government = get_race(plr)->goals.government;
-  if (can_change_to_government(plr, government))
-    return 0;
-  switch (government) {
-  case G_MONARCHY:
-    return A_MONARCHY;
-    break;
+  int goal = get_nation_by_plr(plr)->goals.government;
+  int subgoal = get_government(goal)->subgoal;
+  
+  if (can_change_to_government(plr, goal)) {
+    freelog(LOG_DEBUG, "get_gov_tech (%s): have %d", plr->name, goal);
+    return A_NONE;
+  }
 
-  case G_COMMUNISM:
-    if (get_invention(plr, A_MONARCHY) == TECH_KNOWN)
-      return A_COMMUNISM;
-    else
-      return A_MONARCHY;
-    break;
-
-  case G_REPUBLIC:
-      return A_REPUBLIC;
-    break;
-
-  case G_DEMOCRACY:
-    if (get_invention(plr, A_REPUBLIC) == TECH_KNOWN)
-      return A_DEMOCRACY;
-    else 
-      return A_REPUBLIC;
-    break;
+  if (subgoal >= 0) {
+    struct government *subgov = get_government(subgoal);
+    if (get_invention(plr, subgov->required_tech) == TECH_KNOWN) {
+      freelog(LOG_DEBUG, "get_gov_tech (%s): have sub %d %s",
+	      plr->name, goal, subgov->name);
+      return get_government(goal)->required_tech;
+    } else {
+      freelog(LOG_DEBUG, "get_gov_tech (%s): do sub %d %s",
+	      plr->name, goal, subgov->name);
+      return subgov->required_tech;
+    }
+  } else {
+    freelog(LOG_DEBUG, "get_gov_tech (%s): no sub %d", plr->name, goal);
+    return get_government(goal)->required_tech;
   }
-  return 0; /* to make compiler happy */
 }
 
-int get_wonder_tech(struct player *plr)
+#else  /* following may need updating before enabled --dwp */
+
+static Tech_Type_id get_government_tech(struct player *plr)
 {
-  int building;
-  int tech=0; /* make compiler happy */
-  building = get_race(plr)->goals.wonder;
+  int i, rating;
+  int best_government = -1, best_rating = 0;
+
+  for (i = 0; i < game.government_count; ++i) {
+    struct government *g = get_government(i);
+    rating = ai_evaluate_government (plr, g);
+    if (rating > best_rating &&
+        get_invention (plr, g->required_tech) != TECH_KNOWN) {
+      best_rating = rating;
+      best_government = i;
+    }
+  }
+  if (best_government == -1)
+    return A_NONE;
+  return get_government(best_government)->required_tech;
+}
+#endif /* NEW_GOV_EVAL */
+
+/**************************************************************************
+  Returns tech corresponding to players wonder goal from nations[],
+  if it makes sense, and wonder is not already built and not obsolete.
+  Otherwise returns A_NONE.
+**************************************************************************/
+static int get_wonder_tech(struct player *plr)
+{
+  int tech = A_NONE;
+  int building = get_nation_by_plr(plr)->goals.wonder;
+  
+  if (!improvement_exists(building))
+    return A_NONE;
   if (game.global_wonders[building] || wonder_obsolete(building)) 
-    return 0;
+    return A_NONE;
   tech = improvement_types[building].tech_requirement;
+  if (!tech_exists(tech))
+    return A_NONE;
   if (get_invention(plr, tech) == TECH_KNOWN)
-    return 0;
+    return A_NONE;
   return tech;
 }
 
-void ai_next_tech_goal_default(struct player *pplayer, 
-			       struct ai_choice *choice)
+static void ai_next_tech_goal_default(struct player *pplayer, 
+				      struct ai_choice *choice)
 {
-  struct player_race *prace;
+  struct nation_type *prace;
   int bestdist = A_LAST + 1;
   int dist, i;
   int goal = 0;
   int tech;
-  prace = get_race(pplayer);
-  for (i = 0 ; i < TECH_GOALS; i++) {
-    if (get_invention(pplayer, prace->goals.tech[i]) == TECH_KNOWN) 
+  prace = get_nation_by_plr(pplayer);
+  for (i = 0 ; i < MAX_NUM_TECH_GOALS; i++) {
+    int j = prace->goals.tech[i];
+    if (!tech_exists(j) || get_invention(pplayer, j) == TECH_KNOWN) 
       continue;
-    dist = tech_goal_turns(pplayer, prace->goals.tech[i]);
+    dist = tech_goal_turns(pplayer, j);
     if (dist < bestdist) { 
       bestdist = dist;
-      goal = prace->goals.tech[i];
+      goal = j;
       break; /* remove this to restore old functionality -- Syela */
     }
   } 
   tech = get_government_tech(pplayer);
-  if (tech) {
+  if (tech != A_NONE && tech_exists(tech)) {
     dist = tech_goal_turns(pplayer, tech);
     if (dist < bestdist) { 
       bestdist = dist;
@@ -106,7 +143,7 @@ void ai_next_tech_goal_default(struct player *pplayer,
     }
   }
   tech = get_wonder_tech(pplayer);
-  if (tech) {
+  if (tech != A_NONE) {
     dist = tech_goal_turns(pplayer, tech);
     if (dist < bestdist) { 
       bestdist = dist;
@@ -119,7 +156,7 @@ void ai_next_tech_goal_default(struct player *pplayer,
   }
 }
 
-void adjust_tech_choice(struct player *pplayer, struct ai_choice *cur, 
+static void adjust_tech_choice(struct player *pplayer, struct ai_choice *cur, 
 			       struct ai_choice *best, int advisor)
 {
   if (cur->want) {
@@ -130,22 +167,30 @@ void adjust_tech_choice(struct player *pplayer, struct ai_choice *cur,
 
 /* Syela-code starts here ................................. */
 
-void find_prerequisites(struct player *pplayer, int i, int *prereq)
+static void find_prerequisites(struct player *pplayer, int i, int *prereq)
 {
   /* add tech_want[i] / j to all subtechs */
-  int t1,t2;
+  int t1, t2, known;
+
   t1 = advances[i].req[0];
   t2 = advances[i].req[1];
   if(t1>=A_LAST || t2>=A_LAST) return;
-  if (get_invention(pplayer, t1) != TECH_KNOWN) prereq[t1]++;
-  if (get_invention(pplayer, t1) == TECH_UNKNOWN)
-        find_prerequisites(pplayer, t1, prereq);
-  if (get_invention(pplayer, t2) != TECH_KNOWN) prereq[t2]++;
-  if (get_invention(pplayer, t2) == TECH_UNKNOWN)
-        find_prerequisites(pplayer, t2, prereq);
+  known=get_invention(pplayer, t1);
+  if (known != TECH_KNOWN) {
+    prereq[t1]++;
+    if (known == TECH_UNKNOWN)
+      find_prerequisites(pplayer, t1, prereq);
+  }
+  known=get_invention(pplayer, t2);
+  if (known!= TECH_KNOWN) {
+    prereq[t2]++;
+    if (known== TECH_UNKNOWN)
+      find_prerequisites(pplayer, t2, prereq);
+  }
 }
 
-void ai_select_tech(struct player *pplayer, struct ai_choice *choice, struct ai_choice *gol)
+static void ai_select_tech(struct player *pplayer, struct ai_choice *choice,
+			   struct ai_choice *gol)
 {
   int i, j, k;
   int values[A_LAST];
@@ -157,13 +202,13 @@ void ai_select_tech(struct player *pplayer, struct ai_choice *choice, struct ai_
   memset(values, 0, sizeof(values));
   memset(goal_values, 0, sizeof(goal_values));
   memset(cache, 0, sizeof(cache));
-  for (i = A_NONE; i < A_LAST; i++) {
+  for (i = A_NONE; i < game.num_tech_types; i++) {
     j = pplayer->ai.tech_turns[i];
     if (j) { /* if we already got it we don't want it */
       values[i] += pplayer->ai.tech_want[i];
       memset(prereq, 0, sizeof(prereq));
       find_prerequisites(pplayer, i, prereq);
-      for (k = A_NONE; k < A_LAST; k++) {
+      for (k = A_NONE; k < game.num_tech_types; k++) {
         if (prereq[k]) {
           cache[i][k]++;
           values[k] += pplayer->ai.tech_want[i] / j;
@@ -172,31 +217,35 @@ void ai_select_tech(struct player *pplayer, struct ai_choice *choice, struct ai_
     }
   }
 
-  for (i = A_NONE; i < A_LAST; i++) {
+  for (i = A_NONE; i < game.num_tech_types; i++) {
     if (pplayer->ai.tech_turns[i]) {
-      for (k = A_NONE; k < A_LAST; k++) {
+      for (k = A_NONE; k < game.num_tech_types; k++) {
         if (cache[i][k]) {
           goal_values[i] += values[k];
         }
       }
       goal_values[i] += values[i];
+      
 /* this is the best I could do.  It still sometimes does freaky stuff like
 setting goal to Republic and learning Monarchy, but that's what it's supposed
 to be doing; it just looks strange. -- Syela */
+      
       goal_values[i] /= pplayer->ai.tech_turns[i];
-/*if (pplayer->ai.tech_turns[i]<6)
-printf("%s: want = %d, value = %d, goal_value = %d\n", advances[i].name,
-pplayer->ai.tech_want[i], values[i], goal_values[i]);*/
+      if (pplayer->ai.tech_turns[i]<6) {
+	freelog(LOG_DEBUG, "%s: want = %d, value = %d, goal_value = %d",
+		advances[i].name, pplayer->ai.tech_want[i],
+		values[i], goal_values[i]);
+      }
     } else goal_values[i] = 0;
   }
 
   j = 0; k = 0;
-  for (i = A_NONE; i < A_LAST; i++) {
+  for (i = A_NONE; i < game.num_tech_types; i++) {
     if (values[i] > values[j] && get_invention(pplayer, i) == TECH_REACHABLE) j = i;
     if (goal_values[i] > goal_values[k]) k = i;
   }
-/*  printf("%s wants %s with desire %d (%d).\n", pplayer->name,
-    advances[j].name, values[j], pplayer->ai.tech_want[j]); */
+  freelog(LOG_DEBUG, "%s wants %s with desire %d (%d).", pplayer->name,
+		advances[j].name, values[j], pplayer->ai.tech_want[j]);
   if (choice) {
     choice->choice = j;
     choice->want = values[j] / c;
@@ -208,13 +257,14 @@ pplayer->ai.tech_want[i], values[i], goal_values[i]);*/
     gol->choice = k;
     gol->want = goal_values[k] / c;
     gol->type = goal_values[pplayer->ai.tech_goal] / c;
-/*printf("Gol->choice = %s, gol->want = %d, goal_value = %d, c = %d\n",
-advances[gol->choice].name, gol->want, goal_values[k], c);*/
+    freelog(LOG_DEBUG,
+	    "Gol->choice = %s, gol->want = %d, goal_value = %d, c = %d",
+	    advances[gol->choice].name, gol->want, goal_values[k], c);
   }
   return;
 }
 
-void ai_select_tech_goal(struct player *pplayer, struct ai_choice *choice)
+static void ai_select_tech_goal(struct player *pplayer, struct ai_choice *choice)
 {
   ai_select_tech(pplayer, 0, choice);
 }
@@ -223,7 +273,7 @@ void calculate_tech_turns(struct player *pplayer)
 {
   int i;
   memset(pplayer->ai.tech_turns, 0, sizeof(pplayer->ai.tech_turns));
-  for (i = A_NONE + 1; i < A_LAST; i++) {
+  for (i = A_FIRST; i < game.num_tech_types; i++) {
     pplayer->ai.tech_turns[i] = tech_goal_turns(pplayer, i);
   }
 }
@@ -272,17 +322,17 @@ void ai_manage_tech(struct player *pplayer)
   if (choice.choice != pplayer->research.researching) {
     if ((choice.want - choice.type) > penalty &&             /* changing */
    penalty + pplayer->research.researched <= research_time(pplayer)) {
-/*printf("%s switching from %s to %s with penalty of %d.\n",
-        pplayer->name, advances[pplayer->research.researching].name,
-        advances[choice.choice].name, penalty);*/
+      freelog(LOG_DEBUG, "%s switching from %s to %s with penalty of %d.",
+	      pplayer->name, advances[pplayer->research.researching].name,
+	      advances[choice.choice].name, penalty);
       choose_tech(pplayer, choice.choice);
     }
   }
-/* crossing my fingers on this one! -- Syela (seems to have worked!) */
+  /* crossing my fingers on this one! -- Syela (seems to have worked!) */
   if (gol.choice != pplayer->ai.tech_goal) {
-/*printf("%s changing goal from %s (want = %d) to %s (want = %d)\n",
-pplayer->name, advances[pplayer->ai.tech_goal].name, gol.type,
-advances[gol.choice].name, gol.want);*/
+    freelog(LOG_DEBUG, "%s changing goal from %s (want = %d) to %s (want = %d)",
+	    pplayer->name, advances[pplayer->ai.tech_goal].name, gol.type,
+	    advances[gol.choice].name, gol.want);
     choose_tech_goal(pplayer, gol.choice);
   }
 }
