@@ -10,20 +10,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
-#ifndef __AITOOLS_H
-#define __AITOOLS_H
+#ifndef FC__AITOOLS_H
+#define FC__AITOOLS_H
 
-#ifndef __CITY_H
+#include "city.h"		/* ai_choice */
+#include "player.h"
 
-struct ai_choice {
-  int choice;            /* what the advisor wants */
-  int want;              /* how bad it wants it (0-100) */
-  int type;              /* unit/building or other depending on question */
-};
-
-#endif
-
-#include <player.h>
+struct government;
 
 struct city *dist_nearest_city(struct player *pplayer, int x, int y);
 
@@ -34,7 +27,8 @@ int ai_gold_reserve(struct player *pplayer);
 void adjust_choice(int type, struct ai_choice *choice);
 void copy_if_better_choice(struct ai_choice *cur, struct ai_choice *best);
 void ai_advisor_choose_building(struct city *pcity, struct ai_choice *choice);
+int ai_assess_military_unhappiness(struct city *pcity, struct government *g);
 
+int ai_evaluate_government(struct player *pplayer, struct government *g);
 
-
-#endif
+#endif  /* FC__AITOOLS_H */
