@@ -10,24 +10,40 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
-#ifndef __CLINET_H
-#define __CLINET_H
-
-#include <X11/Intrinsic.h>
+#ifndef FC__CLINET_H
+#define FC__CLINET_H
 
 #define DEFAULT_SOCK_PORT 5555
+#define METALIST_ADDR "http://meta.freeciv.org/metaserver/"
 
 struct connection;
 
 int connect_to_server(char *name, char *hostname, int port, char *errbuf);
+void input_from_server(int fd);
 void disconnect_from_server(void);
-void get_net_input(XtPointer client_data, int *fid, XtInputId *id);
-void close_server_connection(void);
-int client_open_connection(char *host, int port);
-void connection_gethostbyaddr(struct connection *pconn, char *desthost);
-int get_meta_list(char *server, char **list, char *errbuf);
 
 extern struct connection aconnection;
 /* this is the client's connection to the server */
 
-#endif
+struct server
+{
+  char *name;
+  char *port;
+  char *version;
+  char *status;
+  char *players;
+  char *metastring;
+};
+
+#define SPECLIST_TAG server
+#define SPECLIST_TYPE struct server
+#include "speclist.h"
+
+#define server_list_iterate(serverlist, pserver) \
+  TYPED_LIST_ITERATE(struct server, serverlist, pserver)
+#define server_list_iterate_end  LIST_ITERATE_END
+
+struct server_list *create_server_list(char *errbuf);
+void delete_server_list(struct server_list *server_list);
+
+#endif  /* FC__CLINET_H */
