@@ -10,79 +10,37 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <player.h>
-#include <unitfunc.h>
-#include <civserver.h>
-#include <map.h>
-#include <maphand.h>
-#include <mapgen.h>
-#include <cityhand.h>
-#include <citytools.h>
-#include <cityturn.h>
-#include <unit.h>
-#include <city.h>
-#include <player.h>
-#include <tech.h>
-#include <shared.h>
-#include <plrhand.h>
-#include <events.h>
-#include <aicity.h>
-#include <settlers.h>
-#include <log.h>
+#include "city.h"
+#include "events.h"
+#include "fcintl.h"
+#include "log.h"
+#include "map.h"
+#include "mem.h"
+#include "player.h"
+#include "shared.h"
+#include "tech.h"
+#include "unit.h"
 
-static struct city **citycache = NULL;
-static int citycachesize = 0;
+#include "citytools.h"
+#include "cityturn.h"
+#include "civserver.h"
+#include "gamelog.h"
+#include "maphand.h"
+#include "plrhand.h"
+#include "settlers.h"
+#include "unitfunc.h"
 
-void initialize_city_cache(void)
-{
-  int i;
+#include "aicity.h"
 
-  if(citycache) free(citycache);
-  citycachesize=128;
-  citycache=malloc(sizeof(*citycache) * citycachesize);
-  for(i=0;i<citycachesize;i++) citycache[i]=NULL;
-}
-
-void reallocate_cache(void)
-{
-  int i;
-
-  flog(LOG_DEBUG,"Increasing max city id index from %d to %d",
-       citycachesize,citycachesize*2);
-  citycachesize*=2;
-  citycache=realloc(citycache,sizeof(*citycache)*citycachesize);
-  for(i=citycachesize/2;i<citycachesize;i++)  citycache[i]=NULL;
-}
-
-void add_city_to_cache(struct city *pcity)
-{
-  if(pcity->id < citycachesize)  {
-    citycache[pcity->id]=pcity;
-  } else {
-    reallocate_cache();
-    add_city_to_cache(pcity);
-  }
-}
-
-void remove_city_from_cache(int id)
-{
-  citycache[id]=NULL;
-}
-
-struct city *find_city_by_id(int id)
-{
-  /* This is sometimes called with id=unit.ai.charge, which is either
-   * a unit id or a city id; if its a unit id then that id won't be used
-   * for a city (?), so that is ok, except that it is possible that it
-   * might exceed citycachesize. --dwp
-   */
-  if(id<0 || id>=citycachesize) return 0;
-  return citycache[id];
-}
+#include "cityhand.h"
 
 /**************************************************************************
 Establish a trade route, notice that there has to be space for them, 
@@ -110,16 +68,55 @@ int establish_trade_route(struct city *pc1, struct city *pc2)
 /* should this be real_map_distance?  Leaving for now -- Syela */
   tb=(tb*(pc1->trade_prod+pc2->trade_prod))/24;
   if (map_get_continent(pc1->x, pc1->y) == map_get_continent(pc2->x, pc2->y))
-    tb*=0.5;
+    tb/=2;
   if (pc1->owner==pc2->owner)
-    tb*=0.5;
-  if (get_invention(city_owner(pc1), A_RAILROAD)==TECH_KNOWN)
-    tb*=0.66;
-  if (get_invention(city_owner(pc1), A_FLIGHT)==TECH_KNOWN)
-    tb*=0.66;
+    tb/=2;
+
+  for(i=0;i<player_knows_techs_with_flag(city_owner(pc1),TF_TRADE_REVENUE_REDUCE);i++) {
+    tb = (tb * 2)/3;
+  }
+  /* was: A_RAILROAD, A_FLIGHT */
   return tb;
 }
 
+/****************************************************************
+...
+*****************************************************************/
+
+char *city_name_suggestion(struct player *pplayer)
+{
+  char **nptr;
+  int i, j, k;
+  static int n_misc=0;
+  static char tempname[100];
+
+  freelog(LOG_VERBOSE, "Suggesting city name for %s", pplayer->name);
+  
+  if (!n_misc) {
+    for (i=0; misc_city_names[i]; i++) {}
+    n_misc = i;
+  }
+
+  for(nptr=get_nation_by_plr(pplayer)->default_city_names; *nptr; nptr++) {
+    if(!game_find_city_by_name(*nptr))
+      return *nptr;
+  }
+
+  j = myrand(n_misc);
+  for (i=0; i<n_misc; i++) {
+    k = (i+j) % n_misc;
+    if (!game_find_city_by_name(misc_city_names[k])) 
+      return misc_city_names[k];
+  }
+
+  for (i = 0; i < 1000;i++ ) {
+    sprintf(tempname, _("city %d"), i);
+    if (!game_find_city_by_name(tempname)) 
+      return tempname;
+  }
+  return "";
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -127,8 +124,11 @@ void create_city(struct player *pplayer, int x, int y, char *name)
 {
   struct city *pcity, *othercity;
   int i;
-/* printf("Creating city %s\n", name);   */
-  pcity=(struct city *)malloc(sizeof(struct city));
+  
+  freelog(LOG_DEBUG, "Creating city %s", name);
+  gamelog(GAMELOG_FOUNDC,"%s (%i, %i) founded by the %s", name, 
+	  x,y, get_nation_name_plural(pplayer->nation));
+  pcity=fc_malloc(sizeof(struct city));
 
   pcity->id=get_next_id_number();
   add_city_to_cache(pcity);
@@ -153,16 +153,7 @@ void create_city(struct player *pplayer, int x, int y, char *name)
   pcity->is_building_unit=1;
   pcity->did_buy=-1; /* code so we get a different message */
   pcity->airlift=0;
-  if (can_build_unit(pcity, U_RIFLEMEN))
-      pcity->currently_building=U_RIFLEMEN;
-  else if (can_build_unit(pcity, U_MUSKETEERS))
-      pcity->currently_building=U_MUSKETEERS;
-  else if (can_build_unit(pcity, U_PIKEMEN))
-      pcity->currently_building=U_PIKEMEN;
-  else if (can_build_unit(pcity, U_PHALANX))
-      pcity->currently_building=U_PHALANX;
-  else
-      pcity->currently_building=U_WARRIORS;
+  pcity->currently_building=best_role_unit(pcity, L_FIRSTBUILD);
 
   for (y = 0; y < CITY_MAP_SIZE; y++)
     for (x = 0; x < CITY_MAP_SIZE; x++)
@@ -175,6 +166,9 @@ void create_city(struct player *pplayer, int x, int y, char *name)
     pcity->improvements[B_PALACE]=1;
   }
   pcity->anarchy=0;
+
+  pcity->city_options = CITYOPT_DEFAULT;
+  
   pcity->ai.ai_role = AICITY_NONE;
   pcity->ai.trade_want = 8; /* default value */
   memset(pcity->ai.building_want, 0, sizeof(pcity->ai.building_want));
@@ -254,15 +248,13 @@ void handle_city_change_specialist(struct player *pplayer,
     return;
   }
   switch (preq->specialist_to) {
-  case SP_ELVIS:
-    pcity->ppl_elvis++;
-    break;
   case SP_TAXMAN:
     pcity->ppl_taxman++;
     break;
   case SP_SCIENTIST:
     pcity->ppl_scientist++;
     break;
+  case SP_ELVIS:
   default:
     pcity->ppl_elvis++;
     break;
@@ -297,7 +289,7 @@ void handle_city_make_specialist(struct player *pplayer,
     send_city_info(pplayer, pcity, 0);
   } else {
     notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT,
-		     "Game: you don't have a worker here"); 
+		     _("Game: you don't have a worker here.")); 
   }
 }
 
@@ -363,7 +355,7 @@ void really_handle_city_sell(struct player *pplayer, struct city *pcity, int id)
 {  
   if (pcity->did_sell) {
     notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT, 
-		  "Game: You have already sold something here this turn.");
+		  _("Game: You have already sold something here this turn."));
     return;
   }
 
@@ -372,7 +364,7 @@ void really_handle_city_sell(struct player *pplayer, struct city *pcity, int id)
 
   pcity->did_sell=1;
   notify_player_ex(pplayer, pcity->x, pcity->y, E_IMP_SOLD,
-		   "Game: You sell %s in %s for %d credits.", 
+		   _("Game: You sell %s in %s for %d credits."), 
 		   get_improvement_name(id), pcity->name,
 		   improvement_value(id));
   do_sell_building(pplayer, pcity, id);
@@ -400,32 +392,39 @@ void really_handle_city_buy(struct player *pplayer, struct city *pcity)
  
   if (pcity->did_buy > 0) {
     notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT,
-		  "Game: You have already bought this turn.");
+		  _("Game: You have already bought this turn."));
     return;
   }
 
   if (pcity->did_buy < 0) {
     notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT,
-		  "Game: Cannot buy in city created this turn.");
+		  _("Game: Cannot buy in city created this turn."));
     return;
   }
 
-  if (!pcity->is_building_unit) {
-    total=improvement_value(pcity->currently_building);
-    name=get_improvement_name(pcity->currently_building);
-    
-  } else {
+  if (!pcity->is_building_unit && pcity->currently_building==B_CAPITAL)  {
+    notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT,
+                     _("Game: You don't buy %s!"),
+		     improvement_types[B_CAPITAL].name);
+    return;
+  }
+
+  if (pcity->is_building_unit && pcity->anarchy) {
+    notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT, 
+		     _("Game: Can't buy units when city is in disorder."));
+    return;
+  }
+
+  if (pcity->is_building_unit) {
     name=unit_types[pcity->currently_building].name;
     total=unit_value(pcity->currently_building);
-    if (pcity->anarchy) {
-      notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT, 
-		    "Game: Can't buy units when city is in disorder.");
-      return;
-    }
+  } else {
+    name=get_improvement_name(pcity->currently_building);
+    total=improvement_value(pcity->currently_building);
   }
   cost=city_buy_cost(pcity);
-   if (cost>pplayer->economic.gold)
-    return;
+  if ((!cost) || (cost>pplayer->economic.gold))
+   return;
 
   pplayer->economic.gold-=cost;
   if (pcity->shield_stock < total){
@@ -437,7 +436,7 @@ void really_handle_city_buy(struct player *pplayer, struct city *pcity)
   connection_do_buffer(pplayer->conn);
   notify_player_ex(pplayer, pcity->x, pcity->y, 
                    pcity->is_building_unit?E_UNIT_BUY:E_IMP_BUY,
-		   "Game: %s bought in %s for %d gold.", 
+		   _("Game: %s bought in %s for %d gold."), 
 		   name, pcity->name, cost);
   send_city_info(pplayer, pcity, 1);
   send_player_info(pplayer,pplayer);
@@ -473,7 +472,8 @@ void handle_city_change(struct player *pplayer,
   struct city *pcity;
   pcity=find_city_by_id(preq->city_id);
   if (!pcity) {
-    printf("Pcity null in handle_city_change (%s, id = %d)!\n", pplayer->name, preq->city_id);
+    freelog(LOG_NORMAL, "Pcity null in handle_city_change"
+	                " (%s, id = %d)!", pplayer->name, preq->city_id);
     return;
   }
    if(!player_owns_city(pplayer, pcity))
@@ -484,14 +484,14 @@ void handle_city_change(struct player *pplayer,
      return;
   if (pcity->did_buy && pcity->shield_stock) { /* did_buy > 0 should be same -- Syela */
     notify_player_ex(pplayer, pcity->x, pcity->y, E_NOEVENT,
-		     "Game: You have bought this turn, can't change.");
+		     _("Game: You have bought this turn, can't change."));
     return;
   }
 
    if(!pcity->is_building_unit && is_wonder(pcity->currently_building)) {
      notify_player_ex(0, pcity->x, pcity->y, E_WONDER_STOPPED,
-		   "Game: The %s have stopped building The %s in %s.",
-		   get_race_name_plural(pplayer->race),
+		   _("Game: The %s have stopped building The %s in %s."),
+		   get_nation_name_plural(pplayer->nation),
 		   get_imp_name_ex(pcity, pcity->currently_building),
 		   pcity->name);
    }
@@ -511,9 +511,10 @@ void handle_city_change(struct player *pplayer,
     
     if(is_wonder(preq->build_id)) {
       notify_player_ex(0, pcity->x, pcity->y, E_WONDER_STARTED,
-		       "Game: The %s have started building The %s.",
-		       get_race_name_plural(pplayer->race),
-		       get_imp_name_ex(pcity, pcity->currently_building));
+		       _("Game: The %s have started building The %s in %s."),
+		       get_nation_name_plural(pplayer->nation),
+		       get_imp_name_ex(pcity, pcity->currently_building),
+		       pcity->name);
     }
   }
   
@@ -542,7 +543,36 @@ void handle_city_rename(struct player *pplayer,
     send_city_info(pplayer, pcity, 1);
   }
   else
-    notify_player(pplayer, "Game: %s is not a valid name.", preq->name);
+    notify_player(pplayer, _("Game: %s is not a valid name."), preq->name);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void handle_city_options(struct player *pplayer,
+				struct packet_generic_values *preq)
+{
+  struct city *pcity = find_city_by_id(preq->value1);
+  if (!pcity || pcity->owner != pplayer->player_no) return;
+  pcity->city_options = preq->value2;
+  /* We don't need to send the full city info, since no other properties
+   * depend on the attack options. --dwp
+   * Otherwise could do:
+   *   send_city_info(pplayer, pcity, 1);
+   */
+  send_packet_generic_values(pplayer->conn, PACKET_CITY_OPTIONS, preq);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void handle_city_name_suggest_req(struct player *pplayer,
+				  struct packet_generic_integer *packet)
+{
+  struct packet_city_name_suggestion reply;
+  reply.id = packet->value;
+  strcpy(reply.name, city_name_suggestion(pplayer));
+  send_packet_city_name_suggestion(pplayer->conn, &reply);
 }
 
 /**************************************************************************
@@ -635,6 +665,8 @@ void send_city_info(struct player *dest, struct city *pcity, int dosend)
   packet.food_stock=pcity->food_stock;
   packet.shield_stock=pcity->shield_stock;
   packet.pollution=pcity->pollution;
+
+  packet.city_options=pcity->city_options;
   
   packet.is_building_unit=pcity->is_building_unit;
   packet.currently_building=pcity->currently_building;
@@ -698,15 +730,18 @@ void remove_city(struct city *pcity)
 {
   int o, x, y;
   struct packet_generic_integer packet;
+  gamelog(GAMELOG_LOSEC,"%s lose %s (%i,%i)",
+          get_nation_name_plural(game.players[pcity->owner].nation),
+          pcity->name,pcity->x,pcity->y);
+  while(unit_list_size(&pcity->units_supported)) {
+    wipe_unit(0, unit_list_get(&pcity->units_supported,0));
+  }
   for (o=0; o<4; o++)
-    remove_trade_route(pcity->trade[0], pcity->id);
+    remove_trade_route(pcity->trade[o], pcity->id); 
   packet.value=pcity->id;
   for(o=0; o<game.nplayers; o++)           /* dests */
     send_packet_generic_integer(game.players[o].conn,
 				PACKET_REMOVE_CITY,&packet);
-  while(unit_list_size(&pcity->units_supported)) {
-    wipe_unit(0, unit_list_get(&pcity->units_supported,0));
-  }
   remove_city_from_cache(pcity->id);
   dealloc_id(pcity->id);
   x = pcity->x; y = pcity->y;
@@ -714,3 +749,39 @@ void remove_city(struct city *pcity)
   game_remove_city(pcity);
   reset_move_costs(x, y);
 }
+
+
+/**************************************************************************
+The following has to be called every time a city, pcity, has changed 
+owner to update the tile->worked pointer.
+**************************************************************************/
+
+void update_map_with_city_workers(struct city *pcity)
+{
+  int x, y;
+    city_map_iterate(x,y) {
+	if (pcity->city_map[x][y] == C_TILE_WORKER)
+	       set_worker_city(pcity, x, y, C_TILE_WORKER);
+       }
+}
+
+/**************************************************************************
+The following has to be called every time a city, pcity, has changed
+owner to update the city's traderoutes.
+**************************************************************************/
+
+void reestablish_city_trade_routes(struct city *pcity) 
+{
+  int i;
+  struct city *oldtradecity;
+
+  for (i=0;i<4;i++) {
+    if (pcity->trade[i]) {
+      oldtradecity=find_city_by_id(pcity->trade[i]);
+      pcity->trade[i]=0;
+      if (can_establish_trade_route(pcity, oldtradecity)) {   
+         establish_trade_route(pcity, oldtradecity);
+      }
+    }
+  }
+}
