@@ -17,38 +17,47 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 
-#include <civserver.h>
-#include <registry.h>
-#include <game.h>
-#include <gamehand.h>
-#include <unitfunc.h>
-#include <cityturn.h>
-#include <maphand.h>
-#include <plrhand.h>
-#include <packets.h>
-#include <map.h>
-#include <meta.h>
-#include <capability.h>
-#include <time.h>
-#include <log.h>
+#include "capability.h"
+#include "game.h"
+#include "log.h"
+#include "map.h"
+#include "mem.h"
+#include "packets.h"
+#include "registry.h"
+#include "version.h"
+
+#include "cityturn.h"
+#include "civserver.h"
+#include "maphand.h"
+#include "meta.h"
+#include "plrhand.h"
+#include "ruleset.h"
+#include "unitfunc.h"
+
+#include "gamehand.h"
 
 extern char metaserver_info_line[];
+extern char metaserver_addr[];
 extern RANDOM_TYPE RandomState[];
 extern int iRandJ, iRandK, iRandX; 
 extern int rand_init;
 
-#define SAVEFILE_OPTIONS "1.7, startoptions, unirandom"
+#define SAVEFILE_OPTIONS "1.7 startoptions unirandom spacerace2 rulesets"
 
 /**************************************************************************
 ...
 **************************************************************************/
 void init_new_game(void)
 {
-  int i, x, y,j;
+  int i, j, x, y;
+  int start_pos[MAX_NUM_PLAYERS]; /* indices into map.start_positions[] */
+  
   if (game.scenario!=1 && game.scenario!=2) {
     /* except in a scenario which provides them,
        shuffle the start positions around... */
+    assert(game.nplayers==map.num_start_positions);
     for (i=0; i<game.nplayers;i++) { /* no advantage to the romans!! */
       j=myrand(game.nplayers);
       x=map.start_positions[j].x;
@@ -59,21 +68,59 @@ void init_new_game(void)
       map.start_positions[i].y=y;
     }
   }
-  for(i=0; i<game.nplayers; i++) {
-    if(game.scenario==1 || game.scenario==2) {
-      /* in a scenario, choose starting positions by race */
-      x=map.start_positions[game.players[i].race].x;
-      y=map.start_positions[game.players[i].race].y;
-    } else {
-      x=map.start_positions[i].x;
-      y=map.start_positions[i].y;
+  /* In a scenario, choose starting positions by nation.
+     If there are too few starts for number of nations, assign
+     to nations with specific starts first, then assign rest
+     to random from remainder.  (Would be better to label start
+     positions by nation etc, but this will do for now. --dwp)
+  */
+  if(game.scenario!=1 && game.scenario!=2) {
+    for(i=0; i<game.nplayers; i++) {
+      start_pos[i] = i;
+    } 
+  } else {
+    const int npos = map.num_start_positions;
+    int *pos_used = fc_calloc(npos, sizeof(int));
+    int nrem = npos;		/* remaining unused starts */
+    
+    for(i=0; i<game.nplayers; i++) {
+      int nation = game.players[i].nation;
+      if (nation < npos) {
+	start_pos[i] = nation;
+	pos_used[nation] = 1;
+	nrem--;
+      } else {
+	start_pos[i] = npos;
+      }
     }
+    for(i=0; i<game.nplayers; i++) {
+      if (start_pos[i] == npos) {
+	int k;
+	assert(nrem>0);
+	k = myrand(nrem);
+	for(j=0; j<npos; j++) {
+	  if (!pos_used[j] && (0==k--)) {
+	    start_pos[i] = j;
+	    pos_used[j] = 1;
+	    nrem--;
+	    break;
+	  }
+	}
+	assert(start_pos[i] != npos);
+      }
+    }
+    free(pos_used);
+  }
+  
+  for(i=0; i<game.nplayers; i++) {
+    x=map.start_positions[start_pos[i]].x;
+    y=map.start_positions[start_pos[i]].y;
     /* For scenarios, huts may coincide with player starts;
        remove any such hut: */
     if(map_get_special(x,y)&S_HUT) {
       map_clear_special(x,y,S_HUT);
-      flog(LOG_DEBUG, "Removed hut on start position for %s",
-	  game.players[i].name);
+      freelog(LOG_VERBOSE, "Removed hut on start position for %s",
+	      game.players[i].name);
     }
     /* Civ 1 exposes a single square radius of the map to start the game. */
     /* Civ 2 exposes a "city radius". -AJS */
@@ -85,9 +132,9 @@ void init_new_game(void)
       light_square(&game.players[i], x, y+1, 1);
     }
     for (j=0;j<game.settlers;j++) 
-      create_unit(&game.players[i], x, y, U_SETTLERS, 0, 0, -1);
+      create_unit(&game.players[i], x, y, get_role_unit(F_SETTLERS,0), 0, 0, -1);
     for (j=0;j<game.explorer;j++) 
-      create_unit(&game.players[i], x, y, U_EXPLORER, 0, 0, -1);
+      create_unit(&game.players[i], x, y, get_role_unit(L_EXPLORER,0), 0, 0, -1);
   }
 }
 
@@ -151,10 +198,8 @@ void send_game_info(struct player *dest)
   ginfo.techpenalty=game.techpenalty;
   ginfo.foodbox = game.foodbox;
   ginfo.civstyle=game.civstyle;
-  ginfo.rail_food = game.rail_food;
-  ginfo.rail_trade = game.rail_trade;
-  ginfo.rail_prod = game.rail_prod;
-  for(i=0; i<A_LAST; i++)
+  ginfo.spacerace = game.spacerace;
+  for(i=0; i<A_LAST/*game.num_tech_types*/; i++)
     ginfo.global_advances[i]=game.global_advances[i];
   for(i=0; i<B_LAST; i++)
     ginfo.global_wonders[i]=game.global_wonders[i];
@@ -171,7 +216,7 @@ void send_game_info(struct player *dest)
  Assign values to ord_city and ord_map for each unit, so the
  values can be saved.
 ***************************************************************/
-void calc_unit_ordering(void)
+static void calc_unit_ordering(void)
 {
   int i, j, x, y;
   
@@ -202,7 +247,7 @@ void calc_unit_ordering(void)
  For each city and tile, sort unit lists according to
  ord_city and ord_map values.
 ***************************************************************/
-void apply_unit_ordering(void)
+static void apply_unit_ordering(void)
 {
   int i, x, y;
   
@@ -236,6 +281,9 @@ int game_load(struct section_file *file)
   strcpy(metaserver_info_line,
 	 secfile_lookup_str_default(file, DEFAULT_META_SERVER_INFO_STRING,
 				    "game.metastring"));
+  strcpy(metaserver_addr,
+	 secfile_lookup_str_default(file, DEFAULT_META_SERVER_ADDR,
+				    "game.metaserver"));
   
   game.gold          = secfile_lookup_int(file, "game.gold");
   game.tech          = secfile_lookup_int(file, "game.tech");
@@ -255,16 +303,21 @@ int game_load(struct section_file *file)
 
   if (game.version >= 10100) {
     game.cityfactor = secfile_lookup_int(file, "game.cityfactor");
+    game.civilwarsize =
+      secfile_lookup_int_default(file, GAME_DEFAULT_CIVILWARSIZE,
+				 "game.civilwarsize");
     game.diplcost   = secfile_lookup_int(file, "game.diplcost");
     game.freecost   = secfile_lookup_int(file, "game.freecost");
     game.conquercost   = secfile_lookup_int(file, "game.conquercost");
-    game.rail_food  = secfile_lookup_int(file, "game.rail_food");
-    game.rail_prod  = secfile_lookup_int(file, "game.rail_prod");
-    game.rail_trade = secfile_lookup_int(file, "game.rail_trade");
-
     game.foodbox     = secfile_lookup_int(file, "game.foodbox");
     game.techpenalty = secfile_lookup_int(file, "game.techpenalty");
     game.razechance  = secfile_lookup_int(file, "game.razechance");
+
+    /* suppress warnings about unused entries in old savegames: */
+    section_file_lookup(file, "game.rail_food");
+    section_file_lookup(file, "game.rail_prod");
+    section_file_lookup(file, "game.rail_trade");
+    section_file_lookup(file, "game.farmfood");
   }
   if (game.version >= 10300) {
     game.civstyle = secfile_lookup_int(file, "game.civstyle");
@@ -290,6 +343,8 @@ int game_load(struct section_file *file)
 					       "game.diplchance");
   game.aqueductloss = secfile_lookup_int_default(file, game.aqueductloss,
 						 "game.aqueductloss");
+  game.turnblock = secfile_lookup_int_default(file,game.turnblock,
+                         "game.turnblock");
   
   if(has_capability("unirandom", savefile_options)) {
     game.randseed = secfile_lookup_int(file, "game.randseed");
@@ -306,25 +361,49 @@ int game_load(struct section_file *file)
     rand_init=1;
   }
 
+  strcpy(game.ruleset.techs,
+	 secfile_lookup_str_default(file, "default", "game.ruleset.techs"));
+  strcpy(game.ruleset.units,
+	 secfile_lookup_str_default(file, "default", "game.ruleset.units"));
+  strcpy(game.ruleset.buildings,
+	 secfile_lookup_str_default(file, "default", "game.ruleset.buildings"));
+  strcpy(game.ruleset.terrain,
+	 secfile_lookup_str_default(file, "classic", "game.ruleset.terrain"));
+  strcpy(game.ruleset.governments,
+	 secfile_lookup_str_default(file, "default",
+				    "game.ruleset.governments"));
+  strcpy(game.ruleset.nations,
+         secfile_lookup_str_default(file, "default", "game.ruleset.nations"));
+  strcpy(game.ruleset.cities,
+         secfile_lookup_str_default(file, "default", "game.ruleset.cities"));
+
+
+  strcpy(game.demography,
+	 secfile_lookup_str_default(file, GAME_DEFAULT_DEMOGRAPHY,
+				    "game.demography"));
+
+  game.spacerace = secfile_lookup_int_default(file, game.spacerace,
+					      "game.spacerace");
+
   game.heating=0;
   if(tmp_server_state==PRE_GAME_STATE 
      || has_capability("startoptions", savefile_options)) {
     if (game.version >= 10300) {
       game.settlers = secfile_lookup_int(file, "game.settlers");
       game.explorer = secfile_lookup_int(file, "game.explorer");
-      
+
       map.riches = secfile_lookup_int(file, "map.riches");
       map.huts = secfile_lookup_int(file, "map.huts");
       map.generator = secfile_lookup_int(file, "map.generator");
-      if (tmp_server_state==PRE_GAME_STATE &&
-	  map.generator == 0) { /* generator 0 = map done with map editor */
-        map_tiles_load(file);
-        if (has_capability("startpos",savefile_options)) {
-          map_startpos_load(file);
-          return 2; /* make this a scenario */
-        }
-	return 3; /* make this some kind of scenario */
-      }
+      map.seed = secfile_lookup_int(file, "map.seed");
+      map.landpercent = secfile_lookup_int(file, "map.landpercent");
+      map.grasssize = secfile_lookup_int_default(file, MAP_DEFAULT_GRASS, "map.grasssize");
+      map.swampsize = secfile_lookup_int(file, "map.swampsize");
+      map.deserts = secfile_lookup_int(file, "map.deserts");
+      map.riverlength = secfile_lookup_int(file, "map.riverlength");
+      map.mountains = secfile_lookup_int(file, "map.mountains");
+      map.forestsize = secfile_lookup_int(file, "map.forestsize");
+
       if (has_capability("startoptions", savefile_options)) {
 	map.xsize = secfile_lookup_int(file, "map.width");
 	map.ysize = secfile_lookup_int(file, "map.height");
@@ -333,28 +412,52 @@ int game_load(struct section_file *file)
 	map.xsize = secfile_lookup_int(file, "map.xsize");
 	map.ysize = secfile_lookup_int(file, "map.ysize");
       }
-      map.seed = secfile_lookup_int(file, "map.seed");
-      map.landpercent = secfile_lookup_int(file, "map.landpercent");
-      map.swampsize = secfile_lookup_int(file, "map.swampsize");
-      map.deserts = secfile_lookup_int(file, "map.deserts");
-      map.riverlength = secfile_lookup_int(file, "map.riverlength");
-      map.mountains = secfile_lookup_int(file, "map.mountains");
-      map.forestsize = secfile_lookup_int(file, "map.forestsize");
+
+      if (tmp_server_state==PRE_GAME_STATE &&
+	  map.generator == 0) { /* generator 0 = map done with map editor */
+        if (has_capability("specials",savefile_options)) {
+          map_load(file);
+          section_file_check_unused(file, NULL);
+          section_file_free(file);
+          return 1; /* make this a type 1 scenario */
+        }
+        map_tiles_load(file);
+        if (has_capability("riversoverlay",savefile_options)) {
+	  map_rivers_overlay_load(file);
+	}
+        if (has_capability("startpos",savefile_options)) {
+          map_startpos_load(file);
+          return 2; /* make this a type 2 scenario */
+        }
+	return 3; /* make this a type 3 scenario */
+      }
     }
     if(tmp_server_state==PRE_GAME_STATE) {
       return 0;
     }
   }
+
+  load_rulesets();
+
   map_load(file);
 
+  /* destroyed wonders: */
+  string = secfile_lookup_str_default(file, "", "game.destroyed_wonders");
+  for(i=0; i<B_LAST && string[i]; i++) {
+    if (string[i] == '1') {
+      game.global_wonders[i] = -1; /* destroyed! */
+    }
+  }
+  
   for(i=0; i<game.nplayers; i++) {
     player_load(&game.players[i], i, file); 
   }
+  
   initialize_globals();
   apply_unit_ordering();
 
   for(i=0; i<game.nplayers; i++) {
-    /*    player_load(&game.players[i], i, file); */
+    update_research(&game.players[i]);
     city_list_iterate(game.players[i].cities, pcity)
       city_refresh(pcity);
     city_list_iterate_end;
@@ -374,15 +477,39 @@ void game_save(struct section_file *file)
 {
   int i;
   int version;
-  char name[20],temp[100];
+  char name[20],temp[100], temp1[100], *temp2;
 
   version = MAJOR_VERSION *10000 + MINOR_VERSION *100 + PATCH_VERSION; 
   secfile_insert_int(file, version, "game.version");
   secfile_insert_int(file, (int) server_state, "game.server_state");
   secfile_insert_str(file, metaserver_info_line, "game.metastring");
-  secfile_insert_str(file, SAVEFILE_OPTIONS, "savefile.options");
+  secfile_insert_str(file, metaserver_addr, "game.metaserver");
+
+  if(server_state!=PRE_GAME_STATE) {
+    secfile_insert_str(file, SAVEFILE_OPTIONS, "savefile.options");
+  } else { /* cut out unirandom, and insert startpos if necessary */
+    strcpy(temp, SAVEFILE_OPTIONS);
+    temp2=strtok(temp," ");
+    *temp1='\0';
+    while(temp2 != NULL) {
+      /* we don't have unirandom in settings and scenarios */
+      if(strcmp(temp2,"unirandom")!=0) {
+        strcat(temp1," ");
+        strcat(temp1,temp2);
+      }
+      temp2=strtok(NULL," ");
+    }
+    if(game.scenario==2) {
+      /* we have start positions in type 2 scenarios */
+      strcat(temp1," startpos");
+    } else
+    if(game.scenario==1) {
+      /* we have start positions and specials in type 1 scenarios */
+      strcat(temp1," specials");
+    }
+    secfile_insert_str(file, temp1+1, "savefile.options");
+  }
 
-  
   secfile_insert_int(file, game.gold, "game.gold");
   secfile_insert_int(file, game.tech, "game.tech");
   secfile_insert_int(file, game.skill_level, "game.skill_level");
@@ -397,12 +524,10 @@ void game_save(struct section_file *file)
   secfile_insert_int(file, game.warminglevel, "game.warminglevel");
   secfile_insert_int(file, game.unhappysize, "game.unhappysize");
   secfile_insert_int(file, game.cityfactor, "game.cityfactor");
+  secfile_insert_int(file, game.civilwarsize, "game.civilwarsize");
   secfile_insert_int(file, game.diplcost, "game.diplcost");
   secfile_insert_int(file, game.freecost, "game.freecost");
   secfile_insert_int(file, game.conquercost, "game.conquercost");
-  secfile_insert_int(file, game.rail_food, "game.rail_food");
-  secfile_insert_int(file, game.rail_prod, "game.rail_prod");
-  secfile_insert_int(file, game.rail_trade, "game.rail_trade");
   secfile_insert_int(file, game.foodbox, "game.foodbox");
   secfile_insert_int(file, game.techpenalty, "game.techpenalty");
   secfile_insert_int(file, game.razechance, "game.razechance");
@@ -410,9 +535,19 @@ void game_save(struct section_file *file)
   secfile_insert_int(file, game.save_nturns, "game.save_nturns");
   secfile_insert_int(file, game.aifill, "game.aifill");
   secfile_insert_int(file, game.scorelog, "game.scorelog");
+  secfile_insert_int(file, game.spacerace, "game.spacerace");
   secfile_insert_int(file, game.diplchance, "game.diplchance");
   secfile_insert_int(file, game.aqueductloss, "game.aqueductloss");
   secfile_insert_int(file, game.randseed, "game.randseed");
+  secfile_insert_int(file, game.turnblock, "game.turnblock");
+  secfile_insert_str(file, game.ruleset.techs, "game.ruleset.techs");
+  secfile_insert_str(file, game.ruleset.units, "game.ruleset.units");
+  secfile_insert_str(file, game.ruleset.buildings, "game.ruleset.buildings");
+  secfile_insert_str(file, game.ruleset.terrain, "game.ruleset.terrain");
+  secfile_insert_str(file, game.ruleset.governments, "game.ruleset.governments");
+  secfile_insert_str(file, game.ruleset.nations, "game.ruleset.nations");
+  secfile_insert_str(file, game.ruleset.cities, "game.ruleset.cities");
+  secfile_insert_str(file, game.demography, "game.demography");
 
   if (1) {
     /* Now always save these, so the server options reflect the
@@ -435,10 +570,10 @@ void game_save(struct section_file *file)
     secfile_insert_int(file, map.huts, "map.huts");
     secfile_insert_int(file, map.generator, "map.generator");
   } 
-  if (server_state==PRE_GAME_STATE) {
-    return;
-  } else {
-
+  if ((server_state==PRE_GAME_STATE) && (game.scenario==0)) {
+    return; /* want to save scenarios as well */
+  }
+  if (server_state!=PRE_GAME_STATE) {
     secfile_insert_int(file,iRandJ,"random.index_J");
     secfile_insert_int(file,iRandK,"random.index_K");
     secfile_insert_int(file,iRandX,"random.index_X");
@@ -453,6 +588,18 @@ void game_save(struct section_file *file)
      
   }
 
+  /* destroyed wonders: */
+  for(i=0; i<B_LAST; i++) {
+    if (is_wonder(i) && game.global_wonders[i]!=0
+	&& !find_city_by_id(game.global_wonders[i])) {
+      temp[i] = '1';
+    } else {
+      temp[i] = '0';
+    }
+  }
+  temp[i] = '\0';
+  secfile_insert_str(file, temp, "game.destroyed_wonders");
+
   calc_unit_ordering();
   
   for(i=0; i<game.nplayers; i++)
