@@ -15,14 +15,16 @@
 #include <assert.h>
 #include <string.h>
 
-#include <map.h>
-#include <game.h>
-#include <unitfunc.h>
-#include <unithand.h>
-#include <unittools.h>
-#include <gotohand.h>
-#include <settlers.h>
-#include <log.h>
+#include "game.h"
+#include "log.h"
+#include "map.h"
+
+#include "settlers.h"
+#include "unitfunc.h"
+#include "unithand.h"
+#include "unittools.h"
+
+#include "gotohand.h"
 
 struct move_cost_map warmap;
 
@@ -34,7 +36,8 @@ struct move_cost_map warmap;
    
 struct stack_element {
   unsigned char x, y;
-} warstack[WARSTACK_DIM];
+};
+static struct stack_element warstack[WARSTACK_DIM];
 
 /* This wastes ~20K of memory and should really be fixed but I'm lazy  -- Syela
  *
@@ -44,8 +47,8 @@ struct stack_element {
  * smaller costs.  --dwp
  */
 
-unsigned int warstacksize;
-unsigned int warnodes;
+static unsigned int warstacksize;
+static unsigned int warnodes;
 
 static void add_to_stack(int x, int y)
 {
@@ -63,34 +66,21 @@ static void get_from_warstack(unsigned int i, int *x, int *y)
   *y = warstack[i].y;
 }
 
-void init_warmap(int orig_x, int orig_y, enum unit_move_type which)
+static void init_warmap(int orig_x, int orig_y, enum unit_move_type which)
 {
-  int x, y, i, j;
-  int maxcost = THRESHOLD * 6 + 2; /* should be big enough without being TOO big */
-  for (x = 0; x < map.xsize; x++) {
-    if (x > orig_x)
-      i = MIN(x - orig_x, orig_x + map.xsize - x);
-    else
-      i = MIN(orig_x - x, x + map.xsize - orig_x);      
-    for (y = 0; y < map.ysize; y++) {
-      if (y > orig_y)
-        j = MAX(y - orig_y, i);
-      else
-        j = MAX(orig_y - y, i);
-      j *= 3;
-      if (j < maxcost) j = maxcost;
-      if (j > 255) j = 255;
-      if (which == LAND_MOVING) warmap.cost[x][y] = j; /* one if by land */
-      else warmap.seacost[x][y] = j;
-    }
-  }
+  int x, y;
+  for (x = 0; x < map.xsize; x++) 
+    for (y = 0; y < map.ysize; y++) 
+      if (which == LAND_MOVING) warmap.cost[x][y] = 255; /* one if by land */
+      else warmap.seacost[x][y] = 255;
+      /* why a seacost and a landcost nb */
   if (which == LAND_MOVING) warmap.cost[orig_x][orig_y] = 0;
   else warmap.seacost[orig_x][orig_y] = 0;
 }  
 
 void really_generate_warmap(struct city *pcity, struct unit *punit, enum unit_move_type which)
 { /* let generate_warmap interface to this function */
-  int x, y, c, i, j, k, xx[3], yy[3], tm;
+  int x, y, c, k, xx[3], yy[3], x1, y1, tm;
   int orig_x, orig_y;
   int igter = 0;
   int ii[8] = { 0, 1, 2, 0, 2, 0, 1, 2 };
@@ -116,24 +106,22 @@ void really_generate_warmap(struct city *pcity, struct unit *punit, enum unit_mo
   add_to_stack(orig_x, orig_y);
 
   if (punit && unit_flag(punit->type, F_IGTER)) igter++;
-  if (punit && punit->type == U_SETTLERS) maxcost >>= 1;
+  if (punit && unit_flag(punit->type, F_SETTLERS)
+      && get_unit_type(punit->type)->move_rate==3) maxcost /= 2;
+  /* (?) was punit->type == U_SETTLERS -- dwp */
 
   do {
     get_from_warstack(warnodes, &x, &y);
     warnodes++; /* for debug purposes */
     tile0 = map_get_tile(x, y);
-    if((xx[2]=x+1)==map.xsize) xx[2]=0;
-    if((xx[0]=x-1)==-1) xx[0]=map.xsize-1;
-    xx[1] = x;
-    if ((yy[0]=y-1)==-1) yy[0] = 0;
-    if ((yy[2]=y+1)==map.ysize) yy[2]=y;
-    yy[1] = y;
+    map_calc_adjacent_xy(x, y, xx, yy);
     for (k = 0; k < 8; k++) {
-      i = ii[k]; j = jj[k]; /* saves CPU cycles? */
+      x1 = xx[ii[k]];
+      y1 = yy[jj[k]];
       if (which == LAND_MOVING) {
 /*        if (tile0->move_cost[k] == -3 || tile0->move_cost[k] > 16) c = maxcost;*/
-        if (map_get_terrain(xx[i], yy[j]) == T_OCEAN) {
-          if (punit && is_transporter_with_free_space(pplayer, xx[i], yy[j])) c = 3;
+        if (map_get_terrain(x1, y1) == T_OCEAN) {
+          if (punit && is_transporter_with_free_space(pplayer, x1, y1)) c = 3;
           else c = maxcost;
         } else if (tile0->terrain == T_OCEAN) c = 3;
         else if (igter) c = 3; /* NOT c = 1 */
@@ -141,37 +129,39 @@ void really_generate_warmap(struct city *pcity, struct unit *punit, enum unit_mo
 /*        else c = tile0->move_cost[k]; 
 This led to a bad bug where a unit in a swamp was considered too far away */
         else {
-          tm = map_get_tile(xx[i], yy[j])->move_cost[7-k];
-          c = (tile0->move_cost[k] + tm + (tile0->move_cost[k] > tm ? 1 : 0))>>1;
+          tm = map_get_tile(x1, y1)->move_cost[7-k];
+          c = (tile0->move_cost[k] + tm + (tile0->move_cost[k] > tm ? 1 : 0))/2;
         }
         
         tm = warmap.cost[x][y] + c;
-        if (warmap.cost[xx[i]][yy[j]] > tm && tm < maxcost) {
-          warmap.cost[xx[i]][yy[j]] = tm;
-          add_to_stack(xx[i], yy[j]);
+        if (warmap.cost[x1][y1] > tm && tm < maxcost) {
+          warmap.cost[x1][y1] = tm;
+          add_to_stack(x1, y1);
         }
       } else {
         c = 3; /* allow for shore bombardment/transport/etc */
         tm = warmap.seacost[x][y] + c;
-        if (warmap.seacost[xx[i]][yy[j]] > tm && tm < maxcost) {
-          warmap.seacost[xx[i]][yy[j]] = tm;
-          if (tile0->move_cost[k] == -3) add_to_stack(xx[i], yy[j]);
+        if (warmap.seacost[x1][y1] > tm && tm < maxcost) {
+          warmap.seacost[x1][y1] = tm;
+          if (tile0->move_cost[k] == -3) add_to_stack(x1, y1);
         }
       }
     } /* end for */
   } while (warstacksize > warnodes);
   if (warnodes > WARSTACK_DIM) {
-    flog(LOG_DEBUG, "Warning: %u nodes in map #%d for (%d, %d)",
+    freelog(LOG_VERBOSE, "Warning: %u nodes in map #%d for (%d, %d)",
 	 warnodes, which, orig_x, orig_y);
   }
-/* printf("Generated warmap for (%d,%d) with %u nodes checked.\n", orig_x, orig_y, warnodes); */
-/* warnodes is often as much as 2x the size of the continent -- Syela */
+  freelog(LOG_DEBUG, "Generated warmap for (%d,%d) with %u nodes checked.",
+	  orig_x, orig_y, warnodes); 
+  /* warnodes is often as much as 2x the size of the continent -- Syela */
 }
 
 void generate_warmap(struct city *pcity, struct unit *punit)
 {
-/*printf("Generating warmap, pcity = %s, punit = %s\n", (pcity ?
-pcity->name : "NULL"), (punit ? unit_types[punit->type].name : "NULL"));*/
+  freelog(LOG_DEBUG, "Generating warmap, pcity = %s, punit = %s",
+	  (pcity ? pcity->name : "NULL"),
+	  (punit ? unit_types[punit->type].name : "NULL"));
 
   if (punit) {
     if (pcity && pcity == warmap.warcity) return; /* time-saving shortcut */
@@ -201,16 +191,16 @@ pcity->name : "NULL"), (punit ? unit_types[punit->type].name : "NULL"));*/
 }
 
 /* ....... end of old advmilitary.c, beginning of old gotohand.c. ..... */
-int dir_ok(int x0, int y0, int x1, int y1, int k)
+static int dir_ok(int x0, int y0, int x1, int y1, int k)
 { /* The idea of this is to check less nodes in the wrong direction.
 These if's might cost some CPU but hopefully less overall. -- Syela */
   int n = 0, s = 0, e = 0, w = 0, dx;
   if (y1 > y0) s = y1 - y0;
   else n = y0 - y1;
   dx = x1 - x0;
-  if (dx > map.xsize>>1) w = map.xsize - dx;
+  if (dx > map.xsize/2) w = map.xsize - dx;
   else if (dx > 0) e = dx;
-  else if (dx + (map.xsize>>1) > 0) w = 0 - dx;
+  else if (dx + (map.xsize/2) > 0) w = 0 - dx;
   else e = map.xsize + dx;
   if (e == map.xsize / 2 || w == map.xsize / 2) { /* thanks, Massimo */
     if (k < 3 && s >= MAX(e, w)) return 0;
@@ -243,40 +233,65 @@ These if's might cost some CPU but hopefully less overall. -- Syela */
       if (w >= s && n >= e) return 0;
       else return 1;
     default:
-      printf("Bad dir_ok call: (%d, %d) -> (%d, %d), %d\n", x0, y0, x1, y1, k);
+      freelog(LOG_NORMAL, "Bad dir_ok call: (%d, %d) -> (%d, %d), %d",
+	      x0, y0, x1, y1, k);
       return 0;
   }
 }
 
-int could_be_my_zoc(struct unit *myunit, int x0, int y0)
-{ /* Fix to bizarre did-not-find bug.  Thanks, Katvrr -- Syela */
+/**************************************************************************
+  Similar to is_my_zoc(), but with some changes:
+  - destination (x0,y0) need not be adjacent?
+  - don't care about some directions?
+  
+  Note this function only makes sense for ground units.
+**************************************************************************/
+static int could_be_my_zoc(struct unit *myunit, int x0, int y0)
+{
+  /* Fix to bizarre did-not-find bug.  Thanks, Katvrr -- Syela */
   int ii[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
   int jj[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
   int ax, ay, k;
   int owner=myunit->owner;
 
-  if (same_pos(x0, y0, myunit->x, myunit->y)) return 0; /* can't be my zoc */
-  if (is_tiles_adjacent(x0, y0, myunit->x, myunit->y) &&
-      !is_enemy_unit_tile(x0, y0, owner)) return 0;
+  assert(is_ground_unit(myunit));
+  
+  if (same_pos(x0, y0, myunit->x, myunit->y))
+    return 0; /* can't be my zoc */
+  if (is_tiles_adjacent(x0, y0, myunit->x, myunit->y)
+      && !is_enemy_unit_tile(x0, y0, owner))
+    return 0;
 
   for (k = 0; k < 8; k++) {
-    ax = map_adjust_x(myunit->x + ii[k]); ay = map_adjust_y(myunit->y + jj[k]);
-    if (is_enemy_city_tile(ax,ay,owner))
+    ax = map_adjust_x(myunit->x + ii[k]);
+    ay = map_adjust_y(myunit->y + jj[k]);
+    
+    if (!dir_ok(x0, y0, myunit->goto_dest_x, myunit->goto_dest_y, k))
+      continue;
+    /* don't care too much about ZOC of units we've already gone past -- Syela */
+    
+    if ((map_get_terrain(ax,ay)!=T_OCEAN) && is_enemy_unit_tile(ax,ay,owner))
       return 0;
-    if (!dir_ok(x0, y0, myunit->goto_dest_x, myunit->goto_dest_y, k)) continue;
-/* don't care too much about ZOC of units we've already gone past -- Syela */
-    if (is_enemy_unit_tile(ax,ay,owner))
-      if((is_sailing_unit(myunit) && (map_get_terrain(ax,ay)==T_OCEAN)) ||
-         (!is_sailing_unit(myunit) && (map_get_terrain(ax,ay)!=T_OCEAN)) )
-        return 0;
   }
-  return -1; /* flag value */
+  
+  return 1;
+  /* return was -1 as a flag value; I've moved -1 value into
+   * could_unit_move_to_tile(), since that's where it becomes
+   * distinct/relevant --dwp
+   */
 }
 
 
+/**************************************************************************
+  this WAS can_unit_move_to_tile with the notifys removed -- Syela 
+  but is now a little more complicated to allow non-adjacent tiles
+  returns:
+    0 if can't move
+    1 if zoc_ok
+   -1 if zoc could be ok?
+**************************************************************************/
 int could_unit_move_to_tile(struct unit *punit, int x0, int y0, int x, int y)
-{ /* this WAS can_unit_move_to_tile with the notifys removed -- Syela */
-/* but is now a little more complicated to allow non-adjacent tiles */
+{
   struct tile *ptile,*ptile2;
   struct city *pcity;
 
@@ -320,24 +335,21 @@ int could_unit_move_to_tile(struct unit *punit, int x0, int y0, int x, int y)
     }
   }
 
-/*  zoc = zoc_ok_move(punit, x, y);      can't use this, so reproducing ... */
-  /* if you have units there, you can move */
-  if (is_friendly_unit_tile(x,y,punit->owner))
+  if (zoc_ok_move_gen(punit, x0, y0, x, y))
     return 1;
-  if (!is_ground_unit(punit) || unit_flag(punit->type, F_IGZOC))
-    return 1;
-  if (map_get_city(x,y) || map_get_city(x0, y0))
-    return 1;
-  if (is_my_zoc(punit, x0, y0) || is_my_zoc(punit, x, y)) return 1;
-  return (could_be_my_zoc(punit, x0, y0));
+  else if(could_be_my_zoc(punit, x0, y0))
+    return -1;	/* flag value  */
+  else
+    return 0;
 }
 
-int goto_tile_cost(struct player *pplayer, struct unit *punit, int x0, int y0, int x1, int y1, int m)
+static int goto_tile_cost(struct player *pplayer, struct unit *punit,
+			  int x0, int y0, int x1, int y1, int m)
 {
   int i;
   if (!pplayer->ai.control && !map_get_known(x1, y1, pplayer)) {
-/*    printf("Venturing into the unknown at (%d, %d).\n", x1, y1);*/
-/*    return(3);   People seemed not to like this. -- Syela */
+    freelog(LOG_DEBUG, "Venturing into the unknown at (%d, %d).", x1, y1);
+    /* return(3);   People seemed not to like this. -- Syela */
     return(15); /* arbitrary deterrent. */
   }
   if (get_defender(pplayer, punit, x1, y1)) {
@@ -345,10 +357,10 @@ int goto_tile_cost(struct player *pplayer, struct unit *punit, int x0, int y0, i
        return(MIN(m, unit_types[punit->type].move_rate));
      if (!can_unit_attack_tile(punit, x1, y1)) return(255);
      return(15);
-/* arbitrary deterrent; if we wanted to attack, we wouldn't GOTO */
+     /* arbitrary deterrent; if we wanted to attack, we wouldn't GOTO */
   } else {
     i = could_unit_move_to_tile(punit, x0, y0, x1, y1);
-    if (!i && !unit_flag(punit->type, F_IGZOC) && is_ground_unit(punit) &&
+    if (!i && !unit_really_ignores_zoc(punit) && 
         !is_my_zoc(punit, x0, y0) && !is_my_zoc(punit, x1, y1) &&
         !could_be_my_zoc(punit, x0, y0)) return(255);
     if (!i && !same_pos(x1, y1, punit->goto_dest_x, punit->goto_dest_y)) return(255);
@@ -358,7 +370,7 @@ int goto_tile_cost(struct player *pplayer, struct unit *punit, int x0, int y0, i
   return(MIN(m, unit_types[punit->type].move_rate));
 }
 
-void init_gotomap(int orig_x, int orig_y)
+static void init_gotomap(int orig_x, int orig_y)
 {
   int x, y;
   for (x = 0; x < map.xsize; x++) {
@@ -373,7 +385,7 @@ void init_gotomap(int orig_x, int orig_y)
   return;
 } 
 
-int dir_ect(int x0, int y0, int x1, int y1, int k)
+static int dir_ect(int x0, int y0, int x1, int y1, int k)
 {
   int ii[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
   int jj[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
@@ -384,13 +396,13 @@ int dir_ect(int x0, int y0, int x1, int y1, int k)
   return (1 - dir_ok(x, y, x1, y1, 7-k));
 }
 
-int find_the_shortest_path(struct player *pplayer, struct unit *punit,
-                           int dest_x, int dest_y)
+static int find_the_shortest_path(struct player *pplayer, struct unit *punit,
+				  int dest_x, int dest_y)
 {
-/*  char *d[] = { "NW", "N", "NE", "W", "E", "SW", "S", "SE" };*/
+  char *d[] = { "NW", "N", "NE", "W", "E", "SW", "S", "SE" };
   int ii[8] = { 0, 1, 2, 0, 2, 0, 1, 2 };
   int jj[8] = { 0, 0, 0, 1, 1, 2, 2, 2 };
-  int igter = 0, xx[3], yy[3], x, y, i, j, k, tm, c;
+  int igter = 0, xx[3], yy[3], x, y, x1, y1, k, tm, c;
   int orig_x, orig_y;
   struct tile *tile0;
   enum unit_move_type which = unit_types[punit->type].move_type;
@@ -431,85 +443,89 @@ and independently I can worry about optimizing them. -- Syela */
     get_from_warstack(warnodes, &x, &y);
     warnodes++; /* for debug purposes */
     tile0 = map_get_tile(x, y);
-    if((xx[2]=x+1)==map.xsize) xx[2]=0;
-    if((xx[0]=x-1)==-1) xx[0]=map.xsize-1;
-    xx[1] = x;
-    if ((yy[0]=y-1)==-1) yy[0] = 0;
-    if ((yy[2]=y+1)==map.ysize) yy[2]=y;
-    yy[1] = y;
+    map_calc_adjacent_xy(x, y, xx, yy);
     
     for (k = 0; k < 8; k++) {
-      i = ii[k]; j = jj[k]; /* saves CPU cycles? */
+      x1 = xx[ii[k]];
+      y1 = yy[jj[k]];
       if (which != SEA_MOVING) {
         if (which != LAND_MOVING) c = 3;
         else if (tile0->move_cost[k] == -3 || tile0->move_cost[k] > 16) c = maxcost; 
         else if (igter) c = (tile0->move_cost[k] ? 3 : 0); /* Reinier's fix -- Syela */
         else c = tile0->move_cost[k];
-        c = goto_tile_cost(pplayer, punit, x, y, xx[i], yy[j], c);
+        c = goto_tile_cost(pplayer, punit, x, y, x1, y1, c);
         if (!dir_ok(x, y, dest_x, dest_y, k)) c += c;
         if (!c && !dir_ect(x, y, dest_x, dest_y, k)) c = 1;
         tm = warmap.cost[x][y] + c;
 #ifdef ACTUALLYTESTED
         if (warmap.cost[x][y] < punit->moves_left && tm < maxcost &&
-            tm >= punit->moves_left - MIN(3, c) && enemies_at(punit, xx[i], yy[j])) {
+            tm >= punit->moves_left - MIN(3, c) && enemies_at(punit, x1, y1)) {
           tm += unit_types[punit->type].move_rate;
-/*printf("%s#%d@(%d,%d) dissuaded from (%d,%d) -> (%d,%d)\n",
-unit_types[punit->type].name, punit->id, punit->x, punit->y, xx[i], yy[j],
-punit->goto_dest_x, punit->goto_dest_y);*/
+	  freelog(LOG_DEBUG, "%s#%d@(%d,%d) dissuaded from (%d,%d) -> (%d,%d)",
+			unit_types[punit->type].name, punit->id,
+			punit->x, punit->y, x1, y1,
+			punit->goto_dest_x, punit->goto_dest_y);
         }
 #endif
         if (tm < maxcost) {
-          if (warmap.cost[xx[i]][yy[j]] > tm) {
-            warmap.cost[xx[i]][yy[j]] = tm;
-            add_to_stack(xx[i], yy[j]);
-            local_vector[xx[i]][yy[j]] = 128>>k;
-/*printf("Candidate: %s from (%d, %d) to (%d, %d) +%d to %d\n", d[k], x, y, xx[i], yy[j], c, tm);*/
-          } else if (warmap.cost[xx[i]][yy[j]] == tm) {
-            local_vector[xx[i]][yy[j]] |= 128>>k;
-/*printf("Co-Candidate: %s from (%d, %d) to (%d, %d) +%d to %d\n", d[k], x, y, xx[i], yy[j], c, tm);*/
+          if (warmap.cost[x1][y1] > tm) {
+            warmap.cost[x1][y1] = tm;
+            add_to_stack(x1, y1);
+            local_vector[x1][y1] = 128>>k;
+	    freelog(LOG_DEBUG,
+		    "Candidate: %s from (%d, %d) to (%d, %d) +%d to %d",
+		    d[k], x, y, x1, y1, c, tm);
+          } else if (warmap.cost[x1][y1] == tm) {
+            local_vector[x1][y1] |= 128>>k;
+	    freelog(LOG_DEBUG,
+		    "Co-Candidate: %s from (%d, %d) to (%d, %d) +%d to %d",
+		    d[k], x, y, x1, y1, c, tm);
           }
         }
       } else {
         if (tile0->move_cost[k] != -3) c = maxcost;
-        else if (unit_flag(punit->type, F_TRIREME) && !is_coastline(xx[i], yy[j])) c = 7;
+        else if (unit_flag(punit->type, F_TRIREME) && !is_coastline(x1, y1)) c = 7;
         else c = 3;
 /* I want to disable these totally but for some reason it bugs. -- Syela */
-        c = goto_tile_cost(pplayer, punit, x, y, xx[i], yy[j], c);
-        if (xx[i] == dest_x && yy[j] == dest_y && passenger && c < 60 &&
+        c = goto_tile_cost(pplayer, punit, x, y, x1, y1, c);
+        if (x1 == dest_x && y1 == dest_y && passenger && c < 60 &&
             !is_my_zoc(passenger, x, y)) c = 60; /* passenger cannot disembark */
         if (!dir_ok(x, y, dest_x, dest_y, k)) c += c;
         tm = warmap.seacost[x][y] + c;
         if (warmap.seacost[x][y] < punit->moves_left && tm < maxcost &&
-            (pplayer->ai.control || !same_pos(xx[i], yy[j], dest_x, dest_y)) &&
+            (pplayer->ai.control) &&
             tm >= punit->moves_left - (get_transporter_capacity(punit) >
             unit_types[punit->type].attack_strength ? 3 : 2) &&
-            enemies_at(punit, xx[i], yy[j])) {
+            enemies_at(punit, x1, y1)) {
           tm += unit_types[punit->type].move_rate;
-/*printf("%s#%d@(%d,%d) dissuaded from (%d,%d) -> (%d,%d)\n",
-unit_types[punit->type].name, punit->id, punit->x, punit->y, xx[i], yy[j],
-punit->goto_dest_x, punit->goto_dest_y);*/
+	  freelog(LOG_DEBUG, "%s#%d@(%d,%d) dissuaded from (%d,%d) -> (%d,%d)",
+		  unit_types[punit->type].name, punit->id,
+		  punit->x, punit->y, x1, y1,
+		  punit->goto_dest_x, punit->goto_dest_y);
         }
         if (tm < maxcost) {
-          if (warmap.seacost[xx[i]][yy[j]] > tm) {
-            warmap.seacost[xx[i]][yy[j]] = tm;
-            add_to_stack(xx[i], yy[j]);
-            local_vector[xx[i]][yy[j]] = 128>>k;
-          } else if (warmap.seacost[xx[i]][yy[j]] == tm) {
-            local_vector[xx[i]][yy[j]] |= 128>>k;
+          if (warmap.seacost[x1][y1] > tm) {
+            warmap.seacost[x1][y1] = tm;
+            add_to_stack(x1, y1);
+            local_vector[x1][y1] = 128>>k;
+          } else if (warmap.seacost[x1][y1] == tm) {
+            local_vector[x1][y1] |= 128>>k;
           }
         }
       }
-      if (xx[i] == dest_x && yy[j] == dest_y && maxcost > tm) {
-/*printf("Found path, cost = %d\n", tm);*/
+      if (x1 == dest_x && y1 == dest_y && maxcost > tm) {
+	freelog(LOG_DEBUG, "Found path, cost = %d", tm);
         maxcost = tm + 1; /* NOT = tm.  Duh! -- Syela */
       }
     } /* end for */
   } while (warstacksize > warnodes);
-/*printf("GOTO: (%d, %d) -> (%d, %d), %u nodes, cost = %d\n", 
-orig_x, orig_y, dest_x, dest_y, warnodes, maxcost - 1);*/
+  
+  freelog(LOG_DEBUG, "GOTO: (%d, %d) -> (%d, %d), %u nodes, cost = %d", 
+		orig_x, orig_y, dest_x, dest_y, warnodes, maxcost - 1);
   if (maxcost == 255) return(0);
-/* succeeded.  the vector at the destination indicates which way we get there */
-/* backtracing */
+  /* succeeded.  the vector at the destination indicates which
+     way we get there */
+  /* backtracing */
   warnodes = 0;
   warstacksize = 0;
   add_to_stack(dest_x, dest_y);
@@ -518,31 +534,28 @@ orig_x, orig_y, dest_x, dest_y, warnodes, maxcost - 1);*/
     get_from_warstack(warnodes, &x, &y);
     warnodes++; /* for debug purposes */
     tile0 = map_get_tile(x, y);
-    if((xx[2]=x+1)==map.xsize) xx[2]=0;
-    if((xx[0]=x-1)==-1) xx[0]=map.xsize-1;
-    xx[1] = x;
-    if ((yy[0]=y-1)==-1) yy[0] = 0;
-    if ((yy[2]=y+1)==map.ysize) yy[2]=y;
-    yy[1] = y;
+    map_calc_adjacent_xy(x, y, xx, yy);
 
     for (k = 0; k < 8; k++) {
-      i = ii[k]; j = jj[k]; /* saves CPU cycles? */
+      x1 = xx[ii[k]];
+      y1 = yy[jj[k]];
       if (local_vector[x][y] & (1<<k)) {
-/* && (local_vector[xx[i]][yy[j]] || !warmap.vector[xx[i]][yy[j]])) {
+/* && (local_vector[x1][y1] || !warmap.vector[x1][y1])) {
 is not adequate to prevent RR loops.  Bummer. -- Syela */
-        add_to_stack(xx[i], yy[j]);
-        warmap.vector[xx[i]][yy[j]] |= 128>>k;
+        add_to_stack(x1, y1);
+        warmap.vector[x1][y1] |= 128>>k;
         local_vector[x][y] -= 1<<k; /* avoid repetition */
-/*printf("PATH-SEGMENT: %s from (%d, %d) to (%d, %d)\n", d[7-k], xx[i], yy[j], x, y);*/
+	freelog(LOG_DEBUG, "PATH-SEGMENT: %s from (%d, %d) to (%d, %d)",
+		d[7-k], x1, y1, x, y);
       }
     }
   } while (warstacksize > warnodes);
-/*printf("BACKTRACE: %u nodes\n", warnodes);*/
+  freelog(LOG_DEBUG, "BACKTRACE: %u nodes", warnodes);
   return(1);
-/* DONE! */
+  /* DONE! */
 }
 
-int find_a_direction(struct unit *punit)
+static int find_a_direction(struct unit *punit)
 {
   int k, d[8], x, y, n, a, best = 0, d0, d1, h0, h1, u, c;
   int ii[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
@@ -565,10 +578,12 @@ int find_a_direction(struct unit *punit)
       else c = 3;
       if (unit_flag(punit->type, F_IGTER) && c) c = 1;
       x = map_adjust_x(punit->x + ii[k]); y = map_adjust_y(punit->y + jj[k]);
-/*if (passenger) printf("%d@(%d,%d) evaluating (%d,%d)[%d/%d]\n",
-punit->id, punit->x, punit->y, x, y, c, punit->moves_left);*/
+      if (passenger) {
+	freelog(LOG_DEBUG, "%d@(%d,%d) evaluating (%d,%d)[%d/%d]",
+		punit->id, punit->x, punit->y, x, y, c, punit->moves_left);
+      }
       ptile = map_get_tile(x, y);
-      d0 = get_virtual_defense_power(U_HOWITZER, punit->type, x, y);
+      d0 = get_simple_defense_power(punit->type, x, y);
       pcity = map_get_city(x, y);
       n = 2;
       if (pcity) { /* this code block inspired by David Pfitzner -- Syela */
@@ -583,7 +598,7 @@ punit->id, punit->x, punit->y, x, y, c, punit->moves_left);*/
         if (aunit->owner != punit->owner) d1 = -1; /* MINIMUM priority */
         else {
           u++;
-          a = get_virtual_defense_power(U_HOWITZER, aunit->type, x, y) * n / 2;
+          a = get_simple_defense_power(aunit->type, x, y) * n / 2;
           if (a * aunit->hp > d1 * h1) { d1 = a; h1 = aunit->hp; }
         }
       unit_list_iterate_end;
@@ -603,7 +618,7 @@ punit->id, punit->x, punit->y, x, y, c, punit->moves_left);*/
         adjtile = map_get_tile(x + ii[n], y + jj[n]);
         if (adjtile->terrain != T_OCEAN) nearland++;
         if (!((adjtile->known)&(1u<<punit->owner))) {
-          if (punit->moves_left <= c) d[k] -= (d[k]>>4); /* Avoid the unknown */
+          if (punit->moves_left <= c) d[k] -= (d[k]/16); /* Avoid the unknown */
           else d[k]++; /* nice but not important */
         } else { /* NOTE: Not being omniscient here!! -- Syela */
           unit_list_iterate(adjtile->units, aunit) /* lookin for trouble */
@@ -633,7 +648,8 @@ punit->id, punit->x, punit->y, x, y, c, punit->moves_left);*/
          !punit->ai.passenger || punit->moves_left >= 6)) d[k] = 1;
       if (d[k] > best) { 
         best = d[k];
-/*printf("New best = %d: (%d, %d) -> (%d, %d)\n", best, punit->x, punit->y, x, y);*/
+	freelog(LOG_DEBUG, "New best = %d: (%d, %d) -> (%d, %d)",
+		best, punit->x, punit->y, x, y);
       }
     } /* end is-a-valid-vector */
   } /* end for */
@@ -694,7 +710,7 @@ void do_unit_goto(struct player *pplayer, struct unit *punit)
   int x, y, k;
   int ii[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
   int jj[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
-/*  char *d[] = { "NW", "N", "NE", "W", "E", "SW", "S", "SE" };*/
+  char *d[] = { "NW", "N", "NE", "W", "E", "SW", "S", "SE" };
 
   int id=punit->id;
 
@@ -724,38 +740,41 @@ different but should still pre-empt calculation of impossible GOTO's. -- Syela *
       if(!punit->moves_left) return;
       k = find_a_direction(punit);
       if (k < 0) {
-/*printf("%s#%d@(%d,%d) stalling so it won't be killed.\n",
-unit_types[punit->type].name, punit->id, punit->x, punit->y);*/
-/*        punit->activity=ACTIVITY_IDLE;*/
+	freelog(LOG_DEBUG, "%s#%d@(%d,%d) stalling so it won't be killed.",
+		unit_types[punit->type].name, punit->id,
+		punit->x, punit->y);
+	/* punit->activity=ACTIVITY_IDLE;*/
 	send_unit_info(0, punit, 0);
 	return;
       }
-/*printf("Going %s\n", d[k]);*/
+      freelog(LOG_DEBUG, "Going %s", d[k]);
       x = map_adjust_x(punit->x + ii[k]);
       y = punit->y + jj[k]; /* no need to adjust this */
 
       if(!punit->moves_left) return;
       if(!handle_unit_move_request(pplayer, punit, x, y)) {
-/*printf("Couldn't handle it.\n");*/
+	freelog(LOG_DEBUG, "Couldn't handle it.");
 	if(punit->moves_left) {
 	  punit->activity=ACTIVITY_IDLE;
 	  send_unit_info(0, punit, 0);
 	  return;
 	};
-      } /*else printf("Handled.\n");*/
+      } else {
+	freelog(LOG_DEBUG, "Handled.");
+      }
       if(!unit_list_find(&pplayer->units, id))
 	return; /* unit died during goto! */
 
       if(punit->x!=x || punit->y!=y) {
-/*printf("Aborting, out of movepoints.\n");*/
+	freelog(LOG_DEBUG, "Aborting, out of movepoints.");
 	send_unit_info(0, punit, 0);
 	return; /* out of movepoints */
       }
-/*printf("Moving on.\n");*/
+      freelog(LOG_DEBUG, "Moving on.");
     } while(!(x==punit->goto_dest_x && y==punit->goto_dest_y));
   }
   else {
-    flog(LOG_DEBUG, "Did not find the shortest path for "
+    freelog(LOG_VERBOSE, "Did not find the shortest path for "
 	 "%s's %s at (%d, %d) -> (%d, %d)",
 	 pplayer->name, unit_types[punit->type].name,
 	 punit->x, punit->y, punit->goto_dest_x, punit->goto_dest_y);
@@ -764,3 +783,45 @@ unit_types[punit->type].name, punit->id, punit->x, punit->y);*/
   punit->activity=ACTIVITY_IDLE;
   send_unit_info(0, punit, 0);
 }
+
+
+/**************************************************************************
+Calculate and return cost (in terms of move points) for unit to move
+to specified destination.  
+**************************************************************************/
+int calculate_move_cost(struct player *pplayer, struct unit *punit,
+			int dest_x, int dest_y) {
+  /* perhaps we should do some caching -- fisch */
+    
+  /*
+  static struct unit *last_punit = NULL;
+
+  if(last_punit != punit) {
+    generate_warmap(NULL,punit);
+  }
+  */
+
+  if (is_air_unit(punit) || is_heli_unit(punit)) {
+    /* The warmap only really knows about land and sea
+       units, so for these we just assume cost = distance.
+       (times 3 for road factor).
+       (Could be wrong if there are things in the way.)
+    */
+    return 3 * real_map_distance(punit->x, punit->y, dest_x, dest_y);
+  }
+
+  generate_warmap(NULL,punit);
+  
+  if(is_sailing_unit(punit))
+    return warmap.seacost[dest_x][dest_y];
+  else if (is_ground_unit(punit)) {
+    return warmap.cost[dest_x][dest_y];
+  } else {
+    return warmap.cost[dest_x][dest_y] < warmap.seacost[dest_x][dest_y] ?
+      warmap.cost[dest_x][dest_y]
+      : warmap.seacost[dest_x][dest_y];
+  }
+    
+}
+
+
