@@ -10,10 +10,11 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
-#ifndef __MAPHAND_H
-#define __MAPHAND_H
+#ifndef FC__MAPHAND_H
+#define FC__MAPHAND_H
+
+#include "map.h"
 
-#include <map.h>
 struct player;
 struct section_file;
 
@@ -28,9 +29,12 @@ void light_square(struct player *pplayer, int x, int y, int len);
 void send_map_info(struct player *dest);
 void global_warming(int effect);
 
+void upgrade_city_rails(struct player *pplayer, int discovery);
+
 void map_save(struct section_file *file);
 void map_startpos_load(struct section_file *file);
 void map_tiles_load(struct section_file *file);
 void map_load(struct section_file *file);
+void map_rivers_overlay_load(struct section_file *file);
 
-#endif
+#endif  /* FC__MAPHAND_H */
