@@ -10,29 +10,38 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 
-#include <player.h>
-#include <unithand.h>
-#include <packets.h>
-#include <civserver.h>
-#include <map.h>
-#include <maphand.h>
-#include <cityhand.h>
-#include <citytools.h>
-#include <cityturn.h>
-#include <unit.h>
-#include <plrhand.h>
-#include <city.h>
-#include <log.h>
-#include <mapgen.h>
-#include <events.h>
-#include <shared.h>
-#include <aiunit.h>
-#include <unittools.h>
-#include <unitfunc.h>
+#include "city.h"
+#include "events.h"
+#include "fcintl.h"
+#include "government.h"
+#include "log.h"
+#include "map.h"
+#include "packets.h"
+#include "player.h"
+#include "shared.h"
+#include "unit.h"
+
+#include "cityhand.h"
+#include "citytools.h"
+#include "cityturn.h"
+#include "civserver.h"
+#include "maphand.h"
+#include "plrhand.h"
+#include "unitfunc.h"
+#include "unithand.h"
+
+#include "aiunit.h"
+
+#include "unittools.h"
 
 /**************************************************************************
   unit can be moved if:
@@ -73,8 +82,18 @@ int can_unit_move_to_tile(struct unit *punit, int x, int y)
     /* Moving from ocean */
     if(ptile2->terrain==T_OCEAN) {
       /* Can't attack a city from ocean unless marines */
-      if(!unit_flag(punit->type, F_MARINES) && is_enemy_city_tile(x,y,punit->owner)) {
-        notify_player_ex(&game.players[punit->owner], punit->x, punit->y, E_NOEVENT, "Game: Only Marines can attack from sea.");
+      if(!unit_flag(punit->type, F_MARINES)
+	 && is_enemy_city_tile(x,y,punit->owner)) {
+	char *units_str = get_units_with_flag_string(F_MARINES);
+	if (units_str) {
+	  notify_player_ex(&game.players[punit->owner], punit->x, punit->y,
+			   E_NOEVENT, _("Game: Only %s can attack from sea."),
+			   units_str);
+	  free(units_str);
+	} else {
+	  notify_player_ex(&game.players[punit->owner], punit->x, punit->y,
+			   E_NOEVENT, _("Game: Cannot attack from sea."));
+	}
 	return 0;
       }
     }
@@ -86,7 +105,8 @@ int can_unit_move_to_tile(struct unit *punit, int x, int y)
   zoc = zoc_ok_move(punit, x, y);
   if (!zoc) 
     notify_player_ex(&game.players[punit->owner], punit->x, punit->y, E_NOEVENT,
-    "Game: %s can only move into your own zone of control.", unit_types[punit->type].name);
+		     _("Game: %s can only move into your own zone of control."),
+		     unit_types[punit->type].name);
   return zoc;
 }
 
@@ -149,7 +169,13 @@ int is_friendly_city_tile(int x, int y, int owner)
 }
 
 /**************************************************************************
-  is this square controlled by the units owner
+  Is this square controlled by the unit's owner?
+
+  Here "is_my_zoc" means essentially a square which is
+  *not* adjacent to an enemy unit on a land tile.
+  (Or, currently, an enemy city even if empty.)
+
+  Note this function only makes sense for ground units.
 **************************************************************************/
 int is_my_zoc(struct unit *myunit, int x0, int y0)
 {
@@ -157,42 +183,48 @@ int is_my_zoc(struct unit *myunit, int x0, int y0)
   int ax,ay;
   int owner=myunit->owner;
 
+  assert(is_ground_unit(myunit));
+     
   for (x=x0-1;x<x0+2;x++) for (y=y0-1;y<y0+2;y++) {
-    ax=map_adjust_x(x); ay=map_adjust_y(y);
-    if (is_enemy_city_tile(ax,ay,owner))
+    ax=map_adjust_x(x);
+    ay=map_adjust_y(y);
+    if ((map_get_terrain(ax,ay)!=T_OCEAN) && is_enemy_unit_tile(ax,ay,owner))
       return 0;
-    if (is_enemy_unit_tile(ax,ay,owner))
-      if((is_sailing_unit(myunit) && (map_get_terrain(ax,ay)==T_OCEAN)) ||
-	 (!is_sailing_unit(myunit) && (map_get_terrain(ax,ay)!=T_OCEAN)) )
-        return 0;
   }
   return 1;
 }
 
 /**************************************************************************
-  return whether or not the square the unit wants to enter is blocked by
-  enemy units? 
+  Returns whether the unit is allowed (by ZOC) to move from (x1,y1)
+  to (x2,y2) (assumed adjacent).
   You CAN move if:
   1. You have units there already
   2. Your unit isn't a ground unit
   3. Your unit ignores ZOC (diplomat, freight, etc.)
   4. You're moving from or to a city
-  5. The spot you're moving from or to is in your ZOC
+  5. You're moving from an ocean square (from a boat)
+  6. The spot you're moving from or to is in your ZOC
 **************************************************************************/
-int zoc_ok_move(struct unit *punit,int x, int y)
+int zoc_ok_move_gen(struct unit *punit, int x1, int y1, int x2, int y2)
 {
-  struct unit_list *punit_list;
+  if (unit_really_ignores_zoc(punit))  /* !ground_unit or IGZOC */
+    return 1;
+  if (is_friendly_unit_tile(x2, y2, punit->owner))
+    return 1;
+  if (map_get_city(x1, y1) || map_get_city(x2, y2))
+    return 1;
+  if (map_get_terrain(x1,y1)==T_OCEAN)
+    return 1;
+  return (is_my_zoc(punit, x1, y1) || is_my_zoc(punit, x2, y2)); 
+}
 
-  punit_list=&map_get_tile(x, y)->units;
-  /* if you have units there, you can move */
-  if (is_friendly_unit_tile(x,y,punit->owner))
-    return 1;
-  if (!is_ground_unit(punit) || unit_flag(punit->type, F_IGZOC))
-    return 1;
-  if (map_get_city(x,y) || map_get_city(punit->x, punit->y))
-    return 1;
-  return(is_my_zoc(punit, punit->x, punit->y) || 
-         is_my_zoc(punit, x, y)); 
+/**************************************************************************
+  Convenience wrapper for zoc_ok_move_gen(), using the unit's (x,y)
+  as the starting point.
+**************************************************************************/
+int zoc_ok_move(struct unit *punit, int x, int y)
+{
+  return zoc_ok_move_gen(punit, punit->x, punit->y, x, y);
 }
 
 /**************************************************************************
@@ -205,6 +237,7 @@ int zoc_ok_move(struct unit *punit,int x, int y)
 **************************************************************************/
 int unit_bribe_cost(struct unit *punit)
 {  
+  struct government *g = get_gov_iplayer(punit->owner);
   int cost;
   struct city *capital;
   int dist;
@@ -212,12 +245,14 @@ int unit_bribe_cost(struct unit *punit)
 
   cost = (&game.players[punit->owner])->economic.gold+750;
   capital=find_palace(&game.players[punit->owner]);
-  if (capital)
-    dist=min(32, map_distance(capital->x, capital->y, punit->x, punit->y)); 
+  if (capital) {
+    int tmp = map_distance(capital->x, capital->y, punit->x, punit->y);
+    dist=MIN(32, tmp);
+  }
   else
     dist=32;
-    if (get_government(punit->owner)==G_COMMUNISM)
-      dist = min(10, dist);
+  if (g->fixed_corruption_distance)
+    dist = MIN(g->fixed_corruption_distance, dist);
   cost=(cost/(dist+2))*(get_unit_type(punit->type)->build_cost/10);
   if (unit_flag(punit->type, F_SETTLERS)) 
     cost/=2;
@@ -262,7 +297,8 @@ int hp_gain_coord(struct unit *punit)
   else
     hp=0;
   if((pcity=map_get_city(punit->x,punit->y))) {
-    if ((city_got_barracks(pcity) && is_ground_unit(punit)) ||
+    if ((city_got_barracks(pcity) &&
+	 (is_ground_unit(punit) || improvement_variant(B_BARRACKS)==1)) ||
 	(city_got_building(pcity, B_AIRPORT) && is_air_unit(punit)) || 
 	(city_got_building(pcity, B_AIRPORT) && is_heli_unit(punit)) || 
 	(city_got_building(pcity, B_PORT) && is_sailing_unit(punit))) {
@@ -271,9 +307,7 @@ int hp_gain_coord(struct unit *punit)
     else
       hp=get_unit_type(punit->type)->hp/3;
   }
-  else if (is_heli_unit(punit))
-    hp-=(get_unit_type(punit->type)->hp/10);
-  else
+  else if (!is_heli_unit(punit))
     hp++;
 
   if(punit->activity==ACTIVITY_FORTIFY)
@@ -306,19 +340,22 @@ struct unit *get_defender(struct player *pplayer, struct unit *aunit,
 			  int x, int y)
 {
   struct unit *bestdef = 0;
-  int bestvalue=-1;
-int ct=0;
+  int unit_d, bestvalue=-1, ct=0;
+
   unit_list_iterate(map_get_tile(x, y)->units, punit) {
     if (pplayer->player_no==punit->owner)
       return 0;
-ct++;
-    if(unit_can_defend_here(punit) && rate_unit_d(punit, aunit)>bestvalue) {
-      bestvalue=rate_unit_d(punit, aunit);
-      bestdef=punit;
+    ct++;
+    if (unit_can_defend_here(punit)) {
+      unit_d = rate_unit_d(punit, aunit);
+      if (unit_d > bestvalue) {
+	bestvalue = unit_d;
+	bestdef = punit;
+      }
     }
   }
   unit_list_iterate_end;
-if(ct&&!bestdef)printf("Get_def bugged at (%d,%d)\n", x, y);
+  if(ct&&!bestdef) freelog(LOG_NORMAL, "Get_def bugged at (%d,%d)", x, y);
   return bestdef;
 }
 
@@ -343,12 +380,13 @@ struct unit *get_attacker(struct player *pplayer, struct unit *aunit,
 			  int x, int y)
 { /* get unit at (x, y) that wants to kill aunit */
   struct unit *bestatt = 0;
-  int bestvalue=-1;
+  int bestvalue=-1, unit_a;
   unit_list_iterate(map_get_tile(x, y)->units, punit) {
     if (pplayer->player_no==punit->owner)
       return 0;
-    if(rate_unit_a(punit, aunit)>bestvalue) {
-      bestvalue=rate_unit_a(punit, aunit);
+    unit_a=rate_unit_a(punit, aunit);
+    if(unit_a>bestvalue) {
+      bestvalue=unit_a;
       bestatt=punit;
     }
   }
@@ -379,17 +417,32 @@ int get_defense_power(struct unit *punit)
 {
   int power;
   int terra;
-  if (!punit || punit->type<0 || punit->type>=U_LAST)
+  int db;
+
+  if (!punit || punit->type<0 || punit->type>=U_LAST
+      || punit->type>=game.num_unit_types)
     abort();
   power=get_unit_type(punit->type)->defense_strength*10;
   if (punit->veteran)
     power*=1.5;
   
   terra=map_get_terrain(punit->x, punit->y);
-  power=(power*get_tile_type(terra)->defense_bonus)/10;
+  db = get_tile_type(terra)->defense_bonus;
+  if (map_get_special(punit->x, punit->y) & S_RIVER)
+    db += (db * terrain_control.river_defense_bonus) / 100;
+  power=(power*db)/10;
+
   return power;
 }
 
+/**************************************************************************
+  Takes into account unit move_type as well as IGZOC
+**************************************************************************/
+int unit_really_ignores_zoc(struct unit *punit)
+{
+  return (!is_ground_unit(punit)) || (unit_flag(punit->type, F_IGZOC));
+}
+
 /**************************************************************************
   a wrapper that returns whether or not a unit ignores citywalls
 **************************************************************************/
@@ -398,6 +451,16 @@ int unit_ignores_citywalls(struct unit *punit)
   return (unit_flag(punit->type, F_IGWALL));
 }
 
+/**************************************************************************
+  Takes into account unit move_type as well, and Walls variant.
+**************************************************************************/
+int unit_really_ignores_citywalls(struct unit *punit)
+{
+  return unit_ignores_citywalls(punit)
+    || is_air_unit(punit)
+    || (is_sailing_unit(punit) && !(improvement_variant(B_CITY)==1));
+}
+
 /**************************************************************************
  a wrapper function that returns whether or not the unit is on a citysquare
  with citywalls
@@ -471,34 +534,22 @@ struct city *sdi_defense_close(int owner, int x, int y)
 
 /**************************************************************************
   returns a unit type for the goodie huts
-  Note: it's possible to get Gunpowder before Chivalry;
-  Also note: myrand(num) returns 0 to (num-1) inclusive
 **************************************************************************/
-int find_a_unit_type()
- {
-  int num;
-  
-  num = 3;
-  if (game.global_advances[A_CHIVALRY])
-    num++;
-  if (game.global_advances[A_GUNPOWDER]) 
-    num++;
+int find_a_unit_type(void)
+{
+  int which[U_LAST];
+  int i, num=0, iunit;
 
-  switch (myrand(num)) {
-  case 0:
-    return U_LEGION;
-  case 1:
-    return U_HORSEMEN;
-  case 2:
-    return U_CHARIOT;
-  case 3:
-    return (game.global_advances[A_CHIVALRY]) ? U_KNIGHTS : U_MUSKETEERS;
-  case 4:
-    return U_MUSKETEERS;
-  default:
-    printf("problem in find_a_unit_type\n");
-    return U_HORSEMEN;
+  for(i=0; i<num_role_units(L_HUT); i++) {
+    which[num++] = get_role_unit(L_HUT, i);
   }
+  for(i=0; i<num_role_units(L_HUT_TECH); i++) {
+    iunit = get_role_unit(L_HUT_TECH, i);
+    if (game.global_advances[get_unit_type(iunit)->tech_requirement]) {
+      which[num++] = iunit;
+    }
+  }
+  return which[myrand(num)];
 }
 
 /**************************************************************************
@@ -512,7 +563,7 @@ int can_unit_attack_unit_at_tile(struct unit *punit, struct unit *pdefender, int
 {
   int fromtile=map_get_terrain(punit->x, punit->y);
   int totile=map_get_terrain(dest_x, dest_y);
-
+ 
   if(!is_military_unit(punit))
     return 0;
 
@@ -572,9 +623,12 @@ int can_place_partisan(int x, int y)
 int enemies_at(struct unit *punit, int x, int y)
 {
   int i, j, a = 0, d;
+  int db;
   struct player *pplayer = get_player(punit->owner);
-  d = unit_vulnerability_virtual(punit) *
-      get_tile_type(map_get_terrain(x, y))->defense_bonus;
+  db = get_tile_type(map_get_terrain(x, y))->defense_bonus;
+  if (map_get_special(x, y) & S_RIVER)
+    db += (db * terrain_control.river_defense_bonus) / 100;
+  d = unit_vulnerability_virtual(punit) * db;
   if (is_friendly_city_tile(x, y, punit->owner)) return 0;
   for (j = y - 1; j <= y + 1; j++) {
     if (j < 0 || j >= map.ysize) continue;
@@ -593,3 +647,127 @@ int enemies_at(struct unit *punit, int x, int y)
   }
   return 0; /* as good a quick'n'dirty should be -- Syela */
 }
+
+
+
+/**************************************************************************
+  Return first unit on square that is not owned by player.
+
+                          - Kris Bubendorfer 
+**************************************************************************/
+
+struct unit *is_enemy_unit_on_tile(int x, int y, int owner)
+{
+  unit_list_iterate(map_get_tile(x, y)->units, punit) 
+    if (owner!=punit->owner) 
+      return punit;
+  unit_list_iterate_end;
+  
+  return 0;
+}
+
+/**************************************************************************
+Teleport punit to city at cost specified.  Returns success.
+                         - Kris Bubendorfer
+**************************************************************************/
+
+int teleport_unit_to_city(struct unit *punit, struct city *pcity, int mov_cost)
+{
+  if(pcity->owner == punit->owner){
+    unit_list_unlink(&map_get_tile(punit->x, punit->y)->units, punit);
+    punit->x = pcity->x;
+    punit->y = pcity->y;
+    if(punit->moves_left < mov_cost)
+      punit->moves_left = 0;
+    else
+      punit->moves_left -= mov_cost;
+    
+    unit_list_insert(&map_get_tile(pcity->x, pcity->y)->units, punit);
+    send_unit_info(0, punit, 1);
+
+    return 1;
+  }
+  return 0;
+}
+
+/**************************************************************************
+  Resolve unit stack
+
+  When in civil war (or an alliance breaks) there will potentially be units 
+  from both sides coexisting on the same squares.  This routine resolves 
+  this by teleporting the units in multiowner stacks to the closest city.
+
+  That is, if a unit is closer to it's city than the coexistent enemy unit
+  then the enemy unit is teleported home.
+
+                         - Kris Bubendorfer
+
+  If verbose is true, the unit owner gets messages about where each
+  units goes.  --dwp
+**************************************************************************/
+
+void resolve_unit_stack(int x, int y, int verbose)
+{
+  struct unit *punit = unit_list_get(&map_get_tile(x, y)->units, 0);
+  struct unit *cunit = is_enemy_unit_on_tile(x, y, punit->owner);
+  
+  while(punit && cunit){
+    struct city *pcity = find_closest_owned_city(get_player(punit->owner), x, y);
+    struct city *ccity = find_closest_owned_city(get_player(cunit->owner), x, y);
+    
+    if(map_distance(x, y, pcity->x, pcity->y) 
+       < map_distance(x, y, ccity->x, ccity->y)){
+      teleport_unit_to_city(cunit, ccity, 0);
+      freelog(LOG_VERBOSE, "Teleported %s's %s from (%d, %d) to %s",
+	      get_player(cunit->owner)->name, unit_name(cunit->type),
+	      x, y,ccity->name);
+      if (verbose) {
+	notify_player(get_player(cunit->owner),
+		      _("Game: Teleported your %s from (%d, %d) to %s."),
+		      unit_name(cunit->type), x, y,ccity->name);
+      }
+    }else{
+      teleport_unit_to_city(punit, pcity, 0);
+      freelog(LOG_VERBOSE, "Teleported %s's %s from (%d, %d) to %s",
+	      get_player(punit->owner)->name, unit_name(punit->type),
+	      x, y, pcity->name);
+      if (verbose) {
+	notify_player(get_player(punit->owner),
+		      _("Game: Teleported your %s from (%d, %d) to %s."),
+		      unit_name(punit->type), x, y, pcity->name);
+      }
+    }
+    
+    punit = unit_list_get(&map_get_tile(x, y)->units, 0);
+    cunit = is_enemy_unit_on_tile(x, y, punit->owner);
+  }
+
+  /* There is only one players units left on this square.  If there is not 
+     enough transporter capacity left , send surplus to the closest friendly 
+     city. */
+
+  if(!punit && cunit)
+    punit = cunit;
+  
+  if(punit && map_get_terrain(punit->x, punit->y)==T_OCEAN && 
+     !is_enough_transporter_space(get_player(punit->owner), x, y)){    
+    unit_list_iterate(map_get_tile(x, y)->units, vunit){
+      struct city *vcity = find_closest_owned_city(get_player(vunit->owner), x, y);
+      if(is_ground_unit(vunit)){
+	teleport_unit_to_city(vunit, vcity, 0);	  
+	freelog(LOG_VERBOSE, "Teleported  %s's %s to %s"
+		" as there is no transport space on square (%d, %d)",
+		get_player(vunit->owner)->name, unit_name(vunit->type),
+		vcity->name, x, y);
+	if (verbose) {
+	  notify_player(get_player(vunit->owner),
+			_("Game: Teleported your %s to %s as there is"
+			  " no transport space on square (%d, %d)."),
+			unit_name(vunit->type), vcity->name, x, y);
+	}
+      }
+    }
+    unit_list_iterate_end;
+  }
+}
+
